SystemOrganization addCategory: #'Golgi-Basics'!
SystemOrganization addCategory: #'Golgi-Helpers'!
SystemOrganization addCategory: #'Golgi-Tests'!

!MenuMorph methodsFor: '*golgi' stamp: 'miso 10/18/1999 08:46'!
addSmartList: listOfDescriptors
"add a list of menuItemMorphs, where each menuItem is described by a descriptor in listOfDescriptors, possibly recursively. Descriptors can have several forms:
	(menuString selectorSymbol)
	(menuString selectorSymbol argument)
	(menuString selectorSymbol argumentList)
	'-'
	(menuString #submenu selectorForMethodBuildingSubmenuInTarget) 		_ here '#submenu' is a keyword
	(menuString descriptorOfSubItem descriptorOfSubItem ... )
Where the fourth puts a separator in the menu, and the last two build submenus, either via a method or via a recursive array"


	listOfDescriptors do: [: descr | self privParseMenuItemDescriptor: descr]! !

!MenuMorph methodsFor: '*golgi' stamp: 'miso 10/29/1999 13:56'!
privParseMenuItemDescriptor: aDescriptor
"see comment in addList:"

	(aDescriptor = #-) ifTrue: [^ self addLine].

	(aDescriptor second = #subMenu) ifTrue: [ "a submenu constructed by a method in the target" 
		^ self add: aDescriptor first  subMenu: 
			((MenuMorph new defaultTarget: defaultTarget) addSmartList: (defaultTarget perform: aDescriptor last))].

	(aDescriptor second isKindOf: Array) ifTrue: [ "a submenu specified in array"
		^ self	add: aDescriptor first 
				subMenu: ((MenuMorph new defaultTarget: defaultTarget) addSmartList: (aDescriptor copyWithout: aDescriptor first))].

	aDescriptor size = 2 ifTrue: [^ self add: aDescriptor first action: aDescriptor last].
	aDescriptor size = 3 ifTrue: [ "has arguments, either just one, or a list of them"
		^ (aDescriptor last isKindOf: Array)
			ifTrue: [self add: aDescriptor first  target: defaultTarget  
						selector: aDescriptor second   argumentList: aDescriptor last]
			ifFalse: [self add: aDescriptor first selector: aDescriptor second 
						argument: aDescriptor last] ]! !

!MenuMorph methodsFor: '*golgi-override' stamp: 'miso 10/28/1999 13:48'!
setInvokingView: invokingView
	"Re-work every menu item of the form
		<target> perform: <selector>
	to the form
		<target> perform: <selector> orSendTo: <invokingView>.
	This supports MVC's vectoring of non-model messages to the editPane."
	self items do:
		[:item |
		item arguments ifNotNil: [
			item arguments isEmpty ifTrue:  "only the simple messages"
				[item arguments: (Array with: item selector with: invokingView).
				item selector: #perform:orSendTo:]]]! !

TextMorphForEditView subclass: #GolgiTextMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Golgi-Basics'!

!GolgiTextMorph commentStamp: 'miso 1/12/2005 19:33' prior: 0!
I am the typical content of a header. I override TextMorphForEditView in order to catch getting focus (#keyboardFocusChange:) so that I can tell golgi about it. This allows us to keep a single focus per golgi, despite having multiple textMorphs. 

Since this subclass was needed anyway, I add two convenience funcationalities: 
 I override keyboard handling (#keyStroke:), so as to install hooks for special keys dealt with by my owners [if this class doesn't exist, it can be done in a hackish way by implementing #scrollByKeyboard: in the header, and returning true if the key was handled]
 I add #quickContents: instead of putting it as a golgi extention in TextMorph
!

!GolgiTextMorph methodsFor: 'special keys' stamp: 'miso 12/28/2004 23:47'!
key: char combo: comboSymbol stream: s
"if return true/false means action is done, but if return nil, means action was not done and somebody lower in the foodchain will take a shot at it (ultimately, the editor will)"

	"filter cmdTab to be a normal tab"
	(comboSymbol = #cmdShift and: [char = Character tab]) ifTrue: [
			^ editor normalCharacter: s].
	^ editView key: char combo: comboSymbol stream: s.
! !

!GolgiTextMorph methodsFor: 'special keys' stamp: 'miso 1/2/2005 16:45'!
keyStroke: event

| combo  |
combo := self keyStrokeIsModified: event.
(combo notNil or: [self keyStrokeIsSpecial: event]) ifTrue: [
	(self key: event keyValue combo: combo stream: nil) ifNotNil: [^self]].

"if no modifiers or special keys, let it proceed as usual"
super keyStroke: event! !

!GolgiTextMorph methodsFor: 'special keys' stamp: 'miso 12/28/2004 23:44'!
keyStrokeIsModified: evt
"look for cmd/cmdShift. Answer with a modifier keys descriptor : #cmd, #cmdShift, nil"

	evt commandKeyPressed ifTrue: [
		evt shiftPressed
			ifTrue: [^ #cmdShift]
			ifFalse: [^ #cmd]].

	"the control key can be used to invoke shift-cmd shortcuts"
	evt controlKeyPressed ifTrue: [^ #cmdShift].

	^ nil! !

!GolgiTextMorph methodsFor: 'special keys' stamp: 'miso 12/28/2004 23:45'!
keyStrokeIsSpecial: evt
"Answer whether this is a special key-evt that is a command even without modifier"

	"enter, backspace, arrows, escape, etc. are command keys"
	(ParagraphEditor specialShiftCmdKeys includes: evt keyCharacter asciiValue)
		ifTrue: [^ true].

	"-=-= special for golgi, until it is put into specialChars"
	evt keyCharacter = Character tab 
		ifTrue: [^ true].

	^ false! !

!GolgiTextMorph methodsFor: 'focus' stamp: 'miso 12/29/2004 11:02'!
keyboardFocusChange: aBoolean

	aBoolean
		ifTrue: ["A hand is wanting to send us characters..."
			editView coreBecomeFocus]
		ifFalse: ["A hand has clicked elsewhere..."
			editView coreStopBeingFocus].
	^ super keyboardFocusChange: aBoolean.! !

!GolgiTextMorph methodsFor: 'goodie' stamp: 'miso 12/29/2004 20:29'!
quickContents: stringOrText
"override for speed: do not create/relese para cache to set bounds now (as opposed to newContents:). Makes a big difference while loading headers (which will be resizedanyway)."

	text := stringOrText asText! !

!GolgiTextMorph methodsFor: 'goodie' stamp: 'miso 12/30/2004 10:43'!
rawEditor
"for debugging"

^ editor! !

!GolgiTextMorph methodsFor: 'goodie' stamp: 'miso 12/30/2004 10:45'!
rawPara
"for debugging"

^ paragraph! !

Object subclass: #GoCoffreFort
	instanceVariableNames: ''
	classVariableNames: 'Tresor ParaCache'
	poolDictionaries: ''
	category: 'Golgi-Helpers'!

!GoCoffreFort commentStamp: 'miso 1/12/2005 19:17' prior: 0!
cut&paste buffer (aka pasteBuffer/clipboard)

[note: this was written for squeak around 2.6 and hasn't been revisited since]

Why me? The current squeak implementation distributes the clipboard between ParagraphEditor's CurrentSelection and HandMorph's PasteBuffer (concealed calls to Smalltalk.clipboardText which is the system clipboard) without unifying them. The result is that the textMorphs only have access to the text clipboard, thus preventing implementation of pasting headers onto an active meatMorph. It also prvides no mechanism for unicity of info across clipboards. Finally, the HandMorph.PasteBuffer insists on doing veryDeepCopy all the time without allowing any kind of lightweight access. 

I thus decided to write a wrapper around ParagraphEditor.CurrentSelection which is looked at by all goObjects, and that can contain either text or morphs. But now need to make this wrapper accessible from all textMorphs (ParagraphEditor). Considered resorting to a global declaration of GoSession as global and keep a reference to the pasteBuffer/clipboard in GoSession (or the pasteBuffer/clipboard as global, or declare the pasteBuffer/clipboard as a golgi global).  Kept a lightweight solution with a global in GoCoffreFort.

Upgraded the cut/copy/paste mechanism in GoParagraphEditor to use GoCoffreFort so as to ensure unicity of info and accessibility to other kinds of info.

There is one tough issue: since our content overrides that of ParagraphEditor, it becomes quirky to fill the ParagraphEditor buffer outside of golgi (thus not updating us) and then try to paste it into golgi. this would work only if our content is nil. To work around that i keep a copy of the last known state of the ParagraphEditor buffer, and if the actual content is different from that, i serve that (reasoning that it must hav been changed after i have been since i have not heard about it.) This is under 'cooperation fix'.!

!GoCoffreFort class methodsFor: 'access' stamp: 'ms 11/3/1998 15:16'!
content
	"alternative lexeme for 'tresor'"
	^ self tresor! !

!GoCoffreFort class methodsFor: 'access' stamp: 'ms 11/3/1998 15:16'!
content: x
	"alternative lexeme for 'tresor:'"
	^ self tresor: x! !

!GoCoffreFort class methodsFor: 'cooperation fix' stamp: 'miso 12/28/2004 11:16'!
paraBuffer
	^ Clipboard clipboardText! !

!GoCoffreFort class methodsFor: 'cooperation fix' stamp: 'ms 11/4/1998 12:56'!
paraBufferIsFresher
	"see explanation in class comments."
	^ (ParaCache = "sameAs:" self paraBuffer) not! !

!GoCoffreFort class methodsFor: 'access' stamp: 'ms 11/3/1998 15:15'!
put: x
	"alternative lexeme for 'tresor:'"
	^ self tresor: x! !

!GoCoffreFort class methodsFor: 'access' stamp: 'ms 11/4/1998 11:44'!
stealContents
	"to get the content of the paste buffer, emptying it after uses."
	| t |
	t _ self tresor.
	Tresor _ nil.
	^ t! !

!GoCoffreFort class methodsFor: 'access' stamp: 'ms 11/4/1998 11:43'!
tresor
	self paraBufferIsFresher ifTrue: [
		Tresor _ nil.
		^ self paraBuffer].
	^ Tresor! !

!GoCoffreFort class methodsFor: 'access' stamp: 'ms 11/4/1998 13:00'!
tresor: t
	self updateParaCache.
	Tresor _ t! !

!GoCoffreFort class methodsFor: 'cooperation fix' stamp: 'ms 11/4/1998 12:59'!
updateParaCache
	ParaCache _ self paraBuffer! !

Object subclass: #GoPreferences
	instanceVariableNames: 'prefDict sessionMngr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Golgi-Helpers'!

!GoPreferences commentStamp: 'miso 1/12/2005 19:15' prior: 0!
I hold prefs (without presupposing that they are only booleans). When a preference changes, I try to notify all ongoing Golgi sessions of the change.

(Being an independent object i can be swapped, so there can be many pref-sets coexisting in Golgi.)!

!GoPreferences class methodsFor: 'as yet unclassified' stamp: 'miso 10/5/1999 21:52'!
new
	^ super new initialize.! !

!GoPreferences methodsFor: 'admin' stamp: 'ms 3/7/1999 04:53'!
checkCriticalPrefs
	GoHeader checkMargins! !

!GoPreferences methodsFor: 'macros' stamp: 'miso 10/10/1999 11:49'!
colorPref: c for: o

	| theColor |
	theColor _ self normaliseColor: c.

	o = #overall ifTrue: [^self setAllColorsTo: theColor].
	o = #GoHeader ifTrue: [^self headerColorPref: theColor].
	o = #GoMeat ifTrue: [^self meatColorPref: theColor].
	o = #Golgi ifTrue: [^self scrollpaneColorPref: theColor].

	self error: 'cannot set color pref for: ', o printString, ' coz i don''t know what it is'.

"	target _ (Compiler evaluate: o for: self logged: false).
	target setColor: theColor updating: bigBrothers.	"	! !

!GoPreferences methodsFor: 'get/set' stamp: 'miso 1/10/2005 21:41'!
enforcePref: prefUpdatingBlock
"walk all existing headers running prefUpdatingBlock on them. prefUpdatingBlock must take one var: the header being updated. Used when a pref is set, to make the change live"

	"why not GoHeader allInstancesDo: ?"
	Golgi allInstancesDo: [:v |
		v walkHeadersWith: prefUpdatingBlock ].
	Golgi sniffer scratchHeaderBank
	"what about hidden headers? GolgiCache, headerBankCache...: "

! !

!GoPreferences methodsFor: 'get/set' stamp: 'miso 1/8/2005 23:38'!
getPref: prefSymbol

	^ prefDict at: prefSymbol ifAbsent: [false]! !

!GoPreferences methodsFor: 'wrappers' stamp: 'ms 11/2/1998 12:35'!
headerColorPref: c
"reset the pref for color of meat & header, and notify all existing headers of the change."

	self setPref: #headerColor to: c.
	self setPref: #meatColor to: c.
	self enforcePref: [:h| h setColor: c].! !

!GoPreferences methodsFor: 'admin' stamp: 'ms 11/2/1998 12:23'!
initialize
	prefDict _ Dictionary new! !

!GoPreferences methodsFor: 'wrappers' stamp: 'ms 11/2/1998 12:35'!
meatColorPref: c
	"reset the pref for color of meat & header, and notify all existing headers of the change"
	self setPref: #meatColor to: c.
	self enforcePref: [:h| h meatMorph setColor: c]! !

!GoPreferences methodsFor: 'macros' stamp: 'ms 3/7/1999 03:26'!
normaliseColor: c
	| theColor |
	theColor _ c.
	theColor ifNil: [
		theColor _ FillInTheBlank request: 'type a color in:' initialAnswer: 'black'.
		theColor isEmpty ifTrue: [^self].
		theColor _ theColor asSymbol].
	(theColor isKindOf: Symbol)
		ifTrue: [theColor _ Color perform: theColor].
	^ theColor
	"#2	view backgroundColor: Color fromUser; uncacheBits; display"
	"#3- HandMorph chooseColor"
		"w colorPicker"
		"w _ self world"
		"colorPicker _ self changeColorTarget: nil selector: nil"
		"[colorPicker isInWorld] whileTrue: [w doOneCycle]"
		"^ colorPicker selectedColor"! !

!GoPreferences methodsFor: 'wrappers' stamp: 'miso 10/10/1999 11:50'!
scrollpaneColorPref: c
"reset the pref for color of scrollpane, and notify all existing bigBrothers of the change."

	self setPref: #scrollpaneColor to: c.
	self enforcePref: [:h| h setColor: c].! !

!GoPreferences methodsFor: 'admin' stamp: 'ms 11/2/1998 12:23'!
sessionMngr: m
	sessionMngr _ m! !

!GoPreferences methodsFor: 'macros' stamp: 'ms 11/2/1998 12:37'!
setAllColorsTo: realColor
	self headerColorPref: realColor.
	self scrollpaneColorPref: realColor	"scroller"! !

!GoPreferences methodsFor: 'macros' stamp: 'miso 10/10/1999 11:56'!
setMargin: whichMargin in: o to: n

	| target |
	target _ (Compiler evaluate: o for: self logged: false).
	target setMargin: whichMargin to: n updating: Golgi allInstances! !

!GoPreferences methodsFor: 'get/set' stamp: 'ms 11/2/1998 12:20'!
setPref: prefSymbol to: v

	^ prefDict at: prefSymbol put: v! !

Object subclass: #GoSniffer
	instanceVariableNames: 'bigBrothers sniffDaughters theSniffProcess shutUp shortDelay longDelay headerBank useSniffer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Golgi-Helpers'!

!GoSniffer commentStamp: 'miso 1/12/2005 19:14' prior: 0!
I hold a background process which (i) loads the hidden and unloaded daughters of leaf nodes in all the open outlines so that their expansion feels quick, and (ii) makes sure that HeaderBank is stocked up with empty headers ready for use so that header creation feels quick. 

I am governed by a preference as to whether that background process should be used. This preference is currently 'no' by default, because turning me on causes me to eat all the CPU cycles :)  [Probably because this class was coded before the author knew anything about smalltalk and hence the process-cycle code is awful] The preference lives in 'Golgi prefs getPref: #useSniffer' and is cached in my instance variable <useSniffer>. 

[self-note: the code for turning the preference on/off while Golgi is running is not written yet but should be trivial]!

!GoSniffer class methodsFor: 'as yet unclassified' stamp: 'miso 1/10/2005 21:32'!
killAllSessions
	"close all open outlines. if some GoSession instance still exists after that, emergencyKill it."
	| gTop theSniff |
	theSniff := self someInstance.
	theSniff ifNil: [^self].
	theSniff killProcess.
	Object halt. gTop := theSniff mainBigBrother.
	[gTop = nil] whileFalse: [
		"find the top of that golg"
		(gTop owner isKindOf: TransformMorph) ifTrue: [
			gTop := gTop owner owner].
		"find who is responsible for that top. ask him to die"
		(gTop owner isKindOf: SystemWindow)
			ifTrue: [
				"bring to front"
				gTop owner model ifNil: "something went wrong" [gTop die].
				gTop owner delete]
			ifFalse: "weird. just detach it?" [
				gTop die].
		gTop := theSniff mainBigBrother].
	theSniff zapDaughtersToSniff "delete requests for daughters".
	"check this does it"
	theSniff := nil "no more pointers to that instance".
	Smalltalk garbageCollect.
	self someInstance
		ifNil: [
			self notify: 'dead!!']
		ifNotNil: [
			self someInstance inspect.
			self error: 'can''t get this GoSession instance to die...'.]! !

!GoSniffer class methodsFor: 'as yet unclassified' stamp: 'miso 10/5/1999 21:53'!
new
	^ super new initialize.! !

!GoSniffer class methodsFor: 'as yet unclassified' stamp: 'ms 3/7/1999 03:47'!
sniffer

	^ self theOnlyInstance! !

!GoSniffer class methodsFor: 'as yet unclassified' stamp: 'miso 10/10/1999 22:51'!
theOnlyInstance
	self instanceCount > 0 
		ifTrue: [^ self someInstance ]
		ifFalse: [^ super new initialize]! !

!GoSniffer methodsFor: 'big brothers and daughters' stamp: 'miso 1/10/2005 21:47'!
addBigBrother: b

	useSniffer ifFalse: [^self].
	bigBrothers := bigBrothers copyWith: b.
! !

!GoSniffer methodsFor: 'the bank' stamp: 'ms 3/7/1999 04:38'!
askFillHeaderBank
	[self headerBank addOneHeaderToBank ~~ nil] whileTrue:  [
		self giveOneCycle]! !

!GoSniffer methodsFor: 'the process' stamp: 'miso 1/10/2005 21:32'!
checkSnifferAlive

	theSniffProcess ifNil: [ "??? hwo happened"
		self launchSniffingProcess].
	theSniffProcess suspendedContext ifNil: [
		self killProcess.
		self launchSniffingProcess	]
! !

!GoSniffer methodsFor: 'the process' stamp: 'ms 3/7/1999 03:13'!
giveOneCycle
	self mainBigBrother ifNil: [
		^ Processor yield].
	self mainBigBrother world doOneCycle.
	shutUp ifTrue: [
		self suspendSniffer].
	Processor yield! !

!GoSniffer methodsFor: 'misc' stamp: 'miso 11/26/2000 00:26'!
handleListenEvent: evt! !

!GoSniffer methodsFor: 'the bank' stamp: 'ms 3/7/1999 04:38'!
headerBank
	^ headerBank! !

!GoSniffer methodsFor: 'misc' stamp: 'miso 1/10/2005 21:58'!
initSniffing

	sniffDaughters := Array new.
	bigBrothers := Array new.
	shortDelay := Delay forMilliseconds: 500.
	longDelay := Delay forSeconds: 5.
	shutUp := false.
	self launchSniffingProcess.
	
! !

!GoSniffer methodsFor: 'misc' stamp: 'miso 1/10/2005 21:58'!
initialize

	headerBank := HeaderBank new.
	useSniffer := Golgi prefs getPref: #useSniffer.
	useSniffer ifTrue: [self initSniffing].
	
! !

!GoSniffer methodsFor: 'big brothers and daughters' stamp: 'miso 1/12/2005 18:20'!
isEmpty
"for debugging. answer whether I refer to some headers or golgs."

	useSniffer ifFalse: [^true].
	^ bigBrothers isEmpty and: [sniffDaughters isEmptyOrNil]! !

!GoSniffer methodsFor: 'the process' stamp: 'miso 1/10/2005 21:32'!
killProcess

	World primaryHand removeEventListener: self.
	theSniffProcess ifNotNil: [
		theSniffProcess terminate]! !

!GoSniffer methodsFor: 'the process' stamp: 'miso 1/10/2005 21:27'!
launchSniffingProcess

	theSniffProcess :=  [self sniffForever] fork


"pseudoFork: aBlock
This mimicks fork/forkAt: priority, which we will use once it returns the newly created process.

	| forkedProcess |
	forkedProcess _ aBlock newProcess.
	// forkedProcess priority: Processor userBackgroundPriority.
	forkedProcess resume.
	^ forkedProcess.	"
! !

!GoSniffer methodsFor: 'big brothers and daughters' stamp: 'miso 1/10/2005 21:40'!
loadDaughtersOf: aHeader

	useSniffer ifFalse: [^self].
		
	self checkSnifferAlive.
	(sniffDaughters includes: aHeader) ifFalse:
		[sniffDaughters := (Array with: aHeader), sniffDaughters]! !

!GoSniffer methodsFor: 'big brothers and daughters' stamp: 'ms 3/7/1999 03:15'!
loadRequest
	| h  |
	[sniffDaughters size > 0] whileTrue: [
		h _ sniffDaughters first.
		h subheaders do: [
			:newH  |
			self sniff: newH].
		sniffDaughters _ sniffDaughters copyWithout: h]! !

!GoSniffer methodsFor: 'big brothers and daughters' stamp: 'miso 1/10/2005 21:52'!
mainBigBrother
	"return the first bigBro that is currently in world, if none of them is, just the first bigBro, if none, nil."
	bigBrothers isEmpty
		ifTrue: [^nil]
		ifFalse: [
			bigBrothers do: [: b | b world ifNotNil: [^b]].
			"none was in a world, return the first"
			^ bigBrothers first].
	^nil! !

!GoSniffer methodsFor: 'big brothers and daughters' stamp: 'miso 10/13/1999 11:30'!
removeBigBrother: b

	bigBrothers _ bigBrothers copyWithout: b	
! !

!GoSniffer methodsFor: 'the process' stamp: 'ms 3/7/1999 03:19'!
resumeSniffer

	theSniffProcess ifNotNil: [ theSniffProcess resume ]
! !

!GoSniffer methodsFor: 'the bank' stamp: 'miso 1/10/2005 21:40'!
scratchHeaderBank

^ self headerBank scratchHeaderBank! !

!GoSniffer methodsFor: 'the process' stamp: 'miso 12/28/2004 19:23'!
sniff: aHeader
	"load subheader if necessary"
	| hPane  |
	aHeader isExpanded ifTrue: [
		^ self].
	aHeader subheadersNeedLoading ifFalse: [
		^ self
		"	Transcript show: 'loading...', (aHeader getMeat); cr."].
	hPane := aHeader subheaderPane.
	self giveOneCycle.
	aHeader translator daughters do: [
		:d  |
		aHeader dressSubmeme: d.
		self giveOneCycle].
	aHeader isExpanded ifFalse: [
		hPane boundsGivenTopline: ((aHeader bounds left + 25) @ aHeader bounds bottom corner: aHeader bounds right @ 0).
		self giveOneCycle]! !

!GoSniffer methodsFor: 'the process' stamp: 'miso 1/10/2005 21:32'!
sniffForever

	[self mainBigBrother = nil] whileTrue: [shortDelay wait].
	[self mainBigBrother primaryHand = nil] whileTrue: [shortDelay wait].

	[true] whileTrue: [
		bigBrothers isEmpty ifTrue: [self killProcess].
		World ifNotNil: [
			World hands first addEventListener: self.
			self loadRequest.
			self askFillHeaderBank.
			World hands first removeEventListener: self].
		Processor yield.
		"	self backgroundLoadDaughtersOf: bigBro.
		theGlobals scratchHeaderBank.	"
		" using 'longDelay wait' at the end of each cycle is not good: somehow it slows down the main process a lot (on expands)."]! !

!GoSniffer methodsFor: 'the process' stamp: 'miso 11/25/2000 19:47'!
suspendSniffer

	theSniffProcess ifNotNil: [ 
		shutUp_false. 
		self mainBigBrother primaryHand removeEventListener: self.
		theSniffProcess suspend ]
! !

!GoSniffer methodsFor: 'the process' stamp: 'miso 1/9/2005 00:44'!
theProcess

^ theSniffProcess! !

!GoSniffer methodsFor: 'misc' stamp: 'miso 1/10/2005 21:44'!
viewClosingDown: bigBrotherOfView
"an object viewer on a memePool is closing down. Take whatever steps are appropriate to the end of an editing session. eg ask backup mngr if wants to do anything."

	useSniffer ifFalse: [^self].

	self removeBigBrother: bigBrotherOfView.
	bigBrothers isEmpty ifTrue: [self killProcess].


! !

!GoSniffer methodsFor: 'big brothers and daughters' stamp: 'miso 6/16/2000 23:22'!
zapDaughtersToSniff
	sniffDaughters _ Array new.! !

Object subclass: #HeaderBank
	instanceVariableNames: 'dummyMeme'
	classVariableNames: 'MaxHeadersInBank TheHeaderBank'
	poolDictionaries: ''
	category: 'Golgi-Helpers'!

!HeaderBank commentStamp: 'miso 1/12/2005 19:08' prior: 0!
For speed, I preventively stock headers. When a new header is needed, it can be snatched from me and its content reset, which is much faster than recreating a whole new header.

Note that this mechanism is dependent upon the sniffer process (GoSniffer) which however currently turned off by default because it drives cpu up dramatically. !

!HeaderBank class methodsFor: 'serve headers' stamp: 'ms 3/8/1999 16:02'!
headerFromBank
	| resu  |
	TheHeaderBank size > 0 ifTrue: [
		resu _ TheHeaderBank first.
		TheHeaderBank _ TheHeaderBank copyWithout: resu.
		^ resu].
	Transcript
		 show: 'empty';
		 cr.
	^ nil! !

!HeaderBank class methodsFor: 'serve headers' stamp: 'miso 10/5/1999 21:53'!
new
	^ super new initialize.! !

!HeaderBank methodsFor: 'header bank' stamp: 'miso 10/19/1999 14:07'!
addOneHeaderToBank

	self privTempCheckDummymemeSanity.
	TheHeaderBank size < self maxHeadersInBank
		ifTrue: [TheHeaderBank _ TheHeaderBank copyWith: 
			(GoHeader rawWithAccessor: (GoTranslator meme: dummyMeme))]
		ifFalse: [^ nil ]! !

!HeaderBank methodsFor: 'accessors' stamp: 'miso 10/19/1999 13:38'!
dummyMeme
	^ dummyMeme! !

!HeaderBank methodsFor: 'header bank' stamp: 'ms 7/3/1999 22:13'!
fillHeaderBank

	| i |
	i_TheHeaderBank size.
	[i<=self maxHeadersInBank] whileTrue: [
		TheHeaderBank _ TheHeaderBank copyWith: 
			(GoHeader rawWithAccessor: (GoTranslator meme: dummyMeme)).
		i _ TheHeaderBank size]! !

!HeaderBank methodsFor: 'header bank' stamp: 'ms 3/8/1999 16:02'!
headerFromBank
	|resu|
	TheHeaderBank size > 0 ifTrue: [
		resu _ TheHeaderBank first. 
		TheHeaderBank _ TheHeaderBank copyWithout: resu.
		^ resu ].
	^nil! !

!HeaderBank methodsFor: 'header bank' stamp: 'ms 3/8/1999 16:03'!
initialize

	TheHeaderBank ifNil: [TheHeaderBank _ Array new].
	dummyMeme _ Meme new.! !

!HeaderBank methodsFor: 'accessors' stamp: 'ms 9/7/1998 19:40'!
maxHeadersInBank
	MaxHeadersInBank ifNil: [ MaxHeadersInBank _ 200 ].
	^ MaxHeadersInBank! !

!HeaderBank methodsFor: 'accessors' stamp: 'ms 9/5/1998 19:34'!
maxHeadersInBank: m
	MaxHeadersInBank _ m! !

!HeaderBank methodsFor: 'header bank' stamp: 'miso 10/19/1999 13:53'!
privTempCheckDummymemeSanity
"somehow dummy meme once ended with 2700 dependents, none of them in the bank and all of them with fresh unlinked headers..."

	dummyMeme dependents size > 210 ifTrue: [^ self error: 'too many dependents on dummy meme in header bank... ?!!!!?']! !

!HeaderBank methodsFor: 'header bank' stamp: 'ms 3/8/1999 16:03'!
scratchHeaderBank

	TheHeaderBank _ Array new! !

!Object methodsFor: '*golgi' stamp: 'miso 11/27/2000 01:10'!
countLeaves

	^ self subclasses inject: 0 into: [:subTotal :subClass | 
		subTotal + (subClass subclasses size = 0 
			ifTrue: [1]
			ifFalse: [subClass countLeaves])]! !

!Object methodsFor: '*golgi' stamp: 'miso 11/27/2000 01:14'!
countMonoSubs

	^ (self subclasses select: [:sc | sc subclasses size == 0]) size! !

!TextMorph methodsFor: '*golgi-overrides' stamp: 'mich 1/12/2005 22:12'!
editorClass
	"Answer the class used to create the receiver's editor"
	
	^TextMorphEditor! !

!TextMorph methodsFor: '*golgi-overrides' stamp: 'miso 12/29/2004 20:01'!
installEditorToReplace: priorEditor
	"Install an editor for my paragraph.  This constitutes 'hasFocus'.
	If priorEditor is not nil, then initialize the new editor from its state.
	We may want to rework this so it actually uses the prior editor."

	| stateArray |
	priorEditor ifNotNil: [stateArray := priorEditor stateArray].
	editor := self editorClass new morph: self.
	editor changeParagraph: self paragraph.
	priorEditor ifNotNil: [editor stateArrayPut: stateArray].
	self selectionChanged.
self paragraph focused: true.	"if this constitutes hasFocus, then tell our paragraph!!"
	^ editor! !

RectangleMorph subclass: #GoHeadersPane
	instanceVariableNames: 'bigBrother paneToHeaderLink'
	classVariableNames: 'TopMargin PaddingBetweenSubTexts RightMargin BottomMargin LeftMargin'
	poolDictionaries: ''
	category: 'Golgi-Basics'!

!GoHeadersPane commentStamp: 'miso 1/12/2005 19:54' prior: 0!
I am a pane which contains header-related object and I grow vertically as those objects grow, but I keep a fixed horizontal size. In practice I am used as a pane that contains the subheaders of a given GoHeader (and my immediate sublcass - GoHeader - is used as a header containing the content of the meme and an instance of me). 
!

GoHeadersPane subclass: #GoHeader
	instanceVariableNames: 'translator meat wedge meatIsDirty'
	classVariableNames: 'HeaderBottomMargin HeaderLeftMargin PaneColor HeaderRightMargin HeaderTopMargin'
	poolDictionaries: ''
	category: 'Golgi-Basics'!

!GoHeader commentStamp: 'miso 1/12/2005 19:58' prior: 0!
I represent one header in a Golgi outliner. My major components are: a wedge (GoWege), a text (GoMeat), and a pane with subheaders (GoHeadersPane).

[See the comment for GoHeadersPane to see why I am a special type of GoHeadersPane - the main difference coming down to the fact that I keep a margin to the left of my subheaders, and I put a wedge in that margin.]!

!GoHeader class methodsFor: 'prefs' stamp: 'ms 10/7/1998 02:38'!
Margin: whichMargin

	| theMargin |
	whichMargin = #top  ifTrue: 	[theMargin _ HeaderTopMargin].
	whichMargin = #bottom  ifTrue: [theMargin _ HeaderBottomMargin].
	whichMargin = #right  ifTrue: 	[theMargin _ HeaderRightMargin].
	whichMargin = #left  ifTrue: 	[theMargin _ HeaderLeftMargin].
	whichMargin = #interSub  ifTrue: [theMargin _ PaddingBetweenSubTexts].

	theMargin ifNil: [ theMargin _ self setMargin: whichMargin 
						to: (self defaultValueOfMargin: whichMargin)].
	^ theMargin! !

!GoHeader class methodsFor: 'prefs' stamp: 'ms 9/21/1998 02:57'!
checkMargins
"Make sure margins have some value. else fullBounds will fail. Just calling the margins ensures that they get some value."

	self Margin: #top.
	self Margin: #bottom.
	self Margin: #right.
	self Margin: #left.
	self Margin: #interSub.! !

!GoHeader class methodsFor: 'prefs' stamp: 'ms 10/7/1998 12:09'!
defaultValueOfMargin: whichMargin

	whichMargin = #top  ifTrue: 	[^0].
	whichMargin = #bottom  ifTrue: [^0].
	whichMargin = #right  ifTrue: 	[^0].
	whichMargin = #left  ifTrue: 	[^20].
	whichMargin = #interSub  ifTrue: [^0].

	^0! !

!GoHeader class methodsFor: 'instantiating' stamp: 'miso 10/25/1999 10:13'!
rawWithAccessor: anAccessor
"do not try to tap HeaderBank, since HeaderBank calls me to fill itself"

	^ super new translator: anAccessor! !

!GoHeader class methodsFor: 'prefs' stamp: 'ms 10/7/1998 02:38'!
setMargin: whichMargin to: n

	whichMargin = #top  ifTrue: 	[HeaderTopMargin _ n].
	whichMargin = #bottom  ifTrue: [HeaderBottomMargin _ n].
	whichMargin = #right  ifTrue: 	[HeaderRightMargin _ n].
	whichMargin = #left  ifTrue: 	[HeaderLeftMargin _ n].
	whichMargin = #interSub  ifTrue: [PaddingBetweenSubTexts _ n].

	^n
"indentingTopMargin == HeaderPaneTopMargin"
"indentingBottomMargin == HeaderPaneBottomMargin"! !

!GoHeader class methodsFor: 'prefs' stamp: 'miso 10/24/1999 15:24'!
setMargin: whichMargin to: n updating: listOfViews

	self setMargin: whichMargin to: n.
	whichMargin = #wedge
		ifTrue: [
			listOfViews do: [:v |
				v walkHeadersWith: [:h| h resetWedgeFromLeft: n] ]]
		ifFalse: [
			listOfViews ifNotNil: [
				listOfViews do: [:v | v layoutChanged ]]]! !

!GoHeader class methodsFor: 'instantiating' stamp: 'miso 10/25/1999 10:13'!
withTranslator: anAccessor

	^ self new translator: anAccessor "reset accessor and refresh properties from it"

! !

!GoHeader methodsFor: 'wedge' stamp: 'miso 10/24/1999 15:23'!
aNextStateMethod: theButton
"private!! the method passed to wedge to compute next state given current state of affairs. Ideally should be able to get info about modifier key being pressed when button was clicked."

	| state |
	state _ theButton state.

	"returns #expanded if state=nil !!"
	state = #inert  ifTrue: [ ^ #inert ].
	state = #contracted 
		ifTrue:	[ ^ #expanded]
		ifFalse: [ ^ #contracted].

	^ self error: 'I cannot switch from this state, because I do not understand it... ', 
		state printString. ! !

!GoHeader methodsFor: 'drag&drop' stamp: 'miso 7/5/1999 21:49'!
acceptDroppingMorph: aMorph event: evt
	"standard API to allow the user to add submorphs just by dropping them on this morph."

	| newMom newSister |
	(aMorph isKindOf: GoHeader) ifFalse: [^false]. "don't swallow nonHeader objects"

	"decide whether aMorph should be my sister or my daughter"
	(self dropInside: (evt transformedBy: (self transformFrom: World)) cursorPoint)
		ifTrue: [ newMom _ self. self isExpanded ifTrue: [newSister _ #root] ifFalse: [newSister _ #last]]
		ifFalse:  [ newMom _ self headerAbove. newSister _ self].

	^ evt shiftPressed "allow either mv of aMorph or creation of duplicates (aliases/links)"
		ifTrue: [aMorph wannaPutSelfIntoHeader: newMom afterSub: newSister]
		ifFalse: [aMorph wannaMoveSelfIntoHeader: newMom afterSub: newSister]! !

!GoHeader methodsFor: '  core reorg subs' stamp: 'miso 10/29/1999 14:59'!
addSubHeader: incomingHeader after: existingHeader
"internal: called by incomingHeader during the process of moving/deleting itself. Use reorgSelf methods. 
This is a relay verb for the building block of reorgSelf and create, ie. pane.addSubHeader:after:"

	^ self subheaderPane addSubHeader: incomingHeader after: existingHeader.
! !

!GoHeader methodsFor: 'saving' stamp: 'ms 10/11/1998 18:23'!
askBeforeDiscardingEdits
"should be a pref when those will exist"

	^ true! !

!GoHeader methodsFor: 'focus' stamp: 'ms 11/1/1998 20:39'!
askForFocus: hand

	^ hand newKeyboardFocus: self.! !

!GoHeader methodsFor: 'focus' stamp: 'miso 10/25/1999 12:40'!
askForMeatFocus

	^ meat askForFocus: self primaryHand.! !

!GoHeader methodsFor: 'focus' stamp: 'miso 10/25/1999 12:40'!
askForMeatFocus: hand

	^ meat askForFocus: hand! !

!GoHeader methodsFor: 'drag&drop' stamp: 'miso 10/7/1999 11:43'!
beGrabbedByEvt: evt
	"called from wedge, initiates a drag&drop operation. See documentation."

	GoHeaderAlias visualiseDragOf: self fromEvent: evt! !

!GoHeader methodsFor: 'aboveMe' stamp: 'miso 10/8/1999 20:12'!
bigBrother: bb
	bigBrother _ bb.
	paneToHeaderLink ifNotNil: [paneToHeaderLink bigBrother: bb]! !

!GoHeader methodsFor: '  margins' stamp: 'ms 9/12/1998 16:38'!
bottomMargin
	^ HeaderBottomMargin! !

!GoHeader methodsFor: '  subheaderPane' stamp: 'miso 10/28/1999 13:26'!
checkForEmptySubPane

	self subheaders size = 0 ifTrue: [ 
		self setWedgeInert. 
		self hideSubheaders. "undo the owner/submorphs connection"
		paneToHeaderLink newMom: nil.
		paneToHeaderLink _ nil. "now undo the paneLink&bigBro connection"
		].

"do not keep empty subpanes around, so that (even silent) arrival of new subs triggers creation of a new subpane, and the concmittant setting of the wedge (otherwise would need a mechanism to track re-arrival of a first subheader)"! !

!GoHeader methodsFor: 'wedge' stamp: 'miso 1/3/2005 12:33'!
cmdMouseDownIn: wedgeName while: evt

	translator headerMenu "popUpAt: evt hand position event: evt" popUpInWorld! !

!GoHeader methodsFor: 'wedge' stamp: 'ms 9/8/1998 18:29'!
cmdMouseMoveIn: wedgeName while: evt

"	evt hand showTemporaryCursor: Cursor menu.
	^ evt hand showTemporaryCursor: nil."
! !

!GoHeader methodsFor: 'cut&paste' stamp: 'ms 11/3/1998 16:46'!
coffreFort
	"return a reference to the pasteBuffer we use in common with MeatMorph"
	^ GoCoffreFort! !

!GoHeader methodsFor: 'focus' stamp: 'miso 10/30/1999 17:04'!
containsFocus
"am i, or something contained in me, focused?"

	self bigBrother ifNil: [^ false].
	self bigBrother lastFoci do: [: f | (f isInside: self) ifTrue: [^true]].
	^ false


" this does it the recursive way, which I suppose is slower:

	(self hasFocus or: [theMeatMorph hasFocus]) ifTrue: [^true].
	self isContracted ifTrue: [^ false].  // no need to look into contracted subs 
	self subheaders do: [: s | s containsFocus ifTrue: [^true]].
	^false.
"! !

!GoHeader methodsFor: 'meat' stamp: 'miso 11/2/1999 17:10'!
content
"this is neutral between headerMeat and memeMeat. Interpreted as headerMeat if there is some, and memeMeat otherwise (eg in bigBrother)"
	^ self headerMeat! !

!GoHeader methodsFor: 'cut&paste' stamp: 'ms 11/22/1998 17:13'!
copyToCoffreFort
	"= copyToClipbard, copyToPasteBuffer"
	self hasFocus ifFalse: [^nil] "we might be called on a cmdC in our meat. if so, decline to act".
	self coffreFort put: self copyWithAccessor! !

!GoHeader methodsFor: 'create delete copy' stamp: 'miso 10/25/1999 12:37'!
copyWithAccessor
	"copy me, my accessor, my meme, and all the submemes in my meme. don't dress those however."
	^ self class withTranslator: translator copyWithMeme

! !

!GoHeader methodsFor: '  core' stamp: 'miso 12/28/2004 22:02'!
coreBecomeFocus
"called from hand.newKeyboardFocus: via 'self.keyboardFocusChange: true'. Hand already registered us as the focus bearer (in its keyboardFocus var)"

self halt.
	self walkHeadersWith: [:h| 
		h wedgeBackgroundColor: Color yellow.
		h meatColor: Color yellow.].

	self bigBrother ifNotNil: [
		self bigBrother lastFocus: self]. "keep track of last focus for mouseEnter: in golgPane"
! !

!GoHeader methodsFor: '  core reorg self (priv)' stamp: 'miso 10/25/1999 11:30'!
corePutSelfIntoHeader: newContainerHeader afterSub: futureSisterDescriptor
"core mv header verb (does not deal with focus-inheritance, or meme-side of things: only used  after having moved the data around. Use wanna* verbs). Might be a mv from within viewer, outside of viewer or a creation."

	^ newContainerHeader addSubHeader: self after: futureSisterDescriptor! !

!GoHeader methodsFor: '  core reorg self (priv)' stamp: 'miso 10/25/1999 12:57'!
corePutSelfOutOfMom
"core mv header verb, does not check for focus-deletion, or meme-side of things. Might be a mv within viewer, outside of viewer or a deletion."
"see prev version for capacity to come from nonGolg environment"

	^ self headerAbove releaseOneSubheader: self.! !

!GoHeader methodsFor: '  core' stamp: 'miso 12/29/2004 10:42'!
coreStopBeingFocus
"called from hand.newKeyboardFocus: via 'self.keyboardFocusChange: false'. Hand already unregistered us as the focus bearer (in its keyboardFocus var)"

self halt. "is this ever called?"
	self walkHeadersWith: [:h| 
		h defaultWedge.
		h meatColor: self translator defaultColor].! !

!GoHeader methodsFor: 'general properties' stamp: 'miso 10/30/1999 15:37'!
countExpandedSubHeaders

	| n |
	n _ self subheaders size.
	self subheaders do: [:d|
		n _ n + (d countExpandedSubHeaders)].
	^ n
! !

!GoHeader methodsFor: 'cut&paste' stamp: 'miso 10/20/1999 13:08'!
cut
	"put my meme into buffer, forget about me completly. A header will be rebuilt if I am pasted into a golg"

	self hasFocus ifFalse: [^nil] "we might be called on a cmdC in our meat. if so, decline to act".

	self coffreFort put: self meme.
	self wannaSuicide.! !

!GoHeader methodsFor: '  data callbacks' stamp: 'miso 10/20/1999 13:59'!
dataGotOutOfMom "=suicide = getOutOfMom =putSelfOutOfMom"
"I view data under this mom, but data get out of it without going anywhere. I have no more business, suicide (forget about my meme). Don't change focus, except if I contain it."

	| nextFocus |
	nextFocus _ self containsFocus ifTrue: [self whoInheritsFocus].
	self corePutSelfOutOfMom 
		ifFalse: [^false].
	self translator forgetHeaderAndMeme.
	nextFocus ifNotNil: [nextFocus askForMeatFocus].
	^ true! !

!GoHeader methodsFor: '  data callbacks' stamp: 'miso 10/20/1999 13:59'!
dataMovedOutOfMomInto: newContainerHeader afterSub: futureSisterDescriptor "=moveSelfOutOfMom:afterSub"
"make a move that imitates the data move"

	self corePutSelfOutOfMom.
	self corePutSelfIntoHeader: newContainerHeader afterSub: futureSisterDescriptor.
	^ true! !

!GoHeader methodsFor: 'subheaders' stamp: 'miso 10/24/1999 15:00'!
daughters
"added so that walkMeme actions can also walk panes (thus getting access to unsaved content)"

	^ self subheaders! !

!GoHeader methodsFor: 'wedge' stamp: 'miso 10/25/1999 12:42'!
defaultWedge
	wedge imageFamily: translator defaultImageFamily.
	self wedgeBackgroundColor: translator defaultColor.
! !

!GoHeader methodsFor: 'cut&paste' stamp: 'miso 10/20/1999 13:09'!
deferredCut
	"put me into buffer, move me when paste occurs"

	self hasFocus ifFalse: [^nil] "we might be called on a cmdC in our meat. if so, decline to act".

	self coffreFort put: self meme! !

!GoHeader methodsFor: 'create delete copy' stamp: 'miso 10/25/1999 10:11'!
detachFromdata
"make sure there are no more references to me or my translator. this is used when the golgi pane is getting deleted."
"find a better name"

	self subheaders do: [: s | s detachFromdata].
	self translator forgetHeaderAndMeme.! !

!GoHeader methodsFor: 'drag&drop' stamp: 'miso 10/7/1999 11:46'!
doYouWannaDelegateDropeeAt: localPoint
"i was elected as the receiver of the drop-wannabe, because i am the most embedded morph containing the drop-wannabe. but if the localPoint is in my left margin near one of my submorphs, I pass drop on to them"

	| adjustedPoint |
	(self isInMargin: localPoint) 
		ifTrue: [adjustedPoint _ (localPoint x + self leftMargin) @ localPoint y]
		ifFalse: [adjustedPoint _ localPoint].

	self isExpanded ifTrue: [ "contracted headers keep coords... (only some?)"
		self subheaders do: [: s |
			(s fullContainsPoint: adjustedPoint) ifTrue: [^ s doYouWannaDelegateDropeeAt: adjustedPoint]]].

	^ self! !

!GoHeader methodsFor: '  misc' stamp: 'miso 10/30/1999 16:05'!
dressSubheaders
"(re)load all daughters of my model. Return true if there were daughters to dress, false otherwise."

	self translator daughters do: [:d| self dressSubmeme: d].
	^ true



! !

!GoHeader methodsFor: '  misc' stamp: 'miso 10/28/1999 12:19'!
dressSubmeme: aSubmeme

	| h indexOfSisterAbove |
	(self isSubmemeDressed: aSubmeme) ifTrue: [^false].

	h _ GoTranslator createHeaderAndTranslatorFor: aSubmeme viewedInside: self.
	indexOfSisterAbove _ (aSubmeme indexInMom: self meme) - 1.
	^ h corePutSelfIntoHeader: self afterSub: indexOfSisterAbove
! !

!GoHeader methodsFor: 'drag&drop' stamp: 'miso 10/7/1999 12:24'!
dropInside: cursorPoz
	"i am the candidate-host for a drop or paste action, and i am asked whether i would absorb the dropped element as a submorph or not (ie or as a sister below me). Answer with bool. cursorPoz is in local coord."
	self isExpanded ifFalse: [
		"either sis below or first sub"
		(cursorPoz x - bounds left) < 22 ifTrue: [
			^ false]		"put under meat, as first sub"].
	^ true! !

!GoHeader methodsFor: 'misc' stamp: 'miso 1/3/2005 12:27'!
findGlobally
"now add: 
	i. real content of each result: the text around the highlighted result & text offset & clickable url by piece
	ii. layout glitch
	iii. btw menus don't work because of some popUp shit.
	iv. test it in a virgin 3.7 image with flappable scrollbars. test it in 3.8
"

	| searchString results pane scrollpane window |
	searchString := FillInTheBlank request: 'Search for what?'.
	searchString isEmpty ifTrue: [^ self].

	results := (self translator findGlobally: searchString) asArray.

window := SystemWindow labelled: 'Search results'.
window model: results.

scrollpane := ScrollPane new.
window addMorph: scrollpane frame: (0@0 corner: 1@1).

pane := Morph new "dont add that pane?"
			layoutPolicy: TableLayout new;
			listDirection: #topToBottom;
"			wrapDirection: #none;	// same if commented out"
"			hResizing: #spaceFill;	// does not work when owner is a transform... (transforms cannot have layout properties, that destroys the relativeness of the bounds of their submorphs)"
			setToAdhereToEdge: #width;	"hack to get #spaceFill -- depends on TransformMorph.fullBounds change"
			vResizing: #shrinkWrap;
			cellPositioning: #topLeft;
			color: Color transparent.
scrollpane scroller addMorph: pane.

results do: [:r | 
	| tm sel m moms |
	tm := (TextMorph new contentsWrapped: (r textForSearchOn: searchString)) toggleLocked.
"	tm borderColor: Color black; borderWidth: 1.		"
	tm hResizing: #spaceFill.
	pane addMorphBack: tm.
	sel := tm contents asString findString: searchString.
	tm editor selectFrom: sel to: sel + searchString size - 1.
	
	"now add a clickable url"
	m := r. moms := OrderedCollection new.
	[m := m primaryMom. m isNil] whileFalse: [moms addFirst: m].
	moms size > 2 ifTrue: [moms := moms copyFrom: 3 to: moms size].

	tm := '' asText.
	moms do: [:mom | | localPath |
		localPath := mom name asText.
		localPath addAttribute: (PluggableTextAttribute evalBlock: [Golgi view: mom] fixTemps).
		tm := tm, localPath, ' : ' asText].
	tm := TextMorph new contentsWrapped: tm.
	tm hResizing: #spaceFill; margins: 3.
	pane addMorphBack: tm.
	].

"sigh... changing bounds of the window will not be transmitted to pane if window's owner isNil. For some blackmagic reason invoking the sequence below after bounds: does make it stick..."
window bounds: (RealEstateAgent initialFrameFor: window world: self currentWorld).
window doLayoutIn: window layoutBounds.
pane layoutChanged. 

^window openAsIsIn: self currentWorld


"
	results _ results asArray collect: [:meme | 
		{meme content asString. meme internalPathAsArray collect: [:piece | piece unescapePercents]. meme}].
	results inspect
"! !

!GoHeader methodsFor: 'misc' stamp: 'miso 1/11/2005 11:39'!
findInMemes
"now add: 
	i. real content of each result: the text around the highlighted result & text offset & clickable url by piece
	ii. layout glitch
	iii. btw menus don't work because of some popUp shit.
	iv. test it in a virgin 3.7 image with flappable scrollbars. test it in 3.8
"

	| searchString results pane scrollpane window |
	searchString _ FillInTheBlank request: 'Search for what?'.
	searchString isEmpty ifTrue: [^ self].

	results _ Set new.
	MemeSoup primordialSoup walkSelfAndDaughtersWith: [:meme | 
		(meme content asString findString: searchString) > 0 ifTrue: [results add: meme]].

results _ results asArray.

window _ SystemWindow labelled: 'Search results'.
window model: results.

scrollpane _ ScrollPane new.
window addMorph: scrollpane frame: (0@0 corner: 1@1).

pane _ Morph new "dont add that pane?"
			layoutPolicy: TableLayout new;
			listDirection: #topToBottom;
"			wrapDirection: #none;	// same if commented out"
"			hResizing: #spaceFill;	// does not work when owner is a transform... (transforms cannot have layout properties, that destroys the relativeness of the bounds of their submorphs)"
			setToAdhereToEdge: #width;	"hack to get #spaceFill -- depends on TransformMorph.fullBounds change"
			vResizing: #shrinkWrap;
			cellPositioning: #topLeft;
			color: Color transparent.
scrollpane scroller addMorph: pane.

results do: [:r | 
	| tm sel |
	tm _ (TextMorph new contentsWrapped: (r textForSearchOn: searchString)) toggleLocked.
	tm borderColor: Color red; borderWidth: 1.	
	tm hResizing: #spaceFill.
	pane addMorphBack: tm.
	sel _ r content asString findString: searchString.
	tm editor selectFrom: sel to: sel + searchString size - 1.
	].

"sigh... changing bounds of the window will not be transmitted to pane if window's owner isNil. For some blackmagic reason invoking the sequence below after bounds: does make it stick..."
window bounds: (RealEstateAgent initialFrameFor: window world: self currentWorld).
window doLayoutIn: window layoutBounds.
pane layoutChanged. 

^window openAsIsIn: self currentWorld


"
	results _ results asArray collect: [:meme | 
		{meme content asString. meme internalPathAsArray collect: [:piece | piece unescapePercents]. meme}].
	results inspect
"! !

!GoHeader methodsFor: 'reorg self' stamp: 'miso 10/30/1999 19:32'!
goDown

	| lowerSister |
	lowerSister _ self sisterAfter.
		lowerSister == #end  ifTrue: [self flash. ^ false].
	^ self wannaMoveSelfAfter: lowerSister! !

!GoHeader methodsFor: 'reorg self' stamp: 'ms 7/3/1999 22:52'!
goLeft

	self isRoot ifTrue: [self flash. ^ false].
	^ self wannaMoveSelfAfter: self headerAbove! !

!GoHeader methodsFor: 'reorg self' stamp: 'miso 10/17/1999 10:06'!
goRight

	| higherSister |
	higherSister _ self sisterBefore.
		higherSister == #root  ifTrue: [self flash. ^ false].
	^ self wannaMoveSelfInto: higherSister
! !

!GoHeader methodsFor: 'reorg self' stamp: 'miso 11/2/1999 17:18'!
goUp
"eventually should become goUpBy: n. Or else the latter is a call to us with for 1 to n, goUp"

	| higherSister |
	higherSister _ self sisterBefore.
		higherSister == #root  ifTrue: [self flash. ^ false].
	^ self wannaMoveSelfBefore: higherSister
! !

!GoHeader methodsFor: 'focus' stamp: 'miso 10/20/1999 10:35'!
hasFocus
"this is to test whether the *entire* header is focused, ie. returns false if meat is in focus"

	^ self bigBrother 
		ifNil: [false]
		ifNotNil: [self bigBrother submorphHasFocus: self]! !

!GoHeader methodsFor: 'subheaders' stamp: 'miso 10/25/1999 12:37'!
hasSubheaders

	^ translator daughters size > 0! !

!GoHeader methodsFor: 'general properties' stamp: 'ms 7/3/1999 17:23'!
header
"use when you don't know whether you are in a pane or in me to be sure to deal with me (eg. printOn:)"
	^ self! !

!GoHeader methodsFor: 'aboveMe' stamp: 'miso 10/25/1999 13:36'!
headerAbove
"counts on the fact that headers are never contained in anything else but a GoHeadersPane"

	^ owner headerAbove! !

!GoHeader methodsFor: 'subheaders' stamp: 'miso 10/8/1999 20:13'!
headerAt: i
	^ paneToHeaderLink headerAt: i.! !

!GoHeader methodsFor: 'subheaders' stamp: 'miso 10/8/1999 20:12'!
headerAtIndex: i
	^ paneToHeaderLink headerAtIndex: i.! !

!GoHeader methodsFor: 'meat' stamp: 'miso 10/25/1999 12:40'!
headerMeat

	^ meat text! !

!GoHeader methodsFor: '  expansion' stamp: 'miso 10/14/1999 09:41'!
hideSubheaders
"This is called from the wedge, or from boundsGivenMax (and frm raleaseOnSub when releasing the last sub) , the wedge state is thus not to be changed. The owner/submorphs relation is broken so that subheaderpane does not compete for screenspace anymore. I don't really release the subheaderpane: it is referenced from my 'theSubheadersPane' var."

	self changed.			"invalidates old area"
	paneToHeaderLink zapOwner.
	submorphs _ submorphs copyWithout: paneToHeaderLink.
	self layoutChanged.		"force calc of new fullBounds"

	self containsFocus ifTrue: [self askForMeatFocus]

	"this is in case we go back to an implementation of containsFocus that always returns false if the header is contracted (and we don't want to test that before, in case it slows down the contraction:
	(self subheaders detect: [:h | h containsFocus] ifNone: [nil]) ifNotNil: [self askForMeatFocus]. "
! !

!GoHeader methodsFor: 'wedge' stamp: 'miso 10/25/1999 12:43'!
initWedge
"private"

	wedge _  GoWedge switch: [: theButton | self aNextStateMethod: theButton].
	wedge 	buttonName: 'wedge';
				target: self;
				position: bounds origin;
				extent: HeaderLeftMargin@16;
				state: #expanded 	startAction: [self unhideSubheadersAndRefresh]  
									stopAction: nil;
				state: #contracted 	startAction: [self hideSubheaders] 
									stopAction: nil;
				state: #inert			startAction: nil
									stopAction: nil.

	self addMorph: wedge.



"or:	theWedge addStatesFromDescriptorArray: 
							#(	(inert gray nil nil)
								(expanded green nil nil)
								(contracted red nil nil)	).
"
! !

!GoHeader methodsFor: 'create delete copy' stamp: 'miso 10/24/1999 15:38'!
initialize
"this is called at the beginning of instance creation: fill in those values that are presupposed"

	super initialize.
	paneToHeaderLink _ nil.
	meatIsDirty _ false.
	self	initWedge.
	^ self 
! !

!GoHeader methodsFor: 'general properties' stamp: 'miso 10/25/1999 12:43'!
isContracted

	^ wedge state == #contracted
! !

!GoHeader methodsFor: 'saving' stamp: 'ms 7/3/1999 23:49'!
isDirty
	^ self meatIsDirty or: [self subsAreDirty]

	! !

!GoHeader methodsFor: 'general properties' stamp: 'miso 10/25/1999 12:43'!
isExpanded

	^ 	wedge state == #expanded! !

!GoHeader methodsFor: 'drag&drop' stamp: 'ms 12/23/1998 23:36'!
isInMargin: localPoint

	localPoint x <= (self bounds left + self leftMargin) ifTrue: [^true].
	^false! !

!GoHeader methodsFor: 'aboveMe' stamp: 'miso 10/8/1999 21:45'!
isInside: putativeAncestor
	"this helps during drag&drop to prevent dropping inside myself... Delegate to model, as we might not be in the in the same outline"

	^ self translator isInside: putativeAncestor meme! !

!GoHeader methodsFor: 'create delete copy' stamp: 'miso 10/25/1999 12:38'!
isItOkToDie
"asks whether i can be discarded and end my life now. Yes unless me or my daughters contain changes that the user wants to continue editing. (sent by okToChange)"

	| r |
	self needToSave 
		ifTrue: [ r _ self queryForSave.
			r = #save ifTrue: [self recursiveSave. ^ true].
			r = #discard ifTrue: [translator bigBrotherWasDiscarded: self. ^ true].
			r = #cancel ifTrue: [^ false].
			r ifNil: [^ false].
			^ self error: 'dialog internal error']
		ifFalse: [^ true].


"	self needToSave 
		ifTrue: [^ self queryForSave]
		ifFalse: [^ true].		"

! !

!GoHeader methodsFor: 'general properties' stamp: 'ms 11/7/1998 16:46'!
isRoot
	^ owner isRoot! !

!GoHeader methodsFor: 'subheaders' stamp: 'ms 7/1/1999 23:02'!
isSubmemeDressed: aMeme
"answer whether any of my immediate daughters are viewing aMeme"

	(self whichDaughterDressesThisMeme: aMeme)
		ifNil: [^ false]
		ifNotNil: [^ true]! !

!GoHeader methodsFor: 'general properties' stamp: 'miso 10/24/1999 15:03'!
isThisYourMeme: aMeme
"answer whether aMeme is the meme I am currently viewing"

	^ self meme == aMeme! !

!GoHeader methodsFor: 'keys' stamp: 'miso 1/3/2005 11:22'!
key: keyNum combo: comboSymbol stream: s
	"this is the locus of default key mappings for all headers. can be overridden by translator. (default key mappings for meat are in ParagraphEditor)"
	| char  |
	^ (translator key: keyNum combo: comboSymbol) ifNil: [
		"take a shot at doing it ourselves, if people above didn't get it"
		char := Character value: keyNum.
		comboSymbol = nil ifTrue: [
			keyNum = 9 ifTrue: [
				self tab.
				^ true
				"tab reorgs"]].
		comboSymbol = #cmdShift ifTrue: [
			char = $s ifTrue: [
				self save: #local.
				^ true]].
		comboSymbol = #cmd ifTrue: [
			char = $c ifTrue: [
				^ self copyToCoffreFort
				"allow it to return nil"].
"			char = $d ifTrue: [
				self goDown.
				^ true].	"
			char = $f ifTrue: [
				self findGlobally.
				^ true].
			char = $l ifTrue: [
				self goLeft.
				^ true].
			char = $r ifTrue: [
				self goRight.
				^ true].
			char = $u ifTrue: [
				self goUp.
				^ true].
			char = $v ifTrue: [
				^ self paste].
			char = $x ifTrue: [
				^ self cut].
			keyNum = 8 ifTrue: [
				self wannaSuicide.
				^ true
				"cmdDelete"].
			keyNum = 13 ifTrue: "cmdReturn" [
				self wannaProcreate.
				^ true
				""].
			keyNum = 28 ifTrue: [
				self goLeft.
				^ true
				"cmdArrowLeft"].
			keyNum = 29 ifTrue: [
				self goRight.
				^ true
				"cmdArrowRight"].
			keyNum = 30 ifTrue: [
				self goUp.
				^ true
				"cmdArrowUp"].
			keyNum = 31 ifTrue: [
				self goDown.
				^ true
				"cmdArrowDown"].
			char = $q ifTrue: [
				| textmorph | textmorph := meat textMorph.
				Transcript showTemp.
				Transcript show: textmorph paragraph printString, ', focused: ', textmorph paragraph focused printString; cr; cr.
				textmorph paragraph focused ifTrue: [textmorph backgroundColor: Color yellow] ifFalse: [textmorph backgroundColor: Color transparent].
				^ true].
			char = $s ifTrue: [
				self save: #global.
				^ true]].
		^ nil
		"no action from us, allow lower element to act"]! !

!GoHeader methodsFor: 'keys' stamp: 'miso 12/28/2004 10:59'!
keyStroke: evt
"when we are focus we are 'keyboardFocus' and thus receive any kb action. we look for cmd/cmdShift just like GoParagraphEditor.dispatchOnCharacter:with: does, but taking info from evt rather than sensor. If cmd/shiftCmd process it, else deselect self and pass it on to meat."

	| char dummyStream combo |
self halt.
	char := evt keyCharacter.
	dummyStream := WriteStream on: (String new: 10).
	combo := self keyStrokeIsModified: evt.
	(self keyStrokeIsSpecial: evt) ifTrue: [ "special key: enter, delete, tab, etc."
		^ self key: char combo: combo stream: dummyStream].
	combo ifNotNil: [ "modifier pressed: cmd, shiftCmd"
		^ self key: char combo: combo stream: dummyStream].

	"not a special char, deselect self, select meat, and pass evt to it"
	self askForMeatFocus: evt hand.
	^ meat textMorph keyStroke: evt
! !

!GoHeader methodsFor: '  core' stamp: 'miso 10/30/1999 17:10'!
keyboardFocusChange: aBoolean

	aBoolean
		ifTrue: ["A hand is wanting to send us characters..."
			self coreBecomeFocus]
		ifFalse: ["A hand has clicked elsewhere..."
			self coreStopBeingFocus]
! !

!GoHeader methodsFor: '  margins' stamp: 'ms 9/12/1998 16:38'!
leftMargin
	^ HeaderLeftMargin! !

!GoHeader methodsFor: 'saving' stamp: 'miso 10/25/1999 12:39'!
localSave
"this is adapted from the original content of acccept. Inform the model of text to be accepted, and return true if OK."

	meatIsDirty ifFalse: [ self flash ].
		"only happens when a local save is asked in clean header,
		in nonLocal cases the msg never gets from ancestor down to us"

	meat save.
	meatIsDirty _ false.! !

!GoHeader methodsFor: 'aboveMe' stamp: 'miso 10/25/1999 12:05'!
logicalOwner
"answer the real owner, whether visible or not. We are nevere 'hidden' so just give owner"

	^ owner! !

!GoHeader methodsFor: 'aboveMe' stamp: 'miso 10/25/1999 12:07'!
logicalOwner: aMorphOrNil
"set the real owner, whether visible or not. We are nevere 'hidden' so just set owner"

	owner _ aMorphOrNil! !

!GoHeader methodsFor: 'meat' stamp: 'miso 10/25/1999 12:41'!
meatColor: c
"for menu (can be used by toggleBreakpoint)"
	meat setColor:  c.! !

!GoHeader methodsFor: 'saving' stamp: 'ms 10/29/1998 15:38'!
meatIsDirty
	^ meatIsDirty! !

!GoHeader methodsFor: 'focus' stamp: 'miso 10/30/1999 17:09'!
meatLostFocus
"callback: give a chance to my translator to do stuff like automatic cleaning of meat (eg. for syntax of code)"

	self translator meatLostFocus! !

!GoHeader methodsFor: 'meat' stamp: 'miso 10/25/1999 12:40'!
meatMorph
"for menus to be able to set color"
	^ meat! !

!GoHeader methodsFor: 'general properties' stamp: 'miso 10/8/1999 21:44'!
meme
	^ self translator meme! !

!GoHeader methodsFor: 'meat' stamp: 'miso 10/30/1999 16:10'!
memeMeat
	^ self translator memeContent! !

!GoHeader methodsFor: 'meat' stamp: 'miso 10/30/1999 16:09'!
memeMeat: m
"set the text of the header, ignoring previous content. return whether was successful ('save' counts on it)."

	self translator memeContent: m.
	^ true! !

!GoHeader methodsFor: 'saving' stamp: 'ms 10/4/1998 18:33'!
needToSave
	^ self isDirty & self askBeforeDiscardingEdits ! !

!GoHeader methodsFor: 'cut&paste' stamp: 'miso 10/20/1999 14:20'!
paste
	"allow pasting of headers onto me to indicate adding headers after me"
	| pastee hHostData nextMom nextSisterBefore |
	pastee _ self coffreFort content.
	(pastee isKindOf: Meme) ifTrue: [
		hHostData _ self whereAddHeader.
		nextMom _ hHostData first meme.
		nextSisterBefore _ (hHostData second isKindOf: Symbol) ifTrue: [hHostData second] ifFalse: [hHostData second meme].
		pastee putSelfIntoHeader: nextMom afterSub: nextSisterBefore.
		^ true "indicate we handled it, whatever the outcome"
		].
	(pastee isKindOf: GoHeader) ifTrue: [ "should not happen anymore"
		hHostData _ self whereAddHeader.
		pastee wannaMoveSelfIntoHeader: hHostData first afterSub: hHostData second.
		^ true "indicate we handled it, whatever the outcome"
		].
	^ nil "if we are called from meat key-shortcut, allow ParaEditor to handle it"! !

!GoHeader methodsFor: 'aboveMe' stamp: 'ms 10/30/1998 11:27'!
positionInMom
	^ self headerAbove positionOfSubheader: self.! !

!GoHeader methodsFor: 'general properties' stamp: 'miso 10/25/1999 10:23'!
positionInWorld
"used by GoHeaderAlias during dnd"

	^ self boundsInWorld topLeft! !

!GoHeader methodsFor: 'subheaders' stamp: 'miso 10/29/1999 14:44'!
positionOfSubheader: aSubheader
	^ paneToHeaderLink positionOfSubheader: aSubheader.! !

!GoHeader methodsFor: 'meat' stamp: 'miso 10/25/1999 12:40'!
privCreateMeatBox
"(re)create my meatBox. add it to subMorph list, register it as meat."

	meat _ translator meatClass on: self "see fn1"
				text: #memeMeat 
				accept: #memeMeat:
				readSelection: nil
				menu: nil. "#buildGolgiMenu:given:"
	self addMorphBack: meat.
	^meat.

"addMorphFron.layoutChanged->releases para. change-> triggers unnecessary fullBounds calc on fake buonds layoutChanged: percolate to mom who causes recalc."
"fn.1		This call creates the view & registers the 'on' arg as its model. The other args are the names of the model-msgs which will be the makers of the described actions. The editor/controller is installed on the fly by model
"! !

!GoHeader methodsFor: '  subheaderPane' stamp: 'miso 10/24/1999 15:15'!
privSubheaderpaneClass
	^ GoHeadersPane! !

!GoHeader methodsFor: '  expansion' stamp: 'ms 10/4/1998 14:05'!
rawExpandSubheaders
"This is to programmatically show subheaders (as opposed to showing them after a wedge-click, we thus need to set the wedge state"

	self unhideSubheaders.
	self setWedgeExpanded.! !

!GoHeader methodsFor: 'saving' stamp: 'ms 7/4/1999 00:40'!
recursiveSave

	self meatIsDirty ifTrue: [ self localSave ].
	self subheaderPane ifNotNil: [self subheaderPane recursiveSave].
	^ true! !

!GoHeader methodsFor: 'saving' stamp: 'ms 7/3/1999 23:55'!
refreshFromMemes

	meatIsDirty ifTrue: [self refreshMeat. meatIsDirty _ false].

	self subsAreDirty ifTrue: [
		self subheaders do: [: h | h refreshFromMemes]]! !

!GoHeader methodsFor: 'meat' stamp: 'miso 7/4/1999 16:05'!
refreshMeat
	"used by translator when it changes our meme, or when escaping form discarded changes"

	self meatMorph refreshMeat.! !

!GoHeader methodsFor: 'wedge' stamp: 'miso 10/24/1999 15:01'!
refreshWedgeState
	self isExpanded ifTrue: [^self setWedgeExpanded].
	self hasSubheaders 
		ifTrue: [^self setWedgeContracted]
		ifFalse: [^self setWedgeInert]! !

!GoHeader methodsFor: '  core reorg subs' stamp: 'miso 10/29/1999 14:55'!
releaseOneSubheader: aHeader
"internal: called by the subheader during the process of moving/deleting itself" 

	self subheaderPane releaseOneSubheader: aHeader.
	self checkForEmptySubPane.
	^ true
! !

!GoHeader methodsFor: 'saving' stamp: 'miso 10/25/1999 12:38'!
resetMeme: m
"change the meme we are associated to (do not change the translator). crucial to allow editing of one script into another. 
Note that content of meme and ofo header might be out of sync aftr this, it is up to the caller to decide how to resync"

	(translator understandsThisMeme: m) 
		ifFalse: [^ self error: 'cannot change the meme this header is linked to because our meme-translator does not understand this meme'].

	translator meme: m! !

!GoHeader methodsFor: 'wedge' stamp: 'miso 10/25/1999 12:43'!
resetWedgeFromLeft: n
	wedge bounds: (wedge bounds withLeft: (self bounds left+ n))! !

!GoHeader methodsFor: '  margins' stamp: 'ms 9/12/1998 16:38'!
rightMargin
	^ HeaderRightMargin! !

!GoHeader methodsFor: 'saving' stamp: 'miso 12/29/2004 00:16'!
save: specialKey

	specialKey == #local ifTrue: [ ^ self localSave ].  "only local save"
	specialKey == #localWithSubs ifTrue: [ ^ self recursiveSave ]. "not activated"
	"specialKey == #global ifTrue: [ self bigBrother recursiveSave ]"

	^ self bigBrother recursiveSave. 	"save whole outline, even if bogus key"! !

!GoHeader methodsFor: 'general properties' stamp: 'miso 10/25/1999 12:41'!
setColor: c
"sets color for this header only. Called directly from menu"

	| theColor | 
	theColor _ (c isKindOf: String "orSymbol!!") 
					ifTrue: [Color perform: c]
					ifFalse: [c].
	self color: theColor.
	meat setColor: theColor.
! !

!GoHeader methodsFor: 'saving' stamp: 'miso 10/25/1999 12:41'!
setDirty

	"meatIsDirty ifFalse: [ if we've alredy been dirtied, don't bother"
		meatIsDirty _ true.
		meat setDirty. "for visual feedback"! !

!GoHeader methodsFor: 'meat' stamp: 'miso 10/25/1999 12:39'!
setHeaderContent: theContent
	"set the text of the header, ignoring previous content, 
	and returns ref to the contentContainerView"

	self memeMeat: theContent.
	meat refreshMeat.! !

!GoHeader methodsFor: 'meat' stamp: 'miso 10/25/1999 12:40'!
setMeatModel: theModel

	meat model: theModel.
! !

!GoHeader methodsFor: 'wedge' stamp: 'miso 10/25/1999 12:43'!
setWedgeContracted
	wedge jumpToState: #contracted.! !

!GoHeader methodsFor: 'wedge' stamp: 'miso 10/25/1999 12:43'!
setWedgeExpanded
	wedge jumpToState: #expanded.! !

!GoHeader methodsFor: 'wedge' stamp: 'miso 10/25/1999 12:42'!
setWedgeImage: i forState: s

	wedge state: s image: i
! !

!GoHeader methodsFor: 'wedge' stamp: 'miso 10/25/1999 12:42'!
setWedgeImageFamily: fDescriptor

	wedge imageFamily: fDescriptor! !

!GoHeader methodsFor: 'wedge' stamp: 'miso 10/25/1999 12:42'!
setWedgeInert

	wedge jumpToState: #inert.! !

!GoHeader methodsFor: 'meat' stamp: 'miso 10/25/1999 12:41'!
showSelectionBugWorkaround
	meat textMorph updateFromParagraph

"see comment in GoTranslateSmalltalk.notify:at:in:, where this is needed"! !

!GoHeader methodsFor: 'general properties' stamp: 'miso 10/29/1999 14:51'!
sisterAfter
	^ self headerAbove whatIsAfterSubheader: self! !

!GoHeader methodsFor: 'general properties' stamp: 'miso 10/29/1999 14:52'!
sisterBefore
	^ self headerAbove whatIsBeforeSubheader: self! !

!GoHeader methodsFor: '  subheaderPane' stamp: 'miso 10/25/1999 13:21'!
subheaderPane
"construct one if needed"

	paneToHeaderLink ifNil: [ 
		self subheadersNeedLoading ifTrue: [ "construct one, was in privAddSubheaderPane"
			paneToHeaderLink _ self privSubheaderpaneClass new.
			paneToHeaderLink newMom: self.
			self setWedgeContracted ]].
	^ paneToHeaderLink! !

!GoHeader methodsFor: 'subheaders' stamp: 'miso 10/8/1999 20:12'!
subheaders
	^ paneToHeaderLink 
		ifNil: [Array new]
		ifNotNil: [paneToHeaderLink subheaders]! !

!GoHeader methodsFor: 'subheaders' stamp: 'miso 10/24/1999 14:56'!
subheadersNeedLoading
"An early attempt to speedup: upon expanding subheaders, need to know if all are dressed, but want to avoid calculating each one of them... so cached a state 'subsAreLoaded'. This started being messy quickly, so reverted to checking sizes of respective daughters. An alternative is to keep a daughtersAlreadyDressed flag, with the assumptions that we dress them all only once, and they remained fully sync-ed ever after"

	^ self translator daughters size ~= self daughters size! !

!GoHeader methodsFor: 'saving' stamp: 'ms 7/3/1999 23:48'!
subsAreDirty
	self subheaders do: [:s| s isDirty ifTrue: [^true]].
	^ false! !

!GoHeader methodsFor: 'reorg self' stamp: 'miso 10/30/1999 15:40'!
tab
	Sensor leftShiftDown 
		ifTrue: [ ^ self goLeft]
		ifFalse: [ ^ self goRight].! !

!GoHeader methodsFor: '  margins' stamp: 'ms 9/12/1998 16:38'!
topMargin
	^ HeaderTopMargin! !

!GoHeader methodsFor: 'general properties' stamp: 'miso 10/25/1999 12:37'!
translator
	^ translator
! !

!GoHeader methodsFor: 'general properties' stamp: 'miso 10/25/1999 13:56'!
translator: a
	translator _ a.
	a header: self.

	"when plugging in a translator refresh the values that i get from it"
	self color: translator defaultColor.

	meat class = a meatClass ifFalse: [self privCreateMeatBox].
	meat textMorph quickContents: meat getText.

	self defaultWedge.
	self refreshWedgeState.! !

!GoHeader methodsFor: '  expansion' stamp: 'miso 1/10/2005 21:42'!
unhideSubheaders
	"This is the raw version to be used on startup"

	self subheadersNeedLoading
		ifTrue: [self dressSubheaders]
		ifFalse: [self isExpanded ifTrue: [^ self.]].

	submorphs := submorphs copyWith: paneToHeaderLink.
	paneToHeaderLink privateOwner: self.

	Golgi sniffer loadDaughtersOf: self.
! !

!GoHeader methodsFor: '  expansion' stamp: 'miso 1/10/2005 21:42'!
unhideSubheadersAndRefresh
"This is called from the wedge, or from boundsGivenMax, the wedge state is thus not to be changed"

	self subheadersNeedLoading
		ifTrue: [self dressSubheaders]
		ifFalse: [self isExpanded ifTrue: [^ self.]].

"This part used to be in an autonomous 'unhideSubheaders' verb (while we were 'seizeSubheaders'). But we ended up being the only caller of that berb as the distinction between hide/release faded."

"Transcript show: (Time millisecondsToRun: [
]) printString; cr.
"
	submorphs := submorphs copyWith: paneToHeaderLink.
	bigBrother layoutChanged.
	bigBrother fullBounds.
	bigBrother changed.
	"self topContainer //world// displayWorld.	we need all hte speed we can get, here"
	World displayWorld.
	paneToHeaderLink privateOwner: self.

	Golgi sniffer loadDaughtersOf: self.

"
meme daughters size ~~ 0 ifTrue: setWedgeContracted
background	 ProcessorScheduler fork At: priority 
Processor yield.	"

"	paneToHeaderLink preventiveSubDaughterDress.

			#Here should say: #needPreventiveLoading and upon fullBounds=nil
			should diplay and then do all preventive loading on morphs that require
			it."
	
! !

!GoHeader methodsFor: '  misc' stamp: 'miso 10/24/1999 15:26'!
walkHeadersWith: aOneArgumentBlock
"the block is evaluated with each (dressed) header in turn as the argument, except for bigBrother"

	aOneArgumentBlock value: self.
	self subheaders ifNotNil: [
		self subheaders do: [:s | s walkHeadersWith: aOneArgumentBlock]]! !

!GoHeader methodsFor: '  expansion' stamp: 'miso 7/5/1999 21:15'!
wannaContractSubheaders
"This is to programmatically hide subheaders (as opposed to hiding them after a wedge-click, we thus need to set the wedge state"

	self hideSubheaders.
	^ self setWedgeContracted.
! !

!GoHeader methodsFor: '  expansion' stamp: 'miso 7/5/1999 21:16'!
wannaExpandSubheaders
"This is to programmatically show subheaders (as opposed to showing them after a wedge-click, we thus need to set the wedge state"

	self unhideSubheadersAndRefresh.
	self setWedgeExpanded.! !

!GoHeader methodsFor: 'reorg self' stamp: 'ms 7/2/1999 15:47'!
wannaMoveSelfAfter: aHeader
"no symbol accepted as arguments here!! (We could though, interpreting them as mom_self mom.) Used in promoteSelf & createSister.
Ultimately, this is lexical sugar for 'headerPane addSubHeader:after:'"

	^ self wannaMoveSelfIntoHeader: (aHeader headerAbove) afterSub: aHeader
! !

!GoHeader methodsFor: 'reorg self' stamp: 'ms 7/2/1999 15:48'!
wannaMoveSelfBefore: aHeader
"This is slightly ugly, but helps us keep all the registering/dregistering in a single place, moveSelfIntoHeader:afterSub:, and all calls to add ourselves as call to addSubHeader:after:"

	^ self wannaMoveSelfIntoHeader: (aHeader headerAbove) afterSub: (aHeader sisterBefore)
! !

!GoHeader methodsFor: 'reorg self' stamp: 'miso 10/20/1999 12:08'!
wannaMoveSelfInto: aHeader
"Used for demoting: put at end of subheaders of aHeader."

	^ self wannaMoveSelfIntoHeader: aHeader  afterSub: #last
! !

!GoHeader methodsFor: 'reorg self' stamp: 'miso 10/25/1999 10:11'!
wannaMoveSelfIntoHeader: newContainerHeader afterSub: futureSisterDescriptor
"see older version for capacity of mving from a nonGolgi environment"

	(self translator headerWantsToMoveOutOf: self headerAbove
							into: newContainerHeader 
							afterSub: futureSisterDescriptor)   ifFalse: [^false].

	newContainerHeader wannaExpandSubheaders.
	self askForMeatFocus.
	self setDirty.
	^ true! !

!GoHeader methodsFor: 'reorg self' stamp: 'miso 1/2/2005 00:56'!
wannaProcreate
	"this is cmdReturn "

	| newMeme futureMom futureSister newHeader |
	futureMom := self whereAddHeader. "hack: really an array of {mom.sister}"
	futureSister := futureMom second.
	futureMom := futureMom first.

	newMeme := futureMom translator headerWantsADaughterAfter: futureSister.

	"header will be created by callback from meme-creating action. but focus must be dealt with
	here rather than in callback, coz only golg-initiated procreation changes focus."
	newHeader _ futureMom whichDaughterDressesThisMeme: newMeme.
	newHeader askForMeatFocus.
	^ newHeader! !

!GoHeader methodsFor: 'reorg self' stamp: 'miso 10/25/1999 12:38'!
wannaPutSelfIntoHeader: newContainerHeader afterSub: futureSisterDescriptor

	(translator headerWantsToGoInto: newContainerHeader afterSub: futureSisterDescriptor)
		ifFalse: [^false].

	newContainerHeader wannaExpandSubheaders.
	self setDirty.
	^ true! !

!GoHeader methodsFor: 'reorg self' stamp: 'miso 12/28/2004 22:41'!
wannaSuicide
"me and my meme want to leave the mom I'm in. We have no further business, since we only view meme under that particular mom. So I also want me and my translator to be detached from our meme (which might continue to live under some other mom)"

	| higherSister |
	
	"transfer focus to the survivor above me -- should be done by #whoInheritsFocus, check out why it doesn't work anymore"
	higherSister _ self sisterBefore.
	higherSister == #root
		ifTrue: [self headerAbove askForMeatFocus]
		ifFalse: [higherSister askForMeatFocus].

	"hara kiri"
	^ self translator headerWantsToGetOutOf: self headerAbove! !

!GoHeader methodsFor: 'drag&drop' stamp: 'ms 10/29/1998 23:32'!
wantsDroppedMorph: aMorph event: evt
	"Supports adding morphs by dropping."

	^ true! !

!GoHeader methodsFor: 'wedge' stamp: 'miso 10/25/1999 12:43'!
wedgeBackgroundColor: c
	wedge backgroundColor: c; changed.! !

!GoHeader methodsFor: 'subheaders' stamp: 'miso 10/30/1999 20:25'!
whatIsAfterSubheader: subHeaderOrSymbol

	^ self subheaderPane whatIsAfterSubheader: subHeaderOrSymbol! !

!GoHeader methodsFor: 'subheaders' stamp: 'miso 10/30/1999 20:29'!
whatIsBeforeSubheader: subHeaderOrSymbol

	^ self subheaderPane whatIsBeforeSubheader: subHeaderOrSymbol! !

!GoHeader methodsFor: 'general properties' stamp: 'ms 7/3/1999 00:08'!
whereAddHeader
	"somebody is wanting to add a header below me but does not know where. answer with the default location: as my first daughter if i am expanded, as my sister otherwise. Note that the answer needs two parts: #(mom futureSisterAbove)"
	self isExpanded
		ifTrue: [^ {self . #root}]
		ifFalse: [^ {self headerAbove. self}]! !

!GoHeader methodsFor: 'drag&drop' stamp: 'miso 10/25/1999 12:39'!
whereDoIDrop: cursorPoz
	"cursorPoz must be in local coordinates"
	| dropPoz  |
	dropPoz _ self boundsInWorld left @ meat boundsInWorld bottom.
	(self dropInside: cursorPoz)
		ifTrue: [
			^ dropPoz _ (dropPoz x + 40) @ dropPoz y]
		ifFalse: [
			^ dropPoz _ (dropPoz x + 5) @ dropPoz y]! !

!GoHeader methodsFor: 'subheaders' stamp: 'ms 7/1/1999 23:36'!
whichDaughterDressesThisMeme: aMemeOrSymbol
"answer whether any of my immediate daughters are viewing aMeme, and if so, which. I can also get a symbol descriptor such as #root, in which case I just send it back."

	(aMemeOrSymbol isKindOf: Symbol) ifTrue: [^ aMemeOrSymbol].
	self subheaders do: [:s| (s isThisYourMeme: aMemeOrSymbol) ifTrue: [^s]].
	^nil! !

!GoHeader methodsFor: 'subheaders' stamp: 'ms 7/2/1999 10:02'!
whoDressesThisMeme: aMeme
"answer whether any header in me are viewing aMeme, and if so, which"

	| dresser |
	(self isThisYourMeme: aMeme) ifTrue: [^ self].
	self subheaders do: [:s| (dresser _ (s whoDressesThisMeme: aMeme)) ifNotNil: [^ dresser ]].
	^nil! !

!GoHeader methodsFor: 'focus' stamp: 'miso 10/20/1999 13:34'!
whoInheritsFocus
"say who would inherit my focus if i were to die now. Return nil if there is no candidate, bigBro will deal with it"

	| momHeader myFocusPoz |
	momHeader _ self headerAbove.
	momHeader subheaders size = 1	"i am the only header, no sister can inherit, pass up"
		ifTrue: [^ momHeader = self bigBrother ifTrue: [nil] ifFalse: [momHeader]].
	myFocusPoz _  momHeader positionOfSubheader: self.
	myFocusPoz = 1
		ifTrue: [^ (momHeader headerAt: 2)]
		ifFalse: [^ (momHeader headerAt: myFocusPoz-1)].


"	THIS IS TO PUT FOCUS ON VACATED POSITION
	oldFocusPoz > oldMom subheaders size  
		ifTrue: [
			oldMom subheaders size = 0	
				ifTrue: [ oldMom becomeFocus: ourHand]
				ifFalse: [(oldMom headerAt:(oldFocusPoz-1)) becomeFocus: ourHand] ]
		ifFalse: [
			(oldMom headerAt: oldFocusPoz) becomeFocus: ourHand ]
"
! !

!GoHeader methodsFor: 'aboveMe' stamp: 'miso 10/25/1999 10:27'!
world
"I can be 'hidden', ie part of a headerpane which is still owned by a momheader but not currently in submorphs coz not currently shown. In those cases i don't have an 'owner' but i do have a world. What we really want to know is which world my golgi is in, if any."

	^ self bigBrother ifNil: [nil] ifNotNil: [self bigBrother world]! !

GoHeader subclass: #Golgi
	instanceVariableNames: 'lastFocus'
	classVariableNames: 'GolgiGlobals ScrollPaneColor'
	poolDictionaries: ''
	category: 'Golgi-Basics'!

!Golgi commentStamp: 'miso 1/12/2005 19:35' prior: 0!
On the instance side, I am the root pane of an outline view, the one which contains the top-level headers, the one which is typically embedded in a scrollPane. On the class side, I am the entrypoint for the Golgi packages - the main methods being: 

Golgi open
Golgi import: 'filename'
!

!Golgi class methodsFor: 'debugging' stamp: 'miso 6/11/2000 11:15'!
breakAllMemeDependents

	| glued |
	(Golgi allInstances select: [:g | g isInWorld ]) isEmpty
		ifFalse:  [^self error: 'all golgs must be closed before we break dependents'].

	glued _ Meme allInstances select: [:m | m dependents isEmpty not].
	glued do: [:m | m breakDependents].

	Transcript 
		show: 'there were ', glued size printString, ' memes with dependents.'; cr.! !

!Golgi class methodsFor: 'globals' stamp: 'miso 1/9/2005 00:24'!
checkGlobals

	GolgiGlobals ifNil: [GolgiGlobals := Dictionary new].
	GolgiGlobals at: #Prefs ifAbsent: [
		GolgiGlobals at: #Prefs put: GoPreferences new].
	GolgiGlobals at: #CoffreFort ifAbsent: [
		GolgiGlobals at: #CoffreFort put: GoCoffreFort new].
	GolgiGlobals at: #soups ifAbsent: [
		GolgiGlobals at: #soups put: OrderedCollection new].
	"generate arrow-forms and color pseudo wedges in codeHeaders"
	GoTranslator ensureBasicWedgeImages.
	
	"tentative attempts to set preferences here too"
	self prefs setPref: #useSniffer to: false.
! !

!Golgi class methodsFor: '  creating a viewer' stamp: 'mich 1/13/2005 00:06'!
dataInUnopenedWindow: data
"answer a window viewing data, but do not attach it to any world"

	| theGolg scrollPane |
	self prefs checkCriticalPrefs. "before creating anything"

	theGolg := self dress: data.
	self sniffer addBigBrother: theGolg.

	scrollPane _ ScrollPane new.
	scrollPane model: theGolg translator; "so menu-bldg is sent there"
				getMenuSelector: #buildGolgiMenu:given:.
	scrollPane scroller offset: 0@0.
	scrollPane scroller addMorph: theGolg.
	scrollPane setScrollDeltas. "shouldn't this be done automatically when add sub morph to a scrollpane??"

	^ SystemWindow new 
		 	"extent: 250 @ 300;" 
			model: theGolg translator;
			addMorph: scrollPane frame: (0 @ 0 corner: 1 @ 1);
			setLabel: theGolg memeMeat asString! !

!Golgi class methodsFor: 'debugging' stamp: 'miso 1/12/2005 18:18'!
debugMenu
"another bugging place to look at is the DependentsFields class var of Object, sometime holds on to strange things..:"

	| m |
	m _ MenuMorph new defaultTarget: self.
	m add: 'debugging stats' target: Golgi action: #debugStats.
	m add: 'all pointers to first golg' target: Golgi action: #pointersToFirstGolgi.
	m add: 'break meme dependents' action: #breakAllMemeDependents.
	m add: 'vider le sniffeur' action: #emptySniffer.
	^ m popUpInWorld
! !

!Golgi class methodsFor: 'debugging' stamp: 'miso 1/10/2005 21:45'!
debugStats

	Transcript cr;
		show: 'Golgi instances: ', Golgi allInstances size printString; cr;
		show: 'Header instances: ', GoHeader allInstances size printString; cr;
		show: 'Sniffers: ', GoSniffer allInstances size printString; cr;
		show: 'The sniffer is ', (Golgi sniffer isEmpty ifTrue: [''] ifFalse: ['not ']), 'empty'; cr;

	"		(GoSniffer sniffer mainBigBrother 
				ifNil: ['does not hold onto any Golgs']
				ifNotNil: ['is holding on to some Golgs']); cr; cr;"

		show: 'there are ', (Meme allInstances select: [:m | m dependents isEmpty not]) size printString, ' memes with dependents.'; cr;
		cr.! !

!Golgi class methodsFor: 'instance creation' stamp: 'mich 1/12/2005 23:13'!
dress: aMemeOrUrl

	^ self dress: aMemeOrUrl translatorClass: nil! !

!Golgi class methodsFor: 'instance creation' stamp: 'mich 1/12/2005 23:13'!
dress: aMemeOrUrl translatorClass: aMemeTranslatorClass

	| theBigBrother theMeme theTranslator |

	(aMemeOrUrl isKindOf: String) 
		ifTrue: [theMeme _ MemeSoup memeFromAddress: aMemeOrUrl]
		ifFalse: ["assume its a meme" theMeme _ aMemeOrUrl].

	theBigBrother _ aMemeTranslatorClass 
		ifNil: [GoTranslator createHeaderAndTranslatorFor: theMeme viewedInside: nil]
		ifNotNil: [
			theTranslator _ aMemeTranslatorClass meme: theMeme.
			theTranslator createYourHeaderGivenData: theMeme momHeader: nil].

	theBigBrother
			dressSubheaders; 
			checkForEmptySubPane;
			unhideSubheaders.

	^ theBigBrother

! !

!Golgi class methodsFor: 'debugging' stamp: 'miso 1/10/2005 21:47'!
emptySniffer

	| sniff |
	(self allInstances select: [:g | g isInWorld ]) isEmpty ifFalse:  [
		^self error: 'all golgs must be closed before we play with sniffer'].

	sniff := Golgi sniffer.
	sniff zapDaughtersToSniff.
	[sniff mainBigBrother == nil] whileFalse: [sniff removeBigBrother: sniff mainBigBrother]! !

!Golgi class methodsFor: 'golgi entrance points' stamp: 'miso 1/12/2005 18:10'!
import: filename
"recreate a golgi from an exported Meme graph in a file named filename inside the default directory, and ask if it should be added to our list of fridge"

| soup |
Cursor wait showWhile: [soup _ MemeSoup fromBackupFile: filename].
(self confirm: 'Do you want to add ''', soup contentString, ''' to the fridge?') ifTrue: [
	self soups addLast: soup].
self view: soup.
! !

!Golgi class methodsFor: '  prefs' stamp: 'miso 1/8/2005 23:50'!
initialize
" Golgi initialize "

	self checkGlobals.! !

!Golgi class methodsFor: 'golgi entrance points' stamp: 'mich 1/12/2005 23:38'!
open
"pop up an outliner, but first ask which outline should be viewed or whether a new one should be created (if there is none, just create one and view it)"

	self soups 
		ifEmpty: [self view: MemeSoup primordialSoup newOutline]
		ifNotEmpty: [MemeSoup primordialSoup worldMenu popUpInWorld]! !

!Golgi class methodsFor: 'debugging' stamp: 'miso 7/17/2000 08:45'!
pointersToFirstGolgi

	^ self someInstance 
		ifNil: [Transcript show: 'there is no Golgi around'; cr]
		ifNotNil: [(Smalltalk pointersTo: self allInstances first except: #()) inspect].
! !

!Golgi class methodsFor: '  prefs' stamp: 'miso 1/8/2005 23:58'!
prefs

^ GolgiGlobals at: #Prefs 
	"-- already set in #checkGlobals //  ifAbsent: [Globals at: #Prefs put: GoPreferences new]"! !

!Golgi class methodsFor: '  prefs' stamp: 'mich 1/12/2005 23:07'!
scrollPaneColor	
	ScrollPaneColor ifNil: [ScrollPaneColor _ Color transparent].
	^ ScrollPaneColor
! !

!Golgi class methodsFor: '  prefs' stamp: 'mich 1/12/2005 23:07'!
scrollPaneColor: c
	ScrollPaneColor _ c
! !

!Golgi class methodsFor: '  prefs' stamp: 'ms 9/12/1998 02:00'!
setColor: c updating: listOfViews
"reset the pref for color of top level, ie. scroller, and notify all existing scrollers of the change."

	ScrollPaneColor _ c.
	listOfViews ifNotNil: [ listOfViews do: [:v | v scrollPaneColor: c ]]! !

!Golgi class methodsFor: 'globals' stamp: 'miso 10/30/1999 15:26'!
setVersion

	| r |
	r _ self version.
	r _ FillInTheBlank request: 'New version:' initialAnswer: r.
	self version: r.! !

!Golgi class methodsFor: 'globals' stamp: 'miso 1/10/2005 21:52'!
sniffer

"(self prefs getPref: #useSniffer) ifFalse: [^nil]."

^ GoSniffer theOnlyInstance! !

!Golgi class methodsFor: 'globals' stamp: 'miso 1/9/2005 00:03'!
soups

^ GolgiGlobals at: #soups! !

!Golgi class methodsFor: 'golgi entrance points' stamp: 'miso 1/12/2005 19:36'!
start

	^ self open "kept for historical / backward compatibility reasons"! !

!Golgi class methodsFor: 'globals' stamp: 'miso 1/8/2005 23:59'!
version

	^ GolgiGlobals at: #version ifAbsent: [GolgiGlobals at: #version put: '0']! !

!Golgi class methodsFor: 'globals' stamp: 'miso 1/8/2005 23:59'!
version: newVersion

	^ GolgiGlobals at: #version put: newVersion! !

!Golgi class methodsFor: 'golgi entrance points' stamp: 'mich 1/12/2005 22:55'!
view: someData

	self checkGlobals.
	^ self  view: someData inWorld: World! !

!Golgi class methodsFor: '  creating a viewer' stamp: 'mich 1/12/2005 23:04'!
view: data inWorld: w
"open a window in world w viewing data"

	| theWin |
	theWin := self dataInUnopenedWindow: data.
	w ifNotNil: [theWin openInWorld: w].
	^ theWin! !

!Golgi class methodsFor: '  creating a viewer' stamp: 'mich 1/13/2005 00:39'!
viewNewOutlineIn: aSoup

^ self view: aSoup newOutline! !

!Golgi class methodsFor: 'golgi entrance points' stamp: 'mich 1/12/2005 23:15'!
viewUrl: url
"experimental -- do not use yet"

	^ self view: (self soups first submemeFromPath: url)! !

!Golgi methodsFor: 'asMainPane' stamp: 'miso 10/30/1999 16:17'!
OBSOLETEMaybeReinstantiateMemes
"link all headers to a fresh meme. set the content of the fresh meme to the content of the header. This is used to when user wants to keep changes without dirtying original memes. (Created for script editing when selector name is changed: old selctor content must remain, but new changes must be kept too.) "
	
	| theBlock |
	theBlock _ [: header |
		header resetMeme: header meme copy.
		header memeMeat: header headerMeat.
		header meme putSelfInto: (header owner 
									ifNotNil:[header headerAbove meme]
									ifNil:[nil]) 
					after: #last].

	translator meme: Meme new.
	self walkHeadersWith: theBlock.
	super recursiveSave


"		header resetMeme: (Meme addWithString: s headerMeat
							mom: (s owner 
									ifNotNil:[s headerAbove meme]
									ifNil:[nil])   
							after: #last)].
"
! !

!Golgi methodsFor: 'aboveMe' stamp: 'miso 10/28/1999 12:04'!
bigBrother
	^ self! !

!Golgi methodsFor: 'display' stamp: 'mich 10/3/1999 23:32'!
changed
"since our fullBounds calc recurses to us, we cannot do the standard 'invalidRect: self fullBounds', that would be a vicious circle. Both formulations below seem to yield equivalent results."

	self invalidRect: bounds	

"	fullBounds ifNotNil: [ self invalidRect: fullBounds ]	"
! !

!Golgi methodsFor: 'subheaders' stamp: 'miso 10/13/1999 10:29'!
checkForEmptySubPane
"make sure we always have at least one subheader header"

	self subheaders size = 0 ifTrue: [ 
		self dressSubmeme: (Meme string: '' mom: self meme after: #last)]! !

!Golgi methodsFor: 'aboveMe' stamp: 'miso 10/10/1999 11:26'!
container
	^ self scrollPane owner! !

!Golgi methodsFor: 'meat & wedge' stamp: 'miso 11/2/1999 17:11'!
content
"this is neutral between headerMeat and memeMeat. Interpreted as headerMeat if there is some, and memeMeat otherwise (eg in bigBrother)"
	^ self memeMeat! !

!Golgi methodsFor: 'save/close' stamp: 'miso 1/10/2005 21:44'!
detachFromdata
"this is used when the golgi pane is getting deleted. Called from sysWin.delete->translator(=model).release"

	super detachFromdata.
	Golgi sniffer viewClosingDown: self.! !

!Golgi methodsFor: 'display' stamp: 'ms 10/1/1998 13:41'!
displayWorld
	
"this is a hack to allow keeping displayWorld in expand subs, in order to speed it up. during construction of the golgi, we are the topmostMorph and this msg gets to us."! !

!Golgi methodsFor: 'display' stamp: 'miso 12/30/2004 00:51'!
fullBounds
| oldbounds |
"We always have 1-and-only-1 submorph: a subHeader pane. Also, we need to read bounds into fullBouds only after boundsGivenTopline: coz it will generate tons of layoutChanged which will reset our fullBounds to nil..."

	oldbounds_bounds.

	fullBounds ifNil: [
		submorphs isEmpty ifTrue: [self halt].
		fullBounds _ self boundsGivenTopline: bounds].

	"horrendous trick to get slider in scrollbar to update as we change"
	oldbounds=bounds ifFalse: ["self halt." owner owner setScrollDeltas] "extent:" .

	^ fullBounds.


		"a little trick to put golgi into a non sysWin mom: (use ownerChanged instead)"
"		owner ifNotNil: [
			bounds setOrigin: 0@0 corner: owner extent ]."

"		sBounds _ bounds.
		submorphs do: [:s| sBounds _ sBounds withTop: (s boundsGivenMax: sBounds)].
"! !

!Golgi methodsFor: 'focus' stamp: 'ms 7/4/1999 00:55'!
handlesMouseOver: evt
	^ true! !

!Golgi methodsFor: 'save/close' stamp: 'ms 7/4/1999 00:56'!
hasHiddenDirt
"do we have dirt that was discarded by the user"

	^ meatIsDirty! !

!Golgi methodsFor: 'aboveMe' stamp: 'mich 10/3/1999 23:26'!
headerAbove

	^ nil! !

!Golgi methodsFor: 'meat & wedge' stamp: 'miso 11/2/1999 17:12'!
headerMeat
"this asks for the content fo the pane, not of the meme. since we have none, return an empty string."
	^ self error: 'can i remove this now that ''content'' has been introduced?'! !

!Golgi methodsFor: 'creation' stamp: 'miso 10/25/1999 12:43'!
initialize

	super initialize.

	bigBrother _ self.
	wedge _ nil.
	submorphs _ Array new. "super put wedge there"
	color _ Color transparent "white".! !

!Golgi methodsFor: 'save/close' stamp: 'ms 7/3/1999 23:43'!
isDirty
"ignore meatIsDirty, i use that var to note when i contain discarded changes"
	^ self subsAreDirty 
! !

!Golgi methodsFor: 'display' stamp: 'miso 10/8/1999 20:13'!
isExpanded

	^ submorphs includes: paneToHeaderLink

"could just return false, as this is asked only when we first expand "! !

!Golgi methodsFor: 'focus' stamp: 'miso 6/14/2000 23:09'!
lastFoci
"keep multiple foci in anticipation of being able to handle multiple hands working on us. For now just send our unique focus, as list"

	^ lastFocus ifNil: [Array new] ifNotNil: [Array with: lastFocus]! !

!Golgi methodsFor: 'focus' stamp: 'ms 11/4/1998 01:01'!
lastFocus
	^ lastFocus! !

!Golgi methodsFor: 'focus' stamp: 'miso 12/29/2004 22:27'!
lastFocus: f

f ifNil: ["should check wether we have subs. but we always do..." 
	^ self mainHeader askForMeatFocus].

f == lastFocus ifTrue: [^self].
lastFocus ifNotNil: [lastFocus textMorph releaseEditor].
lastFocus := f! !

!Golgi methodsFor: 'display' stamp: 'miso 12/30/2004 01:16'!
layoutChanged
"just trigger recalc of subs by nil-ing fullBounds. no upwards percolation since I change size only when owner tells me to. no downwards since that is done in fullBounds recalc. "
"miso, 29/12/04 - added upwards percolation back, so that the scrollMorph containing me adjusts its scrollbar (which requires that my owning TransformMorph recalculates its localBounds after they get nilled out by #layoutChanged which I now send him)"

	fullBounds _ nil.
	owner ifNotNil: [owner layoutChanged].

	"gets called when:
	 I am being added, in addMorphFront:, before removing 
	me from old owner, so that I can patch up things there.
		Since OutlinerMorph and HeaderPane cannot be moved as such, 
		we do not care about that call. (headers do however, and they need
		tell their owner that they left.
	Something has just been added to me. Just triggering recalc is then ok.
	"
! !

!Golgi methodsFor: 'general properties' stamp: 'ms 9/12/1998 18:41'!
leftMargin
	^ 0! !

!Golgi methodsFor: 'subheaders' stamp: 'mich 10/3/1999 23:30'!
mainHeader
	^ self subheaders first! !

!Golgi methodsFor: 'focus' stamp: 'miso 10/30/1999 17:02'!
mouseEnter: event

	self lastFocus 
		ifNil: [self mainHeader askForMeatFocus: event hand]
		ifNotNil: [lastFocus askForFocus: event hand].! !

!Golgi methodsFor: 'aboveMe' stamp: 'miso 1/2/2005 20:27'!
ownerChanged
"make sure I stay as wide as my owner"

	owner width ~~ self width ifTrue: [
		bounds := bounds withWidth: owner width.
		fullBounds := nil ]
! !

!Golgi methodsFor: 'save/close' stamp: 'miso 10/25/1999 12:37'!
recursiveSave
"saving the whole golgi. first consult with translator that it is ok to save the full monty."

	self subsAreDirty ifFalse: [ self flash. ^ false ].
	(translator authoriseBigBrotherSave: self) ifFalse: [ ^ false ].
	^ super recursiveSave
		ifFalse: [^false]
		ifTrue: [meatIsDirty _ false. ^true]. "could reset 	meatIsDirty in an override of localSave"! !

!Golgi methodsFor: 'subheaders' stamp: 'ms 7/3/1999 23:59'!
refreshFromMemes
"called when someone cached us but fears that we still contain user-discarded dirt"
	self hasHiddenDirt ifFalse: [^self].
	self subheaders do: [: h | h refreshFromMemes].
	meatIsDirty _ false! !

!Golgi methodsFor: 'subheaders' stamp: 'miso 10/10/1999 10:51'!
refreshMeat
"if the content of my meme changes, it will call this (via translator.dataChanged). I don't care."! !

!Golgi methodsFor: 'asMainPane' stamp: 'miso 10/30/1999 16:09'!
renameGolgiTo: aName
	self translator memeContent: aName.
	(self scrollMorph owner isKindOf: SystemWindow) 
		ifTrue: [self scrollMorph owner setLabel: aName]! !

!Golgi methodsFor: 'asMainPane' stamp: 'mich 10/3/1999 23:15'!
scrollMorph
	^ owner owner! !

!Golgi methodsFor: 'asMainPane' stamp: 'ms 9/23/1998 14:25'!
scrollPane
	^ owner owner! !

!Golgi methodsFor: 'asMainPane' stamp: 'mich 1/12/2005 23:07'!
scrollPaneColor
	^ self class scrollPaneColor! !

!Golgi methodsFor: 'asMainPane' stamp: 'ms 9/23/1998 14:26'!
scrollPaneColor: c
	self scrollPane color: c! !

!Golgi methodsFor: 'asMainPane' stamp: 'miso 10/28/1999 12:07'!
setColor: c

	owner owner color: 
		((c isKindOf: String "orSymbol!!") 
			ifTrue: [Color perform: c]
			ifFalse: [c]).
! !

!Golgi methodsFor: 'meat & wedge' stamp: 'ms 8/16/1998 16:50'!
setWedgeContracted
! !

!Golgi methodsFor: 'meat & wedge' stamp: 'ms 8/17/1998 02:28'!
setWedgeExpanded
! !

!Golgi methodsFor: 'focus' stamp: 'miso 10/14/1999 09:02'!
submorphHasFocus: m

	^ lastFocus = m
	"^ self lastFoci includes: h"
! !

!Golgi methodsFor: 'general properties' stamp: 'miso 10/25/1999 13:57'!
translator: a
	translator _ a.
	a header: self.

	"do not update color/meat/wedge!!"

! !

!Golgi methodsFor: 'subheaders' stamp: 'miso 10/24/1999 15:25'!
walkHeadersWith: aOneArgumentBlock
"the block is evaluated with each (expanded) header in turn as the argument, except for bigBrother"

	self subheaders do: [:s | s walkHeadersWith: aOneArgumentBlock ]
! !

!Golgi methodsFor: 'aboveMe' stamp: 'miso 10/28/1999 12:05'!
world
"I inherit a modified version of this from GoHeader (asking me directly). Escape that, go back to normal version."

	^ owner == nil ifTrue: [nil] ifFalse: [owner world]! !

!GoHeadersPane class methodsFor: 'as yet unclassified' stamp: 'ms 11/1/1998 19:50'!
getPrefFor: prefSymbol

	^ self prefs at: prefSymbol ifAbsent: []! !

!GoHeadersPane class methodsFor: 'as yet unclassified' stamp: 'ms 8/16/1998 20:19'!
initialize
	PaddingBetweenSubTexts _ 0.
	LeftMargin _ 0.
	RightMargin _ 0.
	TopMargin _ 0.
	BottomMargin _ 0.
! !

!GoHeadersPane class methodsFor: 'as yet unclassified' stamp: 'miso 1/8/2005 23:54'!
prefs

^ Golgi prefs
! !

!GoHeadersPane class methodsFor: 'as yet unclassified' stamp: 'ms 11/1/1998 19:43'!
setPref: prefSymbol to: v

	^ self prefs at: prefSymbol put: v! !

!GoHeadersPane methodsFor: '  core reorg subs' stamp: 'miso 10/29/1999 14:40'!
addSubHeader: incomingHeader after: existingHeader
"The fundamental building block of header adding. Called both by h.reorgSelf verbs (from move/putSelfInto via h.addH) and by hp.dress verbs (from paneDress:withAccessorType:after:).
	This just registers a preexisting header, next redraw will make it visible if i am expanded. It has no influence on my expansion state. See comment in class comments for more details.
	It used to be called registerHeader:below: which has the advantage of being coherent with the general use of register/ deregister, but since it actually results in a adding of visual entities, add* was more intuitive. The register/ deregister pair is kept for non-visual adding.
	We count on caller having asked the permission to model to make this change!! 
 We do not check against bogus oldMorphs, be careful!!
 return index, or prevMeme?"

	| index |
	"submorphs _ submorphs copyWith: incomingHeader."
	incomingHeader newMom: self.

	index _ nil.
	existingHeader == #nil ifTrue: [ "just re-showing existing sub"
			index _ submorphs size ].
	existingHeader == #root ifTrue: [
			submorphs _ (Array with: incomingHeader), submorphs.
			index _ 1 ].
	existingHeader == #last ifTrue: [
			submorphs _ submorphs copyWith: incomingHeader.
			index _ submorphs size ].

	(index == nil) ifTrue: [
		(existingHeader isKindOf: Number) 
			ifTrue: [index _ existingHeader]
			ifFalse: [index _ submorphs indexOf: existingHeader].
		submorphs _ (submorphs copyFrom: 1 to: index), 
							(Array with: incomingHeader),
							(submorphs copyFrom: (index _ index + 1) 
								to: submorphs size) ].

	self layoutChanged.
	self changed.

	^ index! !

!GoHeadersPane methodsFor: 'aboveMe' stamp: 'miso 10/25/1999 12:18'!
bigBrother

	^ bigBrother
! !

!GoHeadersPane methodsFor: 'aboveMe' stamp: 'ms 11/23/1998 09:21'!
bigBrother: bb
	bigBrother _ bb.
	submorphs do: [:d| d bigBrother: bb]! !

!GoHeadersPane methodsFor: '  margins' stamp: 'ms 8/16/1998 15:37'!
bottomMargin
	^ BottomMargin! !

!GoHeadersPane methodsFor: '  margins' stamp: 'ms 9/12/1998 12:23'!
bottomMargin: n
	BottomMargin _ n! !

!GoHeadersPane methodsFor: 'priv' stamp: 'miso 10/25/1999 10:28'!
bottomRight: aPoint
"Little utility to set my corner to aPoint. Mimicks extent: which i want to avoid coz its unnecessary to translate to extent and back to corner :-) "

	bounds bottomRight = aPoint ifFalse: [
		self changed.
		bounds _ bounds topLeft corner: aPoint.
		self layoutChanged.
		self changed ].! !

!GoHeadersPane methodsFor: 'geometry' stamp: 'ms 11/7/1998 15:36'!
boundsGivenTopline: topLine
"computes the height we need given the topLine, which is a rect of which we only look at topBar. Meaning that 3 of our 4 coordinates are given, and we compute the fourth. We do that recursively, asking each subTextMorph how much height it needs. We answer with our new bounds, including the coordinate for our bottom." 

	| subBox nextBoxTopline |
	self position: topLine topLeft.

	nextBoxTopline _ (topLine left + self leftMargin)@topLine top + self topMargin
				corner: (topLine right - self rightMargin)@0.

	submorphs do: [: h |		"see fn.3"
		h isWedge ifFalse: [ "this verb is ls used by GoHeader"
			subBox _ h boundsGivenTopline: nextBoxTopline.
			nextBoxTopline _ subBox withTop: (subBox bottom + self paddingBetweenSubTexts)]].

	"subLine _ subBox withTop: subBox bottom + self bottomMargin. 
	self bottomRight: subLine topRight.		see fn.1"

	self bottomRight: topLine right @ (subBox bottom + self bottomMargin).

	^ bounds.


"
fn.1: One important thing here is that the verb updating our coordinates, should not end up calling 'privateFullMoveBy:' otherwise we waste cylces on relooping over our subMorphs which we have just done.
	Earlier way of doing it solved the craze thing without solving the cylce-waste problem (but did not need to introduce privatePosition!!) :
		self position: maxRect topLeft.  [was called at the beginning]
		self privateExtent: (maxRect right@topOfNextSub).  [at the end]
	Still earlier ways:
		privateBounds: (maxRect withBottom: topOfNextSub).	[w/out privateExtent:]
	Or
		fullBounds _ bounds _ (maxRect withBottom: topOfNextSub).
	Or
		// subBounds now indicates our bounds
		subBounds _ (maxRect withBottom: topOfNextSub).
		fullBounds _ bounds _ subBounds.
		self updateOtherSubs: subBounds.
		// this was to introduce a hook for other subMorphs, 
		// which turned out not to be needed. could be handy in the future.
	The advertised but rejected solution:
		self softBounds: (maxRect withBottom: topOfNextSub).  

fn.3	All of this would be so much simpler with 'break'... 
		break would also avoid unnecessary looping which will be important when have very long subheader lists

		The earlier version (incorrect):

	meatOrSubheaders do: [: subText |
		(subBoundsMax bottom > topOfNextSub) ifTrue:
		[
		topOfNextSub _	(subText boundsGivenMax: subBoundsMax) 
			+ self paddingBetweenSubTexts.
		subBoundsMax _ subBoundsMax withTop: topOfNextSub.
		]

fn4. call 'position:' before recursive step. If our subMorphs get positioned first, our repositioning will preserve diff :-(. Gets into muddles at startup when everybody has crazy bounds.

"! !

!GoHeadersPane methodsFor: 'geometry' stamp: 'ms 9/6/1998 03:06'!
changed

owner changed

"	self invalidRect: bounds	"	"withBottom: bigBrother bounds bottom)"

"	self isRoot
		ifFalse: [ owner changed ]
		ifTrue:
			[self invalidRect: (self fullBounds withBottom: bigBrother bounds bottom)] "


" owner subChanged: self. See comments there"

"self invalidRect: (self fullBounds withBottom: mom bottom)"! !

!GoHeadersPane methodsFor: 'geometry' stamp: 'miso 10/9/1999 12:31'!
fullBounds
	"do not start a recalc, recalcs are always initiated by bigBrother, coz they 
	might always influence some higher-level header. Just assume the 
	higher recalcs are being done correctly :-) "

	fullBounds ifNil: [ fullBounds _ bounds].
	^ fullBounds
! !

!GoHeadersPane methodsFor: 'prefs' stamp: 'ms 11/1/1998 20:10'!
getPrefFor: prefSymbol
	^ self class getPrefFor: prefSymbol! !

!GoHeadersPane methodsFor: 'aboveMe' stamp: 'ms 7/3/1999 17:23'!
header

	^ self owner! !

!GoHeadersPane methodsFor: 'aboveMe' stamp: 'miso 10/8/1999 20:43'!
headerAbove
	paneToHeaderLink ifNil: [^ self error: 'cannt give you owner, i have no more mom'].
	"temp"
	^ paneToHeaderLink! !

!GoHeadersPane methodsFor: 'subheaders' stamp: 'ms 11/7/1998 15:44'!
headerAt: SymbolOrHeader
"Changed to always return headers, this now acts as a symbol-filter."

	| index |
"	(SymbolOrHeader isKindOf: Symbol) ifFalse: [^ SymbolOrHeader ]. "

	index _ SymbolOrHeader.
	SymbolOrHeader == #root ifTrue: [ index _ 1 ].
	SymbolOrHeader == #last ifTrue: [ index _ submorphs size ].

	^ self headerAtIndex: index! !

!GoHeadersPane methodsFor: 'subheaders' stamp: 'ms 11/7/1998 15:45'!
headerAtIndex: index

	index == 0 ifTrue: [ ^ #root ].  "this is ugly, but seems needed in 'de/pro-moteSelf' "
	index == (submorphs size + 1) ifTrue: [ ^ #end ].
	^ submorphs at: index ifAbsent: [ ^ self error: 
				 				'asking for a non-existing header :-(' ]! !

!GoHeadersPane methodsFor: 'priv' stamp: 'ms 7/3/1999 23:59'!
initialize

	super initialize.

	PaddingBetweenSubTexts ifNil: [ "clunky way of telling Class not inited"
		GoHeadersPane initialize ].

	borderWidth _ 0.	"override sysWin settings"
	color _ Color transparent.

! !

!GoHeadersPane methodsFor: 'saving' stamp: 'ms 7/3/1999 23:43'!
isDirty
	^ self subsAreDirty! !

!GoHeadersPane methodsFor: 'general properties' stamp: 'ms 8/23/1998 14:20'!
isHidden
"tests whether the submorph/owner link is in, ie whether drawing occurs"

	^ owner ~~ nil

! !

!GoHeadersPane methodsFor: 'aboveMe' stamp: 'ms 8/17/1998 23:00'!
isRoot
	^ owner == self bigBrother! !

!GoHeadersPane methodsFor: 'general properties' stamp: 'ms 11/7/1998 13:11'!
isWedge
	^ false! !

!GoHeadersPane methodsFor: 'geometry' stamp: 'miso 10/9/1999 12:31'!
layoutChanged
"just percolate the msg up to bigBrother, and leave a trace telling we need to collect new bounds (can serve as a sign telling change originated from this branch"

	fullBounds _ nil.
	owner ifNotNil: [owner layoutChanged].
! !

!GoHeadersPane methodsFor: '  margins' stamp: 'ms 8/16/1998 15:38'!
leftMargin
	^ LeftMargin! !

!GoHeadersPane methodsFor: 'aboveMe' stamp: 'miso 10/25/1999 12:04'!
logicalOwner
"the 'owner' instVar only gives the visual owner, and returns nil if we are hidden. We give the real owner, whether visible or not"

	^ paneToHeaderLink! !

!GoHeadersPane methodsFor: 'aboveMe' stamp: 'miso 10/25/1999 12:08'!
logicalOwner: aMorphOrNil
"set the real owner, whether we're visible in it or not"

	paneToHeaderLink _ aMorphOrNil! !

!GoHeadersPane methodsFor: 'aboveMe' stamp: 'miso 10/25/1999 12:13'!
newMom: aMorphOrNil
"this changes the hierarchy, it is *not* for visual effects. It should *always* be accessed by setter of aMorphOrNil's logical daughter, so that we keep in sync"

	self logicalOwner: aMorphOrNil.
	aMorphOrNil 
		ifNil: [self bigBrother: aMorphOrNil]
		ifNotNil: [self bigBrother: aMorphOrNil bigBrother].! !

!GoHeadersPane methodsFor: '  margins' stamp: 'miso 10/25/1999 10:34'!
paddingBetweenSubTexts
	^ PaddingBetweenSubTexts! !

!GoHeadersPane methodsFor: 'subheaders' stamp: 'miso 10/29/1999 14:44'!
positionOfSubheader: aHeader

	^ submorphs indexOf: aHeader! !

!GoHeadersPane methodsFor: 'prefs' stamp: 'miso 10/16/1999 07:34'!
prefs
	^ self class prefs! !

!GoHeadersPane methodsFor: 'printing' stamp: 'miso 11/2/1999 17:11'!
printOn: aStream
	"used in inspectors, debuggers, etc."

	super printOn: aStream. aStream nextPutAll: ' for: ', 
		(self header ifNotNil: [self header content asString] ifNil: ['<nada>']).! !

!GoHeadersPane methodsFor: 'saving' stamp: 'ms 3/6/1999 14:06'!
queryForSave
"answer whether 'needToSave'. 
(PopUpMenu/SelectionMenu directly return the result of the action of their menuItems. MenuMorph/GoMenu MenuMorph invokeItem:)"

"
	| m |
	m _ GoMenu newUpOn: self.
	m		add: 'save' action: #recursiveSave;
			add: 'throw away' action: #discard;
			add: 'do nothing' action: #cancel;
		addTitle: 'Changes have not been saved. 
What do you want to do with them?'.
	^ self popUpMenuCarefully: m
"
	^ (SelectionMenu 
			labelList: #('save' 'throw away' 'do nothing')
			selections: #(save discard cancel))
				startUpWithCaption: 'Changes have not been saved. 
What do you want to do with them?'.
! !

!GoHeadersPane methodsFor: 'saving' stamp: 'ms 7/3/1999 23:59'!
recursiveSave
"header panes are irrelevant to saving proper, we just redistribute the msg. 
Note that if we implemented this as 'save: #symbol' which passes the same down, it would allow us to save just our subheaders, one level down. any use for that?"

	submorphs do: [: s | s recursiveSave ]! !

!GoHeadersPane methodsFor: '  core reorg subs' stamp: 'miso 10/29/1999 14:41'!
releaseOneSubheader: aHeader
"This is the basic building block of header releasing, called from h.moveSelf and h.deleteSelf (via mom-h.releaseOneSubheader:). Do not delete it, it might be moved elsewhere. Just deregister it from our subs."
"##should be called deregister to be coherent"

	submorphs _ submorphs copyWithout: aHeader.
	aHeader zapOwner.

	self changed.
	self layoutChanged.
! !

!GoHeadersPane methodsFor: '  margins' stamp: 'ms 8/16/1998 15:38'!
rightMargin
	^RightMargin! !

!GoHeadersPane methodsFor: 'prefs' stamp: 'ms 11/1/1998 20:11'!
setPref: prefSymbol to: v

	^ self class setPref: prefSymbol to: v! !

!GoHeadersPane methodsFor: 'subheaders' stamp: 'miso 10/29/1999 14:56'!
subheaders
	^ submorphs! !

!GoHeadersPane methodsFor: '  margins' stamp: 'ms 8/24/1998 10:01'!
topMargin
	^ TopMargin! !

!GoHeadersPane methodsFor: 'subheaders' stamp: 'miso 10/29/1999 14:53'!
whatIsAfterSubheader: prevSubHeaderOrSymbol

	(prevSubHeaderOrSymbol isKindOf: Symbol) ifFalse: [
		^ self headerAtIndex: (submorphs indexOf: prevSubHeaderOrSymbol) + 1 ].
	prevSubHeaderOrSymbol == #root ifTrue: [
		^ self headerAtIndex: 1 ].
	prevSubHeaderOrSymbol == #last ifTrue: [
		^ submorphs size ]. "treat this always in headerAfterIndex"

^ self error: 'could not find header after ', prevSubHeaderOrSymbol printString, '  because I could not recognise what this is... '.


"	(nextSubHeaderOrSymbol isKindOf: Symbol) 
		ifTrue: [ ^ self headerBeforeIndex: nextSubHeaderOrSymbol ]
		ifFalse: [ ^ self headerBeforeIndex: 
							(submorphs indexOf: nextSubHeaderOrSymbol) ] "! !

!GoHeadersPane methodsFor: 'subheaders' stamp: 'miso 10/29/1999 14:53'!
whatIsBeforeSubheader:   nextSubHeaderOrSymbol

	(nextSubHeaderOrSymbol isKindOf: Symbol) ifFalse: [
		^ self headerAtIndex: (submorphs indexOf: nextSubHeaderOrSymbol) - 1 ].
	nextSubHeaderOrSymbol == #root ifTrue: [
		^ self error: 'asked for element before root, should never arise' ].
	nextSubHeaderOrSymbol == #last ifTrue: [
		^ self headerAt: submorphs size - 1 ].

^ self error: 'could not find header before ', nextSubHeaderOrSymbol printString, '  because I could not recognise what this is... '.


"	(nextSubHeaderOrSymbol isKindOf: Symbol) 
		ifTrue: [ ^ self headerBeforeIndex: nextSubHeaderOrSymbol ]
		ifFalse: [ ^ self headerBeforeIndex: 
							(meatOrSubheaders indexOf: nextSubHeaderOrSymbol) ] "! !

!GoHeadersPane methodsFor: 'aboveMe' stamp: 'miso 10/8/1999 20:04'!
zapOwner

	owner _ nil! !

EllipseMorph subclass: #GoHeaderAlias
	instanceVariableNames: 'target seducteur'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Golgi-Basics'!

!GoHeaderAlias commentStamp: 'miso 1/12/2005 19:55' prior: 0!
I stand for a header during a drag&drop operation, providing visual feedback to the user during the operation. I turn red if the current drop site is forbidden (eg. within myself) and yellow otherwise.

[Part of my technical raison d'etre is that dragged elements seem to be expected to have world coordinates, but GoHeaders have transformed coordins.]!

!GoHeaderAlias class methodsFor: 'all' stamp: 'miso 10/25/1999 10:22'!
target: aHeader

	^ self new 
		target: aHeader;
		position: aHeader positionInWorld! !

!GoHeaderAlias class methodsFor: 'all' stamp: 'miso 11/25/2000 19:47'!
target: aHeader hand: h
	| a |
	a _ self target: aHeader.
	h addEventListener: a.
	^a! !

!GoHeaderAlias class methodsFor: 'all' stamp: 'ms 12/23/1998 23:05'!
visualiseDragOf: aHeader fromEvent: evt
	| a newPoz |
	a _ self new target: aHeader.
	newPoz _ aHeader positionInWorld.
	a position: (newPoz x +10) @ (newPoz y +15).
	"a zieutezLeSeducteur: evt"
	a beGrabbedBy: evt hand.
	^ a! !

!GoHeaderAlias methodsFor: 'drag&drop' stamp: 'miso 11/25/2000 19:47'!
beGrabbedBy: aHand
	"aHand grabMorph: self"
	World addMorph: self.
	aHand addEventListener: self! !

!GoHeaderAlias methodsFor: 'drag&drop' stamp: 'ms 10/30/1998 15:44'!
handleEvent: evt
	evt isMouseMove ifTrue: [^ self mouseMove: evt].
	evt isMouseUp ifTrue: [^ self mouseUp: evt]! !

!GoHeaderAlias methodsFor: 'drag&drop' stamp: 'miso 5/20/2001 15:17'!
handleListenEvent: evt
	"self halt. 2988port. is this needed as a replacement of handleEvent?"
	evt isMouseMove ifTrue: [^ self mouseMove: evt].
	evt isMouseUp ifTrue: [^ self mouseUp: evt]! !

!GoHeaderAlias methodsFor: 'private' stamp: 'miso 10/7/1999 12:40'!
initialize
	super initialize.
	self extent: 16 @ 6.
	self color: Color red! !

!GoHeaderAlias methodsFor: 'drag&drop' stamp: 'ms 12/23/1998 16:04'!
mouseMove: evt
	"this is to follow a drag operation"
	self zieutezLeSeducteur: evt! !

!GoHeaderAlias methodsFor: 'drag&drop' stamp: 'miso 11/25/2000 19:47'!
mouseUp: evt
	"this is to terminate a drag operation"
	evt hand removeEventListener: self.
	self delete "really - 'deregister' ".
	seducteur
		ifNil: [
			target askForMeatFocus: evt hand "put away focus on whole header"]
		ifNotNil: [
			seducteur acceptDroppingMorph: target event: evt.
			target justDroppedInto: seducteur event: evt]! !

!GoHeaderAlias methodsFor: 'private' stamp: 'ms 10/29/1998 23:53'!
target
	^ target! !

!GoHeaderAlias methodsFor: 'private' stamp: 'ms 10/30/1998 14:52'!
target: t
	target _ t! !

!GoHeaderAlias methodsFor: 'drag&drop' stamp: 'miso 10/8/1999 21:59'!
zieutezLeSeducteur: evt
	"called while dragging, to find out who would be the current attractor"
	| newPoz localCursorCoord  |

	"if cursor is over myself i obscure the header under me (and target would not change anyway)"
	(self containsPoint: evt cursorPoint) ifTrue: [ ^ self ].	

	seducteur _ evt hand dropTargetFor: self event: evt.
	(seducteur isKindOf: GoHeader) 	
		ifFalse: ["temp attempt to stop dropping on world, 057p9"
			seducteur _ nil. 
			newPoz _ target boundsInWorld topLeft.
			newPoz _ (newPoz x +10) @ (newPoz y +15).
			color _ Color red] 
		ifTrue: [
			localCursorCoord _ (evt transformedBy: (seducteur transformFrom: World)) cursorPoint.
			seducteur _ seducteur doYouWannaDelegateDropeeAt: localCursorCoord "ask header if drop goes into one of its subheader (ie mouse is in its margin)".
			(seducteur isInside: target) 
				ifTrue: [
					color _ Color red.
					"newPoz _ target boundsInWorld topLeft."
					newPoz _ seducteur boundsInWorld topLeft.
					newPoz _ (newPoz x +10) @ (newPoz y +15).
					seducteur _ nil]
				ifFalse: [
					color _ Color yellow.
					newPoz _ seducteur whereDoIDrop: localCursorCoord "ask header if i shouold shoow as sister or daughter poz".
					newPoz _ newPoz x @ (newPoz y - (self height / 2))].
			self position: newPoz
			"do we want to calc height wrt seducteur? to see if go above/below"]! !

TestCase subclass: #TestGolgi
	instanceVariableNames: 'golg golgWindow'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Golgi-Tests'!

!TestGolgi commentStamp: 'miso 1/12/2005 19:06' prior: 0!
Just a couple of high-level preliminary tests ensuring that the basics of the package work. Intended to be gradually enriched as the early (over-complicated) code gets cleaned up.!

!TestGolgi methodsFor: 'as yet unclassified' stamp: 'miso 1/2/2005 00:34'!
setUp

golgWindow _ Golgi view: MemeSoup new.
golg _ golgWindow paneMorphs first scroller submorphs first.
! !

!TestGolgi methodsFor: 'as yet unclassified' stamp: 'miso 1/2/2005 00:41'!
tearDown

golg recursiveSave.
golg _ nil.
golgWindow delete.
! !

!TestGolgi methodsFor: 'as yet unclassified' stamp: 'miso 1/4/2005 20:48'!
testChangeWindowName
"test that changing the label of a golgi window also changes the content of its root node. We need to mimick #relabel so as to avoid the user interaction asking for the new label"

| newLabel |
newLabel _ 'newGolgiName'.
(golg translator "= golgWindow model" windowReqNewLabel: newLabel) ifTrue: [
	golgWindow setLabel: newLabel].
self assert: golg memeMeat = golgWindow label.! !

!TestGolgi methodsFor: 'as yet unclassified' stamp: 'miso 1/2/2005 00:56'!
testCreateHeaders

| header |
self assert: golg subheaders size == 1. 		"golgi should be created with one empty header"
header := golg subheaders first.

header wannaProcreate.
self assert: golg subheaders size == 2.
header := golg subheaders second.
self assert: golg lastFocus == header meatMorph.				"make sure that the new header has focus according to Golgi"
"self assert: World primaryHand keyboardFocus == header meatMorph.	make sure the hand agrees with that"

header := header wannaProcreate.
"header := header subheaders first."
header goRight.
self assert: golg subheaders size == 2.

! !

TestCase subclass: #TestMeme
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Golgi-Tests'!

!TestMeme commentStamp: 'miso 1/12/2005 19:06' prior: 0!
Tests for the Node, Meme and MemeSoup classes. Very basic for now.!

!TestMeme methodsFor: 'as yet unclassified' stamp: 'miso 1/12/2005 11:25'!
testBackup

| soup a1 a2 b fn restored |
soup _ MemeSoup withMeat: 'a test soup'.
(a1 _ Meme withMeat: 'a1') putSelfInto: soup after: #last.
(b _ Meme withMeat: 'b') putSelfInto: a1 after: #last.

"now try a trivial backup and restore. The #backup operation stores statistics about the meme that was backed up and #fromBackupFile: compares those statistics to the newly recreated meme -- if the two do not match, it throws an error."
fn _ soup backup.
restored _ MemeSoup fromBackupFile: fn.		"this contains a check for newMeme.stats = oldMeme.stats"
self assert: soup content = restored content.		"but do check that the root of the meme-graph has the same content, that eludes that stats-based check"
"self assert: soup = restored."
FileDirectory default deleteFileNamed: fn.

"now create a situation where b is has two moms"
(a2 _ Meme withMeat: 'a2') putSelfInto: soup after: #last.
b putSelfInto: a2 after: #last.

"try a backup in this situation"
fn _ soup backup.
MemeSoup fromBackupFile: fn.
FileDirectory default deleteFileNamed: fn.

"backup a domain in which b's secondary moms is inexistant"
fn _ a1 backup.
MemeSoup fromBackupFile: fn.
FileDirectory default deleteFileNamed: fn.

"backup a domain in which b's primary moms is inexistant"
fn _ a2 backup.
MemeSoup fromBackupFile: fn.
FileDirectory default deleteFileNamed: fn.

"try backup with jumppoints"
(b _ Meme withMeat: 'b2') putSelfInto: a1 after: #last.
soup addJumpPoint: b.
fn _ soup backup.
MemeSoup fromBackupFile: fn.
FileDirectory default deleteFileNamed: fn.

"and with a meme whose first mom is a jumpoint -- this was a major bug on backup/restore"
b _ Meme withMeat: 'b2'.
soup addJumpPoint: b.
b putSelfInto: a1 after: #last.
fn _ soup backup.
MemeSoup fromBackupFile: fn.
"FileDirectory default deleteFileNamed: fn."
! !

!TestMeme methodsFor: 'as yet unclassified' stamp: 'miso 1/12/2005 11:16'!
testBackupPreservesStyle

| soup header runsBefore fn restored runsAfter |
"create a simple bold-styled meme"
soup _ MemeSoup new.
header _ Meme withMeat: 'smalltalkers do: [:it | with: Style]'.
header putSelfInto: soup after: #last.
header content addAttribute: TextEmphasis bold from: 6 to: 13.
runsBefore _ header content runs values size.

"back it up and restore it"
fn _ soup backup.
restored _ MemeSoup fromBackupFile: fn.		"this automatically checks for newMeme = oldMeme via meme.stats comparison"
FileDirectory default deleteFileNamed: fn.

"now check that the style survived the restore"
restored _ restored daughters first.
runsAfter _ restored content runs values size.
self assert: runsBefore =  runsAfter.

"now do the same with more complex styles"
header content addAttribute: TextFontChange font4 from: 14 to: 16.
header content addAttribute: TextEmphasis struckOut from: 1 to: 5.
header content addAttribute: TextEmphasis italic from: 31 to: 35.

"again, back up, restore and check"
runsBefore _ header content runs values size.
fn _ soup backup.
restored _ MemeSoup fromBackupFile: fn.
FileDirectory default deleteFileNamed: fn.
restored _ restored daughters first.
runsAfter _ restored content runs values size.
self assert: runsBefore =  runsAfter.
! !

!TestMeme methodsFor: 'as yet unclassified' stamp: 'miso 1/11/2005 11:51'!
testWalking
"test that the methods walking daughters do it correctly. Do that by harcoding one recursive method and then doing the same thing through a block walked on all daughters and check that we get the same result. There is already a convenient pair of methods like that, both for walking daughters and for walking primary daughters - namely the daughter-counting methods"

| soup a1 a2 b |
"first create a simple meme-graph where the number of daughers is not the same as the number of primary daughers"
soup _ MemeSoup new.
(a1 _ Meme withMeat: 'a1') putSelfInto: soup after: #last.
(a2 _ Meme withMeat: 'a2') putSelfInto: soup after: #last.
(b _ Meme withMeat: 'b') putSelfInto: a1 after: #last.
b putSelfInto: a2 after: #last.

self assert: soup countPrimaryDaughtersRecursively == 3.
self assert: soup countPrimaryDaughtersByWalking == 3.
self assert: soup countDaughtersRecursively == 4.
self assert: soup countDaughtersByWalking == 4.
! !

PluggableTextMorph subclass: #GoMeat
	instanceVariableNames: 'maxHeight rectCache widthCache'
	classVariableNames: 'MeatColor'
	poolDictionaries: ''
	category: 'Golgi-Basics'!

!GoMeat commentStamp: 'miso 1/12/2005 19:49' prior: 0!
I represent the typical visual content of a header: an editable text of arbitrary size, with arbitrary (multiple) style(s). I grow/shrinks vertically as my text grows/shrinks. 
!

!GoMeat methodsFor: 'saving' stamp: 'ms 8/18/1998 12:54'!
accept
"This is standard API, we receive it from kb on cmdS. User wants to save. Inform the header of these intentions, and let him take appropriate action. This gets called whenenever cmdS is hit in a textMorph. Note that cmdShiftSave doesn't get to us at all, it gets sent to search: which has been overriden to directly call header."

	owner save: #global.! !

!GoMeat methodsFor: 'topic/focus' stamp: 'ms 11/4/1998 01:10'!
askForFocus

	^ self askForFocus: self primaryHand.! !

!GoMeat methodsFor: 'topic/focus' stamp: 'miso 12/30/2004 11:12'!
askForFocus: hand

	hand newKeyboardFocus: textMorph.! !

!GoMeat methodsFor: 'geometry' stamp: 'ms 8/29/1998 04:22'!
boundsGivenMax1: maxRect
	| ourTempBottom newRect h |
	maxHeight _ maxRect height.

	h _ self setWidthGetHeight:  maxRect width. 
			"also calls extent: which resets all bounds "
	ourTempBottom _ maxRect top + h.

	(ourTempBottom >= maxRect bottom)
		ifTrue: [
			(maxRect top + self minHeight >= maxRect bottom)
				ifTrue: [ "we cannot squeeze into there..."
					newRect _ nil.
					ourTempBottom _ nil.
					^ nil ]
				ifFalse: [ "just show whatever we can..."
					ourTempBottom _ maxRect bottom.
					newRect _ maxRect withBottom: ourTempBottom. ].
		]
		ifFalse: [
			newRect _ maxRect withBottom: ourTempBottom. ].

	(newRect = rectCache) ifFalse: [
		rectCache _ newRect.
		self bounds: newRect ].

	^ ourTempBottom.
! !

!GoMeat methodsFor: 'geometry' stamp: 'ms 8/29/1998 04:24'!
boundsGivenMax: maxRect
	| ourTempBottom newRect h |
	maxHeight _ maxRect height.

	h _ self setWidthGetHeight:  maxRect width. 
			"also calls extent: which resets all bounds "
	ourTempBottom _ maxRect top + h.
	newRect _ maxRect withBottom: ourTempBottom.

	(newRect = rectCache) ifFalse: [
		rectCache _ newRect.
		self bounds: newRect ].

	^ ourTempBottom.
! !

!GoMeat methodsFor: 'geometry' stamp: 'miso 12/28/2004 22:02'!
boundsGivenTopline: topLine

	| newRect |

	newRect := topLine withBottom: topLine top + (self setWidthGetHeight: topLine width).

	(newRect = rectCache) ifFalse: [
"Transcript show: (Time millisecondsToRun: 
			[
owner bigBrother ubiq ifNotNil: [owner bigBrother ubiq: owner bigBrother ubiq+1]."
		rectCache := newRect.
		self bounds: newRect
"] ) printString, ' - '."
 ].

	^ bounds
! !

!GoMeat methodsFor: 'admin' stamp: 'ms 11/3/1998 17:23'!
coffreFort
	"return a reference to the pasteBuffer we use in common with GoHeader"
	^ GoCoffreFort! !

!GoMeat methodsFor: 'experimental' stamp: 'ms 8/28/1998 09:22'!
copyVars

	self privateBounds: (bounds origin corner: bounds corner).  
			"deep-copy bounds"

	scroller _ scroller copy.
	scroller privateOwner: self.
	"transform?"
	self privateSubmorphs: (Array with: scroller).

	textMorph _ textMorph copy.
	textMorph privateOwner: scroller.
	textMorph setEditView: self.
	scroller privateSubmorphs: (Array with: textMorph).

	scrollBar _ scrollBar copy.
	scrollBar privateOwner: self.
	scrollBar model: self.
	"owner submorphs model"

	^ self! !

!GoMeat methodsFor: '  core' stamp: 'miso 12/30/2004 11:16'!
coreBecomeFocus
"called from hand.newKeyboardFocus: via 'self.keyboardFocusChange: true'. Hand already registered us as the focus bearer (in its keyboardFocus var)"

	"prod the textMorph to actually show the selection -- there are two bugs in textMorph here: (i) no caret at all is shown when it inherits focus, even though one is created in its editor, it is not propagated to the paragraph, whose selectionStart/Stop remain nil - I haven't traced the call chain though; (ii) the prior selection interval is stored by us, but this info is not transmitted to the editor upon creation, so that the editor is recreated with a default positioning of the caret at the beginning"
	selectionInterval ifNotNil: [
		textMorph handleEdit: [
			textMorph editor selectInterval: selectionInterval]].	

	owner bigBrother ifNotNil: [owner bigBrother lastFocus: self]. "keep track of last focus for mouseEnter: in golgPane"
! !

!GoMeat methodsFor: '  core' stamp: 'miso 12/28/2004 22:33'!
coreStopBeingFocus
"like becomeFocus: this is called from hand.newKeyboardFocus: via michTextMorph.kbFocChange"

" // try to put this away, see if it is still needed
	self selectionInterval: textMorph editor selectionInterval.
	self hideScrollbar.
"

	owner meatLostFocus. "callback"! !

!GoMeat methodsFor: 'geometry' stamp: 'miso 1/1/2005 23:35'!
drawOn: aCanvas 
"do not include a thin red inset border for unaccepted edits. experiment with different feedbacks. try putting the whole pane darker than its container for instance. or changing wedge"

	| curOwner theColor ok |
	self hasUnacceptedEdits 
		ifTrue: [
			curOwner _ self.
			ok_false.
			[ok=false] whileTrue: [
				curOwner color == Color transparent 
					ifFalse: [theColor _ curOwner color dansDarker. ok_true].
				curOwner owner 
					ifNil: [theColor _ Color veryVeryLightGray. ok_true]
					ifNotNil: [curOwner _ curOwner owner]]].

	theColor ifNil: [theColor _ self color "Color transparent"].
	aCanvas fillRectangle: bounds color: theColor.
! !

!GoMeat methodsFor: 'geometry' stamp: 'miso 1/1/2005 23:31'!
extentOBSOLETE: newExtent
"override because superclasses set a minimal extent that we don't want (they try to be big enough to get a scrollbar, which defeats the purpose). Needed to override the scrollbar behaviour of GoMeat independently, via the #noVScrollBarPlease property, and hence this fix became unnecessary."


	bounds extent = newExtent ifTrue: [^ self].
	self changed.
	bounds := bounds topLeft extent: newExtent.
	self layoutChanged.
	self changed.

	textMorph ifNotNil: [textMorph extent: (self innerBounds width-6)@self height].
	self resizeScroller ! !

!GoMeat methodsFor: 'accessors' stamp: 'ms 8/9/1998 03:12'!
getTextHeight
	textMorph ifNil: [ ^ self bounds height].
	^ textMorph bounds height.! !

!GoMeat methodsFor: 'topic/focus' stamp: 'miso 10/14/1999 09:08'!
hasFocus

	"owner ifNil: [^false]"
	^ (owner bigBrother submorphHasFocus: self) or: [owner bigBrother submorphHasFocus: textMorph]! !

!GoMeat methodsFor: 'saving' stamp: 'ms 9/24/1998 03:27'!
hasUnacceptedEdits: aBoolean
"Overriden so as to tell header we got dirtied, if we did."

	super hasUnacceptedEdits: aBoolean.
	aBoolean  ifTrue: [ owner ifNotNil: [owner setDirty] ].
! !

!GoMeat methodsFor: 'admin' stamp: 'mich 1/13/2005 16:29'!
initialize

	bounds _ 0@0 corner: 50@40.
	submorphs _ EmptyArray.

		open _ false.
			borderColor _ #inset.

			retractableScrollBar _ scrollBarOnLeft _ true.
			scrollBar _ ScrollBar new model: self slotName: 'vScrollBar'.
			scrollBar borderWidth: 2; borderColor: #inset; initializeEmbedded: false.
			(hScrollBar _ ScrollBar new model: self slotName: 'hScrollBar')
				borderWidth: 1; 
				borderColor: Color black;
				initializeEmbedded: false.
			scroller _ TransformMorph new color: Color transparent.
			scroller offset: 0@0.
			self addMorph: scroller.				
			self on: #mouseEnter send: #mouseEnter: to: self.
			self on: #mouseLeave send: #mouseLeave: to: self.
			self setProperty: #noHScrollBarPlease toValue: true.
			self hHideScrollBar.
			self setProperty: #noVScrollBarPlease toValue: true. 
			self vHideScrollBar.
				askBeforeDiscardingEdits _ true.

	borderWidth _ 0.
	maxHeight _ 50.
	self hasUnacceptedEdits: true.
	self color: self meatColorPref.

"	isFocus _ false.	"
! !

!GoMeat methodsFor: 'accessors' stamp: 'miso 10/14/1999 09:28'!
isInside: aMorph

	^ owner isInside: aMorph! !

!GoMeat methodsFor: 'accessors' stamp: 'ms 11/7/1998 13:12'!
isWedge
	^ false! !

!GoMeat methodsFor: 'keyboard' stamp: 'miso 12/28/2004 11:05'!
key: keyNum combo: comboSymbol stream: s

	^ (owner key: keyNum combo: comboSymbol stream: s) ifNil: [
		"take a shot at doing it ourselves, if people above didn't get it"

		comboSymbol = #cmd ifTrue: [
			keyNum asCharacter = $v ifTrue:	[^self paste]].

		^ nil "no action from us, allow lower element to act"
	].! !

!GoMeat methodsFor: '  core' stamp: 'miso 10/30/1999 17:13'!
keyboardFocusChange: aBoolean
"sent by hand to the new focus bearer. redirect it to textMorph. should go to textMorph directly, but we sometime receive it if bigBrother registered us as lastFocus and wants to reactivate lastFocus (eg. on mouseEnter:)"

	^ textMorph keyboardFocusChange: aBoolean! !

!GoMeat methodsFor: 'debug' stamp: 'ms 8/10/1998 13:08'!
layoutChanged
	^ super layoutChanged.
"	fullBounds_nil.
	TEMP/EXPLORATORY
	we get many such call from TextMorph, and some embarassing
	ones during recalc that lead to infinite recursion. 
	See later how we deal with that."

	"see if we can discriminates our initiating a change of layout.
	(bounds height = self getTextHeight) ifFalse: [
		self boundsGivenMax: (bounds withHeight: maxHeight)]."! !

!GoMeat methodsFor: 'accessors' stamp: 'ms 9/10/1998 17:27'!
meatColorPref
	MeatColor ifNil: [MeatColor _ Color transparent].
	^ MeatColor! !

!GoMeat methodsFor: 'accessors' stamp: 'miso 10/24/1999 15:25'!
meatColorPref: c
"sets color for all header texts"
	MeatColor _ c.
	owner bigBrother walkHeadersWith: [:h| h meme setColor: c]
! !

!GoMeat methodsFor: 'accessors' stamp: 'ms 8/6/1998 19:26'!
minHeight
	^ 25.! !

!GoMeat methodsFor: 'accessors' stamp: 'ms 9/9/1998 23:57'!
model: aModel
"overriden to not register as a dependent. Dependents were never cleared."

	model _ aModel.
! !

!GoMeat methodsFor: '  core' stamp: 'miso 12/28/2004 22:32'!
mouseDown: event
"receive the event from text morph, when mouseDown: is the first click in that morph"

self halt. "is this ever used?"
	self askForFocus: event hand.! !

!GoMeat methodsFor: '  core' stamp: 'miso 12/28/2004 22:31'!
mouseEnter: event

"	self askForFocus: event hand.	"! !

!GoMeat methodsFor: '  core' stamp: 'ms 8/28/1998 14:12'!
mouseLeave: event

"do not say 'self stopBeingFocus: event.' as this is already triggered by the mouseEnter->kbFocChange"
! !

!GoMeat methodsFor: 'admin' stamp: 'miso 12/28/2004 23:31'!
on: anObject text: getTextSel accept: setTextSel readSelection: getSelectionSel menu: getMenuSel

	self model: anObject.
	getTextSelector := getTextSel.
	setTextSelector := setTextSel.
	getSelectionSelector := getSelectionSel.
	getMenuSelector := getMenuSel.

		"remplace 'setText:' pour eviter bound-makers qui sont lents et inutiles"
		scrollBar setValue: 0.0.
		textMorph := self textMorphClass new quickContents: self getText.
			textMorph setEditView: self.
			scroller addMorph: textMorph.
		self hasUnacceptedEdits: false.

	"init selectionInterval or else mouseEnter/becomeFoc will not show anything"
	selectionInterval := 1 to: 0.! !

!GoMeat methodsFor: 'admin' stamp: 'ms 11/3/1998 17:44'!
paste
	"allow pasting of headers onto me to indicate adding headers after me"

	(self coffreFort content isKindOf: GoHeader) ifTrue: [
		^ owner paste].
	^ nil "if we are called from key-shortcut, allow ParaEditor to handle it"! !

!GoMeat methodsFor: 'geometry' stamp: 'ms 11/4/1998 01:18'!
printOn: aStream

	super printOn: aStream.
	aStream nextPutAll: ' - '.
	self text asString printOn: aStream! !

!GoMeat methodsFor: 'accessors' stamp: 'miso 12/29/2004 00:28'!
refreshMeat

	self setText: self getText.
	"self textMorph releaseParagraph. otherwise does not show, sometimes."! !

!GoMeat methodsFor: 'saving' stamp: 'ms 7/3/1999 23:54'!
save
"this is a 'localSave' called from header, *not* directly from cmdS. cmdS goes to meat.accept, then to header.accept who dispatches as necessary throughout the outline (globalSave), and it then comes back to us at some point. (This is better than directly overriding kbController as several places send the 'accept' msg. It is [adapted from?] the original script in pluggable.accept, except for the return value."

	| textToAccept ok |
	self canDiscardEdits ifTrue: "needToSave ifFalse:" [^ self "flash"]. "another header might need saving!!"
	textToAccept _ textMorph asText.
	ok _ (setTextSelector == nil) or:
		[setTextSelector numArgs = 2
			ifTrue: [model perform: setTextSelector with: textToAccept with: self]
			ifFalse: [model perform: setTextSelector with: textToAccept]].
	ok ifTrue:
		[self setText: self getText.
		self hasUnacceptedEdits: false].
	^ ok! !

!GoMeat methodsFor: 'scrollBar enhance' stamp: 'miso 12/28/2004 11:29'!
scrollSelectionIntoView
"don't"! !

!GoMeat methodsFor: 'bug workaround' stamp: 'ms 8/15/1998 23:11'!
selectionBackup
	^ selectionInterval! !

!GoMeat methodsFor: 'bug workaround' stamp: 'ms 8/28/1998 18:11'!
selectionBackup: newSelect
	^ self selectionInterval: newSelect
! !

!GoMeat methodsFor: 'accessors' stamp: 'ms 11/2/1998 11:56'!
setColor: c
"sets color for this meat only. Called from menu therefore allows c to be symbol or nil"

	self color: 
		((c isKindOf: String "orSymbol!!") 
			ifTrue: [Color perform: c]
			ifFalse: [c]).
	self changed
! !

!GoMeat methodsFor: 'saving' stamp: 'ms 10/12/1998 03:06'!
setDirty
	hasUnacceptedEdits _ true ! !

!GoMeat methodsFor: 'accessors' stamp: 'ms 8/6/1998 20:26'!
setMeat: newContent
	self setText: newContent.

"setContents:  contents:
editString: setSelection: setText:"! !

!GoMeat methodsFor: 'bug workaround' stamp: 'miso 12/28/2004 23:31'!
setText: aText
"overriden to install a GolgiTextMorph instead of TextMorphForEditView to work around a bug in the handling of selection by TextMorph. See comments to the MichTextMorph class."
"29/10: do not say 'hasUnacceptedEdits: false' so as to allow normalisation of unsaved content"

true ifTrue: [^super setText: aText]. "see if this is still necessary!!"

scrollBar setValue: 0.0.
textMorph 
	ifNil: [
		textMorph := TextMorphForEditView new contents: aText wrappedTo: self innerBounds width-6.
		textMorph setEditView: self.
		scroller addMorph: textMorph]
	ifNotNil: [textMorph newContents: aText].! !

!GoMeat methodsFor: 'geometry' stamp: 'miso 12/28/2004 19:39'!
setWidthGetHeight: aWidth

	(aWidth == widthCache)
		ifTrue: [ ^ textMorph bounds height ]
		ifFalse: [ widthCache _ aWidth ].
	textMorph extent: aWidth @ (textMorph bounds bottom).
	^ textMorph bounds height.


"	aWidth = maxHeight ifTrue: [
		^ Transcript show:'resetting width ', textMorph bounds width printString;cr.].
	maxHeight _ aWidth."


"	(aWidth = textMorph bounds width) ifTrue: [
		Transcript show: 'meat did not change size.'; cr.	
		^ textMorph bounds height.
	]. 
	Transcript show: 'meat changed size.'; cr.	
	textMorph extent: aWidth @ (textMorph bounds bottom).
	^ textMorph bounds height.	
"

	! !

!GoMeat methodsFor: 'experimental' stamp: 'ms 9/4/1998 16:27'!
textMorph
	^ textMorph! !

!GoMeat methodsFor: 'accessors' stamp: 'miso 12/28/2004 23:30'!
textMorphClass

^ GolgiTextMorph! !

!ScrollPane methodsFor: '*golgi' stamp: 'miso 10/22/1999 12:29'!
getMenuSelector: s
	getMenuSelector _ s! !

Morph subclass: #GoWedge
	instanceVariableNames: 'buttonName target state actWhen nextStateCallback actionDict imageDict joker'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Golgi-Basics'!

!GoWedge commentStamp: 'miso 1/12/2005 19:44' prior: 0!
I am the the little arrow on the left of an outline header. In fact, I am 'yet another button class', because I provide the following goodies:

 hot swappable multi-state: I can have as many states as you want, and you can add states during runtime.
 exit-action (pluggable): each state is associated to an exit-state action (and an enter-state action)
 separate hit-area from img-area: the actual button drawing is a subset of me, and the hit-area is me. you can set the look&feel of the two, thus having a background and an image.
 multiple img sets (hot swappable): each state is associated to an image, and you can swap the image at runtime.
 multiple img type: images can be just a Color, or a Form/ColorForm/ImageMorph

To use me, you provide two types of methods (blocks), 
	i. blocks determining what action to take when entering/exiting a given state,
	ii. a block determining how to calculate the next state.

A note of caution: because of the current lack of proper closures in squeak, in the blocks that you give me, 'self' does not refer to me (the goWedge), it refers to whoever created the block. Typically, however, you only need to refer to me in your nextState block (eg to know what my current state is). For that reason, the block specifying how to compute the next state *must* take one argument. That argument will be initialized with a reference to me. 

Finally, if you provide a block for the #pressed state, it will be used while the mouse button is pressed down (to provide feedback). This is the only standard (ie hardcoded) state name. If you ignore it, nothing happens. If you provide nil value for any of its properties, that property is just ignored.
!

!GoWedge class methodsFor: 'instance creation' stamp: 'ms 8/11/1998 13:28'!
defaultComputingOfNextState
	"just toggle to the next state..."! !

!GoWedge class methodsFor: 'instance creation' stamp: 'ms 8/11/1998 00:19'!
defaultStates

	^#(	(#on 'green' #enterOn #exitOn)
		(#off red #enterOff #exitOff) )

"	(#on 	#[: aCanvas me aColor | aCanvas fillRectangle: me bounds color: color]
 #enterOn #exitOn)	"

! !

!GoWedge class methodsFor: 'instance creation' stamp: 'ms 9/8/1998 13:44'!
name: aN target: aT switch: aBlock states: anArrayOfStateDescriptors

	| b |
	b _ self switch: aBlock states: anArrayOfStateDescriptors.
	b 	buttonName: aN;
		target: aT.	
	^ b 
! !

!GoWedge class methodsFor: 'instance creation' stamp: 'ms 8/11/1998 19:22'!
newDefault
	^ self switch: self defaultComputingOfNextState states: self defaultStates! !

!GoWedge class methodsFor: 'instance creation' stamp: 'ms 9/21/1998 01:13'!
switch: aBlock
	| b |
	b _ super new.
	b 	nextstateMethod: aBlock.
	^ b 
! !

!GoWedge class methodsFor: 'instance creation' stamp: 'ms 9/21/1998 01:13'!
switch: aBlock states: anArrayOfStateDescriptors

	| b |
	b _ super new.
	b 	statesFromDescriptorArray:  anArrayOfStateDescriptors;
		nextstateMethod: aBlock.
	^ b 
! !

!GoWedge methodsFor: '  accessors' stamp: 'ms 9/20/1998 23:09'!
actWhen: condition
	"Accepts symbols:  #buttonDown, #buttonUp, and #whilePressed"

	actWhen _ condition! !

!GoWedge methodsFor: '  accessors' stamp: 'ms 9/24/1998 11:48'!
backgroundColor
	color! !

!GoWedge methodsFor: '  accessors' stamp: 'ms 9/20/1998 13:19'!
backgroundColor: c
	color _ c! !

!GoWedge methodsFor: '  accessors' stamp: 'ms 9/8/1998 13:44'!
buttonName
	^ buttonName! !

!GoWedge methodsFor: '  accessors' stamp: 'ms 9/8/1998 13:43'!
buttonName: n
	buttonName _ n! !

!GoWedge methodsFor: 'priv' stamp: 'miso 10/28/1999 12:13'!
changeState: theNewState
"complete change of state: action to get out of previous, action to get into new, redraw self. Follows  user click"

	| act |
	state = theNewState ifTrue: [^ state].

	self wearLookForState: theNewState.

	act _ self getStopActionFor: state.
	act ifNotNil:  [ act value ].		"act ifNotNil: [ target perform: act ]. "

	act _ self getStartActionFor: theNewState.
	act ifNotNil:  [ act value ].

	^ state _ theNewState! !

!GoWedge methodsFor: 'system API' stamp: 'ms 11/23/1998 10:36'!
click: evt
"we only get this upon yellow-click (option click on mac): upon yellow mouse-down, i ask if this is a click or a drag (#waitForClicksOrDrag:event:) and this method is invoked on clicks. the whole header is already selected."

! !

!GoWedge methodsFor: '  accessors' stamp: 'ms 9/24/1998 11:44'!
defaultImageSize
"decides how big a rectangle we draw when we get given a simple color as subimage. ignored by Forms fr now"

	^ self image maxExtent! !

!GoWedge methodsFor: '  accessors' stamp: 'ms 9/24/1998 11:42'!
defaultImageSize: s
"decides how big a rectangle we draw when we get given a simple color as subimage. ignored by Forms fr now"

	self image maxExtent: s! !

!GoWedge methodsFor: 'system API' stamp: 'ms 11/23/1998 10:04'!
drag: evt

	^ target beGrabbedByEvt: evt! !

!GoWedge methodsFor: 'priv' stamp: 'ms 11/2/1998 14:22'!
drawOn: aCanvas 

	aCanvas fillRectangle: "(self bounds insetBy: 1)" bounds color: color.
! !

!GoWedge methodsFor: 'system API' stamp: 'ms 9/21/1998 00:04'!
extent: aPoint

	bounds extent = aPoint ifFalse: [
		self changed.
		bounds _ bounds topLeft extent: aPoint.
		submorphs first center: bounds center.	"reposition image"
		self layoutChanged.
		self changed].
! !

!GoWedge methodsFor: 'priv' stamp: 'ms 9/23/1998 12:43'!
getImageForState: aState
"get the image associated with this state, and if no such state, return nil. Note that what is returned is a collection consisting of the object to be drawn, and then the type of object it is. Three types are allowed: a form, a color [as string, or symbol] (in which case the button bounds will be filled with that color), and a block (in which case the block will be executed and will be expected to draw the button). Returning the image type along with the image allows us not to compute it all the time..."

	^ imageDict at: aState ifAbsent: [ ^ nil ].
! !

!GoWedge methodsFor: 'experimental' stamp: 'ms 9/21/1998 01:11'!
getImageType: theImage

	(theImage isKindOf: ImageMorph orOf: Form) ifTrue: [ ^ #aForm ].
	(theImage isKindOf: Symbol orOf: String) ifTrue: [ ^ #aSymbol ].
	(theImage isKindOf: BlockContext) ifTrue: [ ^ #aBlock ].

	^ self error: 'OpWedge cannot add this image to its repertoire, because I dont understand what type of image it is: ', (theImage class asString).! !

!GoWedge methodsFor: 'priv' stamp: 'ms 9/21/1998 01:15'!
getNextState
"Call the method computing what to do, which was provided to us on init. That method should take one argument: me."

	^ nextStateCallback value: self.! !

!GoWedge methodsFor: 'priv' stamp: 'ms 9/21/1998 01:25'!
getStartActionFor: aState
	| theAct |
	theAct _ actionDict at: aState  ifAbsent: [ nil ].
	theAct ifNotNil: [
		theAct _ theAct first ].
	^ theAct.! !

!GoWedge methodsFor: 'priv' stamp: 'ms 9/21/1998 01:25'!
getStopActionFor: aState
	| theAct |
	theAct _ actionDict at: aState  ifAbsent: [ nil ].
	theAct ifNotNil: [
		theAct _ theAct second ].
	^ theAct.! !

!GoWedge methodsFor: '  menu' stamp: 'miso 11/26/2000 00:24'!
handleListenEvent: evt
"even a steady mouse on a btton receives mouseMove: so we can use this even to poll for cmdKeys, when those will be reported by Sensor."

"quick fix for 2988, was #handleEvent: "! !

!GoWedge methodsFor: 'system API' stamp: 'ms 8/11/1998 00:35'!
handlesMouseDown: evt

	^ true
! !

!GoWedge methodsFor: '  menu' stamp: 'ms 9/9/1998 14:24'!
handlesMouseOver: evt
"for future use, to show a menu cursor if a cmd key i pressed while mouse is in wedge"

	^ false
! !

!GoWedge methodsFor: 'public' stamp: 'ms 11/2/1998 10:51'!
imageFamily: aWedgeImageFamily
"ex imagesFromDescriptorArray: "

	aWedgeImageFamily do: [: oneImage | 
		oneImage ifNotNil: [self state: oneImage first image: oneImage second]].! !

!GoWedge methodsFor: 'priv' stamp: 'ms 10/4/1998 01:18'!
initialize
	super initialize.

	imageDict _  Dictionary new. 
	color _ Color transparent.
	self addMorph: (UbiqImage image: #red center: bounds center maxExtent: 3@3).

	actionDict _ Dictionary new.
	nextStateCallback _ []. 

	actWhen _ #buttonDown! !

!GoWedge methodsFor: '  accessors' stamp: 'ms 11/7/1998 13:12'!
isWedge
	^ true! !

!GoWedge methodsFor: 'experimental' stamp: 'ms 8/11/1998 21:15'!
joker
"Joker is a variable for use in the method that you provide in order to calculate the next state (stored in the theNextstateMethod var). Joker is unused by the button class as is; it is just a convenience to be able to pass more powerfull nextState methods, eg. remembering past states."

	^ joker.
! !

!GoWedge methodsFor: 'experimental' stamp: 'ms 8/11/1998 12:05'!
joker: aValue
	^ joker _ aValue.! !

!GoWedge methodsFor: 'public' stamp: 'ms 10/4/1998 13:48'!
jumpToState: theNewState
"change my appearance and my recorded state, but do not do any action associated with changes of state. Called externally when the change-actions are already done."

	state = theNewState ifTrue: [^self].
	state _ theNewState.
	self wearLookForState: state.
	^ state
! !

!GoWedge methodsFor: 'system API' stamp: 'miso 10/18/1999 09:39'!
mouseDown: evt

evt yellowButtonPressed ifTrue: [ "initiate a drag&drop operation"
	"transform coordinates to world-centered boundsInWorld"
	^ owner beGrabbedByEvt: evt].

evt shiftPressed ifTrue: [ 
	target ifNotNil: [
		(target class canUnderstand: #cmdMouseDownIn:while:) ifTrue: [
			^ target perform: #cmdMouseDownIn:while: with: buttonName with: evt ]]].

evt yellowButtonPressed ifTrue: [ 
	"ask to tell us whether it is a click or a drag. to do that HandMorph will compare this evt with nxt one. but this evt has been transformed to our local coord while next one will be used raw. this bugs, so retransform this evt to world coord"
	evt setCursorPoint: ((self transformFrom: World) invert: evt targetPoint).
	target askForFocus: evt hand.
	^ evt hand waitForClicksOrDrag: self event: evt].

"	^ owner beGrabbedByEvt: evt].

	target ifNotNil: [
		(target class canUnderstand: #cmdMouseDownIn:while:) ifTrue: [
			^ target perform: #cmdMouseDownIn:while: with: buttonName with: evt ]]]."

self wearLookForState: #pressed.	"The only standard state. hook for pressed feedBack"
actWhen == #buttonDown
	ifTrue: [self changeState: self getNextState]
	ifFalse: [super mouseDown: evt].
	"Allow on:send:to: to set the response to events other than actWhen"! !

!GoWedge methodsFor: '  menu' stamp: 'miso 11/25/2000 19:46'!
mouseEnter: evt
"for future use, to show a menu cursor if a cmd key i pressed while mouse is in wedge"

	evt hand addEventListener: self.
! !

!GoWedge methodsFor: '  menu' stamp: 'miso 11/25/2000 19:48'!
mouseLeave: evt
"for future use, to show a menu cursor if a cmd key i pressed while mouse is in wedge"
	evt hand removeEventListener: self.! !

!GoWedge methodsFor: 'system API' stamp: 'miso 10/18/1999 09:39'!
mouseMove: evt
"only called on move-while-mous-down ? mouseMove: "

(self containsPoint: evt cursorPoint)
	ifTrue: [

evt yellowButtonPressed ifTrue: [
	target ifNotNil: [
		(target class canUnderstand: #cmdMouseMoveIn:while:) ifTrue: [
			^ target perform: #cmdMouseMoveIn:while: with: buttonName with: evt ]]].

	actWhen == #whilePressed 
		ifTrue: [self changeState: self getNextState]
		ifFalse: [super mouseMove: evt]]
		"Allow on:send:to: to set the response to events other than actWhen"! !

!GoWedge methodsFor: 'system API' stamp: 'miso 10/17/1999 14:30'!
mouseUp: evt

actWhen == #buttonUp 
	ifTrue: [
		(self containsPoint: evt cursorPoint)
			ifTrue: [ self changeState: self getNextState ]
			ifFalse: [ target ifNotNil: [target mouseUpBalk: evt] ]
			"Allow owner to keep it selected for radio buttons"]				
	ifFalse: [super mouseUp: evt
		"Allow on:send:to: to set the response to events other than actWhen"]	
! !

!GoWedge methodsFor: 'public' stamp: 'ms 9/21/1998 01:14'!
nextstateMethod: aBlock
	"remember that a block must take one argument, which will be
	the button it is acting in. See comments for this class."

	^ nextStateCallback _ aBlock. ! !

!GoWedge methodsFor: '  accessors' stamp: 'ms 9/20/1998 23:51'!
state
	^ state.! !

!GoWedge methodsFor: 'public' stamp: 'ms 9/21/1998 03:10'!
state: aStateName image: aDrawable
	| ubiq |
	ubiq _ aDrawable.
	(ubiq isKindOf: String) ifTrue: [ ubiq _ ubiq asSymbol ].

	imageDict at: aStateName put: (Array with: ubiq with: (self getImageType: ubiq)).

	aStateName = state ifTrue: [self subImage image: ubiq. self changed].! !

!GoWedge methodsFor: 'public' stamp: 'ms 9/21/1998 01:09'!
state: aStateName image: aDrawable startAction: aStartBlock stopAction: aStopBlock

	self state: aStateName image: aDrawable.
	self state: aStateName startAction: aStartBlock stopAction: aStopBlock.
! !

!GoWedge methodsFor: 'public' stamp: 'ms 9/21/1998 01:08'!
state: aStateName startAction: aStartBlock stopAction: aStopBlock
	"should become 'model addState: ..."

	| cleanStartBlock cleanStopBlock |
	"clean up because #nil <> nil, and thus ifNotNil: failed..."
	aStartBlock = #nil 
		ifTrue: [cleanStartBlock _ nil ].
	aStopBlock = #nil 
		ifTrue: [cleanStopBlock _ nil].

	aStartBlock ifNotNil: [cleanStartBlock _ aStartBlock fixTemps].
	aStopBlock ifNotNil: [cleanStartBlock _ aStopBlock fixTemps].

	actionDict add: (Association key: aStateName value: 
						(Array	with: cleanStartBlock with: cleanStopBlock )).! !

!GoWedge methodsFor: 'public' stamp: 'ms 9/21/1998 00:55'!
stateFromDescriptor: aFullStateDescriptor

	^ self state: aFullStateDescriptor first 
		image: aFullStateDescriptor second 
		startAction: aFullStateDescriptor third 
		stopAction: aFullStateDescriptor fourth.! !

!GoWedge methodsFor: 'public' stamp: 'ms 9/23/1998 11:35'!
statesFromDescriptorArray: anArrayOfStateDescriptors
	"(called from class switch:states:)  "

	anArrayOfStateDescriptors do: [: oneDescriptor | self stateFromDescriptor: oneDescriptor ].! !

!GoWedge methodsFor: '  accessors' stamp: 'ms 9/21/1998 00:13'!
subImage
	^ submorphs first! !

!GoWedge methodsFor: '  accessors' stamp: 'ms 9/20/1998 23:55'!
target
	^ target! !

!GoWedge methodsFor: '  accessors' stamp: 'ms 9/8/1998 13:44'!
target: t
	target _ t! !

!GoWedge methodsFor: 'priv' stamp: 'ms 9/26/1998 12:09'!
wearLookForState: theNewState

	| imageProperties |
	imageProperties _ self getImageForState: theNewState. "a Collec"
	imageProperties ifNil: [ ^ nil ].

	self subImage image: imageProperties first.
	self changed.
! !

Morph subclass: #UbiqImage
	instanceVariableNames: 'image center maxExtent referencePoint'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Golgi-Helpers'!

!UbiqImage commentStamp: 'miso 1/12/2005 19:07' prior: 0!
A wrapper that can contain any img type
[very early class - to be checked and maybe discarded]!

!UbiqImage class methodsFor: 'as yet unclassified' stamp: 'ms 9/20/1998 12:46'!
image: i bounds: b

	^ self new image: i bounds: b! !

!UbiqImage class methodsFor: 'as yet unclassified' stamp: 'ms 9/20/1998 12:46'!
image: i center: c maxExtent: m

	^ self new image: i center: c maxExtent: m
! !

!UbiqImage methodsFor: 'accessors' stamp: 'ms 9/20/1998 13:14'!
center: c
	center _ c.
	self guessBounds.! !

!UbiqImage methodsFor: 'priv' stamp: 'ms 9/21/1998 02:32'!
drawOn: aCanvas

	image ifNil: [^ super drawOn: aCanvas].

	aCanvas image: image at: bounds origin.! !

!UbiqImage methodsFor: 'priv' stamp: 'ms 10/10/1998 14:27'!
guessBounds

	center ifNil: [self error: 'cannot guess bounds without center'].
	image 	ifNil: [
				maxExtent ifNil: [self error: 'cannot guess for rect without extent'].
				^bounds _ self rectFromCenter: center extent: maxExtent]
			ifNotNil: [^bounds _ self rectFromCenter: center extent: image extent]! !

!UbiqImage methodsFor: 'accessors' stamp: 'ms 10/10/1998 14:27'!
image: i
"should we store the img type to speed up drawing? or even request it from sender?"

	(i isKindOf: Symbol)	ifTrue: [
		image _ nil.
		self guessBounds.
		^ color _ Color perform: i].

	(i isKindOf: Color)	ifTrue: [
		image _ nil.
		self guessBounds.
		^ color _ i].

	(i isKindOf: ColorForm orOf: Form) ifTrue: [ 
		image _ i. 
		color _ Color transparent.
		^ self guessBounds ].

	(i isKindOf: ImageMorph) ifTrue: [ 
		image _ i. 
		color _ Color transparent.
		^ self guessBounds ].

	(i isKindOf: BlockContext) ifTrue: [ ^ image _ i ].

	image _ i.! !

!UbiqImage methodsFor: 'setters' stamp: 'ms 10/14/1998 16:56'!
image: i bounds: b
	self image: i center: b center maxExtent: b extent! !

!UbiqImage methodsFor: 'setters' stamp: 'ms 10/14/1998 16:55'!
image: i center: c maxExtent: m
	center _ c.
	maxExtent _ m.
	self image: i.
	self guessBounds.
	self layoutChanged! !

!UbiqImage methodsFor: 'priv' stamp: 'ms 9/21/1998 02:29'!
initialize
	super initialize.
! !

!UbiqImage methodsFor: 'accessors' stamp: 'ms 9/24/1998 11:41'!
maxExtent
	^ maxExtent! !

!UbiqImage methodsFor: 'accessors' stamp: 'ms 9/24/1998 11:41'!
maxExtent: m
	maxExtent _ m! !

!UbiqImage methodsFor: 'priv' stamp: 'ms 9/21/1998 02:39'!
privateMoveBy: d

	super privateMoveBy: d.
	center _ center translateBy: d! !

!UbiqImage methodsFor: 'priv' stamp: 'ms 9/20/1998 13:05'!
rectFromCenter: c extent: e

	| midHeight midWidth |
	midHeight _ e y // 2.
	midWidth _ e x // 2.
	^ Rectangle left: c x-midWidth 
				right: c x+(e x - midWidth) 		"for uneven numbers"
				top: c y-midHeight 
				bottom: c y+(e y - midHeight)
! !

!UbiqImage methodsFor: 'tobe adapted' stamp: 'ms 9/20/1998 12:27'!
subimageDrawOn: aCanvas

| imageType |
imageType_3.
	imageType == #aForm ifTrue: [ ^ aCanvas image: image at: bounds origin ].
	imageType == #aSymbol ifTrue: [ "Rectangle ImageMorph"
		^ aCanvas fillRectangle: bounds color: color ].
	imageType == #aBlock ifTrue: [
		^ image value: aCanvas value: self value: Color red. ].

	^ self error: 'dont know how to draw this class: ', (imageType printString), ' '.

! !

!String methodsFor: '*golgi' stamp: 'miso 1/2/2005 22:21'!
limitsOfWordAt: index
"see also #selectWord for similar functionality"

| start stop |
(self at: index) tokenish ifFalse: [^nil].

start := index.
[start > 0 and: [(self at: start) tokenish]] whileTrue: [start := start - 1].
start := start + 1.

stop := index.
[stop <= self size and: [(self at: stop) tokenish]] whileTrue: [stop := stop + 1].
stop := stop - 1.

^ {start . stop}! !

!String methodsFor: '*golgi' stamp: 'miso 1/2/2005 22:20'!
wordAt: index

| startStop |
startStop := self limitsOfWordAt: index.
startStop ifNil: [^nil].
^ self copyFrom: startStop first to: startStop second.! !

Model subclass: #GoTranslator
	instanceVariableNames: 'meme cmdMap cmdShiftMap'
	classVariableNames: 'DefaultCmdShiftMap DefaultCmdMap ImgFamilies'
	poolDictionaries: ''
	category: 'Golgi-Basics'!

GoTranslator class
	instanceVariableNames: 'wedgeFamilies'!

!GoTranslator commentStamp: 'miso 1/12/2005 19:48' prior: 0!
I translate the data viewed by a Golgi outliner into a language that Golgi understands. Currently I am only implemented for Memes, so for the moment my purpose is to make future expansion of viewable data-types easier.

If I am the translator for a Golgi rather than a GoHeader (ie for the root header), and the Golgi instance is filling a system window, I also act as the model for that window. Meaning that I receive - and manage - #release, #okToChange and friends.
!

!GoTranslator class methodsFor: 'class vars' stamp: 'ms 9/23/1998 15:02'!
acceptableMemeTypes
	^ #(Meme)! !

!GoTranslator class methodsFor: 'instance creation' stamp: 'ms 7/3/1999 20:52'!
accessorClassFor: dataObjectOrSymbol givenMomViewer: momHeader

	(dataObjectOrSymbol isKindOf: CodeMeme) ifTrue: [^ GoTranslateSmalltalk].
	(dataObjectOrSymbol isKindOf: ParseNode) ifTrue: [^ GoTranslateSqueakParseTree].
	(dataObjectOrSymbol isKindOf: Meme) ifTrue: [^ GoTranslator].

	^ GoTranslator "default"! !

!GoTranslator class methodsFor: 'wedgeDict' stamp: 'ms 4/11/1999 21:55'!
addwedgeImage: i toFamily: f forState: s

	| newFamily oldFamily |
	newFamily _ Array new.
	oldFamily _ self sureWedgeImageDict at: f.
	oldFamily ifNil: [ "create it with minimal necessary values"
		oldFamily _ #((inert gray)(contracted gray)(expanded gray))].
	oldFamily do: [: onePair |
		onePair first = s
			ifTrue: [newFamily _ newFamily copyWith: (Array with: s with: i)]
			ifFalse: [newFamily _ newFamily copyWith: onePair]].
	^ self addwedgeImageFamily: newFamily called: f

! !

!GoTranslator class methodsFor: 'wedgeDict' stamp: 'ms 4/11/1999 20:57'!
addwedgeImageFamily: aFamily called: aName

	^ self sureWedgeImageDict at: aName put: aFamily

"	^ ImgFamilies at: aName put: aFamily	"! !

!GoTranslator class methodsFor: 'instance creation' stamp: 'miso 10/28/1999 12:19'!
createHeaderAndTranslatorFor: someData viewedInside: aHeader

	| translator |
	"find an appropriate translator and glue it onto the data"
	translator _ (self accessorClassFor: someData givenMomViewer: aHeader) meme: someData.
	^ translator createYourHeaderGivenData: someData momHeader: aHeader! !

!GoTranslator class methodsFor: 'wedgeDict' stamp: 'ms 4/12/1999 12:54'!
ensureBasicWedgeImages
"root translator's default is to make sure that the #arrow family is present"

	| baseImage curFamily expandedImg contractedImg |

	"is the #arrow family already present?"
	self sureWedgeImageDict. "if none, getImageFamily: will fail"
	curFamily _ self getImageFamily: #arrow.
	curFamily isEmpty ifFalse: [^self].

	baseImage _ Form extent: 6@3			"upArrow, copied from ScrollBar"
					fromArray: #(2r11e28 2r1111e27 2r111111e26)
					offset: 0@0.
"	baseImage replaceColor: Color white withColor: Color transparent.	"
	
	curFamily _ Array new: 3. 
	expandedImg _ baseImage rotateBy: #pi centerAt: 0@0.  
	"must rotate before change white"
	expandedImg depth = 1 ifTrue: [
		expandedImg _ ColorForm mappingWhiteToTransparentFrom: expandedImg].
	curFamily at: 1 put: 
		(Array with: #expanded with: expandedImg).
	contractedImg _ baseImage rotateBy: #right centerAt: 0@0. 
	contractedImg depth = 1 ifTrue: [
		contractedImg _ ColorForm mappingWhiteToTransparentFrom: contractedImg].
	curFamily at: 2 put: 
		(Array with: #contracted with: contractedImg).
	curFamily at: 3 put: (Array with: #inert with: #gray).
	self addwedgeImageFamily: curFamily called: #arrow.! !

!GoTranslator class methodsFor: 'wedgeDict' stamp: 'ms 12/5/1998 20:44'!
getImageFamily: fSymbol
"cycle through the imageDicts in me and my sups looking for the fSymbol family. return the first found so that subclasses can override superclass data"

	| family |
	self wedgeImageDicts do: [: d |
		family _ d at: fSymbol ifAbsent: [nil].
		family ifNotNil: [^family] ].

	^ Array new

"return an empty array if family is absent so that nothing gets changed in wedge"! !

!GoTranslator class methodsFor: 'instance creation' stamp: 'mich 10/2/1999 23:31'!
headerClassFor: dataObjectOrSymbol givenMomViewer: momHeader

	momHeader ifNil: [^ Golgi].

	^ GoHeader   "default"! !

!GoTranslator class methodsFor: 'instance creation' stamp: 'miso 12/27/2004 23:48'!
meme: aMemeRef
	^ super basicNew 
		meme: aMemeRef;			"later download external refs"
		initialize! !

!GoTranslator class methodsFor: 'instance creation' stamp: 'ms 9/21/1998 21:54'!
new
	^ self error: 'i make no sense without a header and some data'! !

!GoTranslator class methodsFor: 'wedgeDict' stamp: 'ms 4/11/1999 20:56'!
sureWedgeImageDict
"return the img dict for this class, ignoring superclass dicts. creates one if none exists."

	wedgeFamilies ifNil: [wedgeFamilies _ Dictionary new].
	^ wedgeFamilies

"	self sharedPools do: [: aPool |
		(aPool at: #isWedgeDict ifAbsent: [false]) ifTrue: [^ aPool]].

	aPool _ Dictionary new.
	aPool at: #isWedgeDict put: true.
	self addSharedPool: aPool.
	^ aPool"! !

!GoTranslator class methodsFor: 'class vars' stamp: 'ms 9/23/1998 15:22'!
understandsThisMeme: m
	^ self acceptableMemeTypes includes: m class name! !

!GoTranslator class methodsFor: 'wedgeDict' stamp: 'ms 4/11/1999 20:57'!
wedgeImageDict
"return the img dict for this class, ignoring superclass dicts. creates one if none exists."

	^ wedgeFamilies! !

!GoTranslator class methodsFor: 'wedgeDict' stamp: 'ms 4/11/1999 21:13'!
wedgeImageDicts
"return the set of img dicts defined in me and my superclasses"

	| allImgDicts |
	self name = #GoTranslator 
		ifTrue: [allImgDicts _ Array new]
		ifFalse: [allImgDicts _ self superclass wedgeImageDicts].
	wedgeFamilies ifNotNil: "avoid creating it inadvertently"
		[allImgDicts _ allImgDicts copyWith: self sureWedgeImageDict]	.
	allImgDicts isEmpty 
		ifFalse: [^ allImgDicts]
		ifTrue: [^ self error: 'cannot find any imgDict although should hv been created at init'].


"	| allImgDicts |
	allImgDicts _ Array new.
	self allSharedPools do: [: aPool |
		(aPool at: #isWedgeDict ifAbsent: [false]) 
			ifTrue: [allImgDicts _ allImgDicts copyWith: aPool]].

	allImgDicts isEmpty 
		ifFalse: [^ allImgDicts]
		ifTrue: [^ self error: 'cannot find any imgDict although should hv been created at init'].
"

"  based on imgDict in class var. doesnt wrk coz cant beoverriden.
	| curClass allImgDicts |
	allImgDicts _ Array new.
	curClass _ self class.
	[curClass isKindOf: GoTranslator] whileTrue: [
		allImgDicts _ allImgDicts copyWith: curClass wedgeImageDict].
	^ allImgDicts

	^ allImgDicts copyWith: aPool
"! !

!GoTranslator class methodsFor: 'class vars' stamp: 'ms 9/26/1998 10:21'!
zapGlobals
	| thePool |
	DefaultCmdMap _ nil.
	DefaultCmdShiftMap _ nil.
	self sharedPools do: [: aPool |
		(aPool at: #isWedgeDict ifAbsent: [false]) ifTrue: [thePool _ aPool]].
	thePool ifNotNil: [self removeSharedPool: thePool]! !

!GoTranslator methodsFor: 'misc' stamp: 'ms 9/22/1998 02:21'!
acceptableMemeDescriptor: headerOrSymbol
"we need to get the daughtr meme corresponding to headerOrSymbol, but it we cannot say 'headerOrSymbol meme' in case it is a symbol, so we need to test it."

	(headerOrSymbol isKindOf: Symbol) 
		ifTrue: [^ headerOrSymbol ]
		ifFalse: [^ headerOrSymbol meme ]! !

!GoTranslator methodsFor: 'header link' stamp: 'miso 10/25/1999 13:55'!
addDependent: anObject
	"ignore requests from sysWins/scrollers. (& mk sure we hv only 1)"

	"I can be the 'model' for sysWin/scroller. Ignore them"
	(anObject isKindOf: GoHeader) ifFalse: [^ self].

	self dependents size > 0 
		ifTrue: [^ self error: '2 deps']
		ifFalse: [^ super addDependent: anObject].
! !

!GoTranslator methodsFor: 'bigBrother/ancestor -specific' stamp: 'ms 9/24/1998 10:53'!
authoriseBigBrotherSave: whoAsked

	^ true! !

!GoTranslator methodsFor: '  misc' stamp: 'miso 10/12/1999 11:16'!
backup
	self meme backup! !

!GoTranslator methodsFor: '  misc' stamp: 'miso 10/30/1999 15:35'!
becomeJumppoint
"ask memeSoup to point to me as an officially recognised area, ie. an entry-point into the meme soup"

	meme ancestor addJumpPoint: meme! !

!GoTranslator methodsFor: 'bigBrother/ancestor -specific' stamp: 'ms 11/7/1998 18:58'!
bigBrotherWasDiscarded: whoAsked! !

!GoTranslator methodsFor: '  keys' stamp: 'ms 11/1/1998 16:57'!
buildDefaultCmdMap 

	"adapted from ParagraphEditor.initializeCmdKeyShortcuts.
	Initialize the (unshifted) command-key shortcut table. This implementation dooe nothing (ie i do not override header actions, i am the default translator) but it shows how to go about doing it."

	| theCmdMap cmds |
	theCmdMap _ Array new: 256.  "use temp in case of a crash"
	theCmdMap atAllPut: nil.

	"temp keep arrow lines here, might need them to implement header-hop"
	theCmdMap at: (28 + 1) put: #noop:.					"left arrow key"
	theCmdMap at: (29 + 1) put: #noop:.					"right arrow key"
	theCmdMap at: (30 + 1) put: #noop:.					"up arrow key"
	theCmdMap at: (31 + 1) put: #noop:.					"down arrow key"

	cmds _ #(
		"$e	doSomething"
	).
	1 to: cmds size by: 2 do: [ :i |
		theCmdMap at: ((cmds at: i) asciiValue + 1) put: (cmds at: i + 1).].

	^ theCmdMap! !

!GoTranslator methodsFor: '  keys' stamp: 'ms 11/1/1998 16:59'!
buildDefaultCmdShiftMap 
	"Initialize the shift-command-key (or control-key) shortcut table."

	| theCmdMap cmds |
	"shift-command and control shortcuts"
	theCmdMap _ Array new: 256.  "use temp in case of a crash"
	theCmdMap atAllPut: nil.

	cmds _ #(
	).

	1 to: cmds size by: 2 do: [ :i |
		theCmdMap at: ((cmds at: i) asciiValue + 1)		put: (cmds at: i + 1).
		theCmdMap at: (((cmds at: i) asciiValue - 96) + 1)	put: (cmds at: i + 1).
	].

	^ theCmdMap! !

!GoTranslator methodsFor: 'bigBrother/ancestor -specific' stamp: 'mich 1/12/2005 22:46'!
buildGolgiMenu: m given: shiftKeyState
"I'm the model for a container of Golgi, and I am being asked to construct a menu for it"

^ (MenuMorph new defaultTarget: self) addSmartList: #(
		('rename'		rename)
		('backup'		backup)
		('save' 			saveAll)
		('misc'
			('edit jumpPoints' 		viewJumpPoints)
			('count headers'			showSubheaderCount))
		('prefs...'		subMenu	itemsForPrefs)
		-
		('dev...'
			('set version num' 		setVersion))
	).
	"add: 'save as...' subMenu: (self buildMenuFrom: self itemsForSaveAs)"
! !

!GoTranslator methodsFor: '  menus' stamp: 'miso 10/28/1999 12:34'!
buildMenuFrom: aList
"this is a commodity verb that allows to pass the list describing the item as an argument."

	^ (MenuMorph new defaultTarget: self) addSmartList: aList! !

!GoTranslator methodsFor: '  menus' stamp: 'miso 10/28/1999 12:33'!
colorPaletteFor: o

	| menu target |
	target _ (Compiler evaluate: o for: self logged: false).
	menu _ MenuMorph new defaultTarget: self.
	self itemsForColorPalette do: [:aColor | 
		menu	add: aColor asString  target: target  selector: #setColor:
				argument: aColor asSymbol ].
	menu addLine.
	menu add: 'tell me...' target: target selector: #setColor: argument: nil;
			popUpAt: self primaryHand position forHand: self primaryHand

"			add: 'tell me...'  target: sessionMngr  selector: #setColor:in:
				argumentList: (Array with: nil with: target);
"
! !

!GoTranslator methodsFor: 'misc' stamp: 'ms 11/4/1998 12:26'!
copyWithMeme
	^ self class meme: meme copyWithDaughters! !

!GoTranslator methodsFor: 'header link' stamp: 'miso 10/28/1999 12:17'!
createYourHeaderGivenData: someData momHeader: aHeader

	| headerClass |
	headerClass _ self class headerClassFor: someData givenMomViewer: aHeader.
	^ headerClass withTranslator: self.! !

!GoTranslator methodsFor: '  reorgs' stamp: 'ms 7/4/1999 00:39'!
dataChanged
	self dependents do: [:d | d refreshMeat]! !

!GoTranslator methodsFor: '  reorgs' stamp: 'miso 10/20/1999 13:42'!
dataGotOutOf: aMomMeme
"if meme left the mom I'm viewing it under, I have no further business, since I only view meme under that particular mom. So header needs to leave its mom and detach me from meme (which might continue to live under some other mom)"

	(self headerIsViewedUnder: aMomMeme) ifTrue: [self header dataGotOutOfMom "=suicide"]
! !

!GoTranslator methodsFor: '  reorgs' stamp: 'miso 10/25/1999 13:41'!
dataHasNewDaughter: newDaughterMeme
"signal that my header needs to refresh its daughters. this can be either upon newMeme creation, or newMeme mv into my meme. 
Note that in the latter case this may be a duplicate msg already signaled in the moved daughter with memeMovedInto:after: This is necessary since there are multiple views of data, and each view can have either me, or newDaughterMeme, or both. In case it has only me, we need this msg, in case it has only meme, we need the mv one."

	self header dressSubheaders 
	"Since header might not have dressed its daughters yet, and aSubmeme might be in the middle of them, it needs to make sure that they all get dressed. Easiest is to tell them all to dress, those that are already will ignore it"! !

!GoTranslator methodsFor: '  reorgs' stamp: 'miso 10/20/1999 13:44'!
dataMovedOutOf: oldMomMeme into: newMomMeme after: memeOrSymbol
"beware!! if data moved into newMomMeme, it doesn't mean that i will do a corresponding move. I will only do so if the prospective new mom is within the same viewer (or else to cumbersome to determine who goes where when multiple instance of me and newMom are present simultaneously). If mom is not in same viewer, this is equivalent to becomeOrphan. The whole buisiness of putting me into a new mom (ie the very existence of this method) is superfluous anyway, since newMoms will be sent a newDaughtersExist msg (so that we could just have had a dataGotOutOf: and newDaughtersExist). Kept this msg for slight optimisation: avoids recreating headers for every mv operation"

	| newMomHeader newSisterHeader |

	(self headerIsViewedUnder: oldMomMeme) ifFalse: [^false].
	newMomHeader _ self header bigBrother whoDressesThisMeme: newMomMeme.
	newMomHeader 
		ifNil: [ "not in our golgi. hara kiri. the real newMomHeader will create another daughter"
			self header dataGotOutOfMom.
			^ false] 
		ifNotNil: [
			newSisterHeader _ newMomHeader whichDaughterDressesThisMeme: memeOrSymbol.
			self header dataMovedOutOfMomInto: newMomHeader afterSub: newSisterHeader].

	^ true
! !

!GoTranslator methodsFor: 'meme properties' stamp: 'ms 9/21/1998 23:14'!
daughters

	^ meme daughters! !

!GoTranslator methodsFor: '    items' stamp: 'miso 10/30/1999 15:30'!
debugTheme
	self header prefs setAllColorsTo: Color transparent.
	self header prefs colorPref: #gray for: #GoHeader.
	self header prefs colorPref: #yellow for: #GoMeat.
	self header prefs setMargin: #interSub in: #GoHeader to: 5.! !

!GoTranslator methodsFor: '  keys' stamp: 'ms 9/22/1998 15:46'!
defaultCmdMap 

	DefaultCmdMap ifNil: [DefaultCmdMap _ self buildDefaultCmdMap].
	^ DefaultCmdMap! !

!GoTranslator methodsFor: '  keys' stamp: 'ms 10/16/1998 18:32'!
defaultCmdShiftMap 
	DefaultCmdShiftMap ifNil: [DefaultCmdShiftMap _ self buildDefaultCmdShiftMap].
	^ DefaultCmdShiftMap! !

!GoTranslator methodsFor: '  properties' stamp: 'mich 10/5/1999 10:06'!
defaultColor
"only for meat. assume that header/wedge are off-limits. if want to allow temp-color that persist through session, can cache them in an iVar here and provide a defaultColor: accessor"

	self meme semanticType = #shadow ifTrue: [^ Color red].

^Color transparent
	"^ (self prefs getPref: #meatColor) ifNil: [Color transparent]."! !

!GoTranslator methodsFor: '  properties' stamp: 'ms 11/7/1998 12:03'!
defaultImageFamily

"	^ #(	(inert gray)
			(expanded green)
			(contracted red)	
		)
"
	^ self getImageFamily: #arrow

! !

!GoTranslator methodsFor: 'misc' stamp: 'ms 11/24/1998 11:02'!
familyAboutToChange
"private!! my headers are about leave their mom, sisters. Give those a chance to adapt. For instance removing a header in a script sets the script dirty"

	"dependents do: [:d | d setDirty]."! !

!GoTranslator methodsFor: 'misc' stamp: 'ms 7/2/1999 12:28'!
familyJustChanged
"private!! my headers are acquiring a new family (changing mom, sisters), give me a chance to adapt. For instance adding/moving a header under a comment in a script results in the incoming header being commented"

! !

!GoTranslator methodsFor: '  misc' stamp: 'miso 1/11/2005 11:39'!
findGlobally: searchString
"find all memes in my meme's soup which contain searchString"

	| results |
	results _ Set new.
	self meme ancestor walkSelfAndDaughtersWith: [:aMeme | 
		(aMeme content asString findString: searchString) > 0 ifTrue: [results add: aMeme]].
	^ results! !

!GoTranslator methodsFor: 'misc' stamp: 'mich 10/3/1999 20:34'!
flashDependents

	self dependents do: [: d | d flash]
! !

!GoTranslator methodsFor: 'header link' stamp: 'miso 6/11/2000 11:11'!
forgetHeaderAndMeme
"also takes care of removing self from meme-deps since i have no more dependents."

	self removeDependent: self header.
	self meme removeDependent: self.
! !

!GoTranslator methodsFor: '  wedge dict' stamp: 'ms 10/4/1998 00:35'!
getImageFamily: fSymbol

	^ self class getImageFamily: fSymbol
! !

!GoTranslator methodsFor: '    items' stamp: 'miso 1/10/2005 21:41'!
globalKillCachedHeaders
	"should really be in Golgi:HeaderBank"
	Golgi sniffer scratchHeaderBank! !

!GoTranslator methodsFor: 'header link' stamp: 'mich 10/3/1999 20:26'!
header

	^ self dependents first! !

!GoTranslator methodsFor: 'header link' stamp: 'miso 10/25/1999 13:56'!
header: aHeader

	^ self addDependent: aHeader! !

!GoTranslator methodsFor: 'misc' stamp: 'ms 7/3/1999 18:31'!
headerDescriptor2memeDescriptor: headerOrSymbol

	(headerOrSymbol isKindOf: Symbol) 
		ifTrue: [^ headerOrSymbol ]
		ifFalse: [^ headerOrSymbol meme ]! !

!GoTranslator methodsFor: 'misc' stamp: 'miso 10/16/1999 17:21'!
headerIsViewedUnder: aMomMeme
"our header might have several moms. see if the viewer i'm translating for is viewing the relevant one"

	| oldHeader |
	oldHeader _ self header headerAbove.
	oldHeader ifNil: [^false]. "happens when oldHeader is bigBro, and some other viewer on him mv"
	^ oldHeader meme == aMomMeme
! !

!GoTranslator methodsFor: '  menus' stamp: 'miso 11/1/1999 22:36'!
headerMenu

	^ (MenuMorph new defaultTarget: self) addSmartList:  #(
			('become a jump point'	becomeJumppoint)
			('hoist' 				hoist)
			('export' 			backup)
			-
			('misc...'
				('save' 					saveAll)
				('count headers'			showSubheaderCount)
				('temp colors'
					('text'				colorPaletteFor:	'self meatMorph')
					('header'			colorPaletteFor:	'self')
					('background'		colorPaletteFor:	'bigBrother')))
			('debug...'
				('kill cached headers'	globalKillCachedHeaders)
				('wedge is comment'		setCommentImage))
			('inspect...'
				('translator'				inspect:		'self translator')
				('meme'					inspect:		'self meme')
				('meme meat'			inspect:		'self meme content')
				-
				('prefs'					inspect:		'bigBrother prefs'))
		)


"		add: 'save as...' subMenu: (self buildMenuFrom: self itemsForSaveAs); "! !

!GoTranslator methodsFor: '  reorgs ' stamp: 'mich 1/13/2005 00:52'!
headerWantsADaughterAfter: headerOrSymbol
"called from GoHeader.procreate"

	| existingMemeOrSymbol newMeme typeOfNode |
	self reorgsAllowed ifFalse: [^false].

	"first decide what kind of meme should be created"
	existingMemeOrSymbol _ self headerDescriptor2memeDescriptor: headerOrSymbol.
	existingMemeOrSymbol = #root
		ifTrue: [typeOfNode _ self meme defaultDaughterClass]
		ifFalse: [typeOfNode _ existingMemeOrSymbol class].
	newMeme _ typeOfNode new.

	newMeme putSelfInto: self meme after: existingMemeOrSymbol.
	^ newMeme! !

!GoTranslator methodsFor: '  reorgs ' stamp: 'miso 10/20/1999 13:16'!
headerWantsToGetOutOf: momHeader
"leave the mom I'm in, first in meme, then in header. I have no further business, since I only view meme under that particular mom. So I also want to be detached from my meme (which might continue to live under some other mom)"

	self reorgsAllowed ifFalse: [^false].

	^ self meme moveSelfOutOf: momHeader meme! !

!GoTranslator methodsFor: '  reorgs ' stamp: 'ms 7/3/1999 23:16'!
headerWantsToGoInto: newHeader afterSub: headerOrSymbol

	self reorgsAllowed ifFalse: [^false].
	^ self meme putSelfInto: newHeader meme
				after: (self headerDescriptor2memeDescriptor: headerOrSymbol)! !

!GoTranslator methodsFor: '  reorgs ' stamp: 'miso 10/20/1999 13:03'!
headerWantsToMoveOutOf: oldHeader into: newHeader afterSub: headerOrSymbol

	self reorgsAllowed ifFalse: [^false].
	^ meme moveSelfOutOf: oldHeader meme
			into: newHeader meme
			after: (self headerDescriptor2memeDescriptor: headerOrSymbol)

	"for mono-mom memes:
	^ meme moveSelfInto: newHeader meme after: headerOrSymbol"! !

!GoTranslator methodsFor: '    items' stamp: 'miso 10/30/1999 15:38'!
hoist
	Golgi view: self meme! !

!GoTranslator methodsFor: 'sysWin modelling' stamp: 'miso 7/7/1999 08:59'!
initialExtent
"sysWin calls to its model: defaultBackgroundColor windowReqNewLabel: okToChange step wantsSteps (stepTime) modelWakeUp initialExtent labelString windowIsClosing release"

	^ 350@400! !

!GoTranslator methodsFor: 'misc' stamp: 'miso 11/1/1999 08:43'!
initialize
	"this is called at *end* of instance creation: fill in those values that are needed but have been left blank up to now"
	meme ifNil: [^ self error: 'cannot work without data'].
	cmdMap ifNil: [cmdMap _ self defaultCmdMap].
	cmdShiftMap ifNil: [cmdShiftMap _ self defaultCmdShiftMap].
	self class ensureBasicWedgeImages.
	dependents ifNil: [dependents _ Array new].
	self familyJustChanged.			"give me a chance to adapt to my environment"! !

!GoTranslator methodsFor: '    items' stamp: 'miso 11/1/1999 22:36'!
inspect: anItemAsSymbol

	(Compiler evaluate: anItemAsSymbol for: self logged: false) explore! !

!GoTranslator methodsFor: 'meme properties' stamp: 'miso 10/8/1999 21:48'!
isInside: putativeMemeAncestor
	"this helps during drag&drop to prevent dropping inside myself... Delegate to model, as we might not be in the in the same outline"

	^ self meme isInside: putativeMemeAncestor! !

!GoTranslator methodsFor: '  menus' stamp: 'miso 10/28/1999 12:32'!
itemsForColorPalette
	^ #(	transparent
			white
			gray
			green
			red
			blue
			yellow
		)
! !

!GoTranslator methodsFor: '  menus' stamp: 'miso 10/29/1999 13:50'!
itemsForPrefs

	^ #(('appearance...'
			('themes...'
				('plain'				plainTheme)
				('debug'				debugTheme))
			-
			('color'
				('overall'		colorPaletteFor:		overall)
				-
				('headers'		colorPaletteFor:		GoHeader)
				('text'			colorPaletteFor:		GoMeat)
				('scroller'		colorPaletteFor:		Golgi))
			('margins'
				('left'			queryMarginNumberFor:in:	(left GoHeader) )
				('right'			queryMarginNumberFor:in:	(right GoHeader) )
				('top'			queryMarginNumberFor:in:	(top GoHeader) )
				('bottom'		queryMarginNumberFor:in:	(bottom GoHeader) )
				-
				('wedge'		queryMarginNumberFor:in:	(wedge GoHeader) )
				('between subs'	queryMarginNumberFor:in:	(interSub GoHeader) )))
		)
! !

!GoTranslator methodsFor: '  menus' stamp: 'miso 10/28/1999 12:29'!
itemsForSaveAs
	^ #(
			('parenthesised text'		saveAs:		parens)
			('tabbed text'			saveAs:		tab)
			('xml'					saveAs:		xml)
			('dotHead'				saveAs:		dotHead)
		)
! !

!GoTranslator methodsFor: '  keys' stamp: 'miso 12/28/2004 11:03'!
key: keyNum combo: comboSymbol
	| theSelector |
	theSelector := nil.
	comboSymbol = #cmd ifTrue: [
		theSelector := cmdMap at: keyNum].
	comboSymbol = #cmdShift ifTrue: [
		theSelector := cmdShiftMap at: keyNum].
	theSelector ifNil: [
		^ nil].
	theSelector = #nil ifTrue: [
		^ nil].
	theSelector = #noop: ifTrue: [
		^ nil].
	self perform: theSelector.
	^ true
	"alway return true, else same keyDown is reprocessed and we get in a loop"! !

!GoTranslator methodsFor: '  properties' stamp: 'ms 11/7/1998 12:05'!
meatClass

	^ GoMeat! !

!GoTranslator methodsFor: '  properties' stamp: 'miso 10/30/1999 16:34'!
meatLostFocus
"callback: headers send this msg after their meat stops being being focus, to allw me to take action. default does nothing."
! !

!GoTranslator methodsFor: 'meme properties' stamp: 'ms 9/21/1998 23:10'!
meme
	^ meme! !

!GoTranslator methodsFor: 'meme properties' stamp: 'ms 7/1/1999 22:53'!
meme: newMeme
"this is only accessed on translator creation and on reinstantiateMemes"

	meme _ newMeme.
	newMeme newDataReader: self! !

!GoTranslator methodsFor: 'meme properties' stamp: 'miso 10/30/1999 16:10'!
memeContent
	^ meme content! !

!GoTranslator methodsFor: 'meme properties' stamp: 'miso 10/30/1999 16:09'!
memeContent: newContent
"set the text of the meme, ignoring previous content. return whether was successful ('save' counts on it)."

	self meme content: newContent.
	^ true! !

!GoTranslator methodsFor: 'sysWin modelling' stamp: 'mich 10/3/1999 20:44'!
okToChange
	"Bizarre semantics: seems to mean something like areYouInADeletableState-or-areYouModifiableAtAll. But the answer to the two questions is not necessarily the same, I can be a modifiable object (i.e. the answer is 'yes') but dirty (i.e. the answer is 'no'). To preserve sanity I treat it as areYouInADeletableState only, ie. the call when sysWin is about to delete, "

	
	^ self header isItOkToDie! !

!GoTranslator methodsFor: '  menus' stamp: 'miso 10/29/1999 14:35'!
perform: selector orSendTo: otherTarget
"In the menu I built, every menuItem without arguments is reworked by setInvokingView: (apparently for MVC compatibility) so as to not be directly performed by me (the legit target) but rather I am asket to perform:orSendTo:, and the Object default for that is to send to otherTarget. Usually the target is the model and otherTarget is the view so all is ok, but here the other target is ScrollPane (which is the invoker) instead of Golgi... So override. "

	^ self perform: selector! !

!GoTranslator methodsFor: '    items' stamp: 'miso 10/30/1999 15:31'!
plainTheme
	self header prefs colorPref: #transparent for: #GoHeader.
	self header prefs setMargin: #interSub in: #GoHeader to: 0.! !

!GoTranslator methodsFor: 'misc' stamp: 'miso 10/8/1999 16:42'!
printOn: aStream
	"used in inspectors, debuggers, etc."

	super printOn: aStream. aStream nextPutAll: 
		'[', self identityHash printString, '] for: '. self meme printOn: aStream.! !

!GoTranslator methodsFor: '    items' stamp: 'miso 10/30/1999 15:32'!
queryMarginNumberFor: whichMargin in: o
	| n target oldVal |
	target _ (Compiler evaluate: o for: self logged: false).
	oldVal _ target Margin: whichMargin.

	n _ 'Value for ', whichMargin printString, ' margin:' .
	n _ FillInTheBlank request: n initialAnswer: oldVal printString .
	n _ n asNumber.
"	n isTypeOf: Number ifFalse: [^self]."
	^ self header prefs setMargin: whichMargin in: o to: n.! !

!GoTranslator methodsFor: 'sysWin modelling' stamp: 'miso 10/8/1999 17:02'!
release

	self header detachFromdata! !

!GoTranslator methodsFor: '    items' stamp: 'miso 10/30/1999 15:33'!
rename

	| name |
	name _ self meme content.
	name _ FillInTheBlank request: 'New name:' initialAnswer: name.
	name isEmpty ifFalse: [
		self header renameGolgiTo: name]! !

!GoTranslator methodsFor: '  reorgs ' stamp: 'ms 11/3/1998 20:28'!
reorgsAllowed
	^ true! !

!GoTranslator methodsFor: '  properties' stamp: 'ms 9/22/1998 10:26'!
save: who

	who save: #global! !

!GoTranslator methodsFor: '    items' stamp: 'miso 10/29/1999 13:50'!
saveAll
	self header bigBrother recursiveSave! !

!GoTranslator methodsFor: '    items' stamp: 'miso 10/30/1999 15:26'!
setVersion
	Golgi setVersion! !

!GoTranslator methodsFor: '    items' stamp: 'miso 10/30/1999 15:37'!
showSubheaderCount

	^ self inform: 'There are ', self meme countDaughtersRecursively asString, ' daughters'! !

!GoTranslator methodsFor: '    items' stamp: 'miso 10/30/1999 15:41'!
tab
	self header tab! !

!GoTranslator methodsFor: 'misc' stamp: 'ms 9/23/1998 15:16'!
understandsThisMeme: m
	^ self class understandsThisMeme: m! !

!GoTranslator methodsFor: '  misc' stamp: 'miso 10/29/1999 14:04'!
viewJumpPoints
	Golgi view: meme ancestor jumpPointsHolder! !

!GoTranslator methodsFor: '  wedge dict' stamp: 'ms 10/3/1998 19:04'!
wedgeImageDict
"return the image dict for this class, ignoring superclasses. create one if absent"

	^ ImgFamilies! !

!GoTranslator methodsFor: '  wedge dict' stamp: 'ms 9/25/1998 14:31'!
wedgeImageDicts
"creates it if absent"

	^ self class wedgeImageDicts! !

!GoTranslator methodsFor: '    items' stamp: 'miso 1/4/2005 19:48'!
windowReqNewLabel: newLabelString
"sent when the menu item 'change label...' is selected. It asks us (the model) whether it is ok to do this."

self memeContent: newLabelString.
^ true! !

Model subclass: #Node
	instanceVariableNames: 'moms daughters content semanticType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Golgi-Basics'!

!Node commentStamp: 'miso 1/12/2005 19:19' prior: 0!
I am a slight abstraction from a Meme, loosely representing a node in a (directed) graph. I have however grown many meme-like appendices...!

Node subclass: #Meme
	instanceVariableNames: ''
	classVariableNames: 'EmptyArray'
	poolDictionaries: ''
	category: 'Golgi-Basics'!

!Meme commentStamp: 'miso 1/12/2005 19:21' prior: 0!
I represent a piece of data, a unit of information (of arbitrary size). My content is a text by default, but there is nothing inherent about that. I am the default object viewed by a Golgi viewer (outliner).
!

!Meme class methodsFor: 'creation' stamp: 'miso 10/9/1999 09:21'!
content: aText type: aSymbolOrNil

	^ (self ofType: aSymbolOrNil) content: aText asText! !

!Meme class methodsFor: 'convertFrom' stamp: 'mich 10/2/1999 23:53'!
fromDotHeadFile: f

	| s |
	s _ (FileStream readOnlyFileNamed: f) contentsOfEntireFile.
	s _ s copyFrom: 1 to: (
		s findString: (26 asCharacter asString) startingAt: 1 caseSensitive: true)-1.
	^ Golgi view: (
		self fromDotHeadString: s title: (FileDirectory on: f) pathName )! !

!Meme class methodsFor: 'convertFrom' stamp: 'ms 3/6/1999 22:00'!
fromDotHeadString: s title: t

	| theAncestor lastMeme headerArray nextLevel i oldLevel |
	headerArray _ self substringsOf: s.
	lastMeme _ theAncestor _ Meme withMeat: t asText.
	oldLevel _ 0.

	headerArray do: [:aDot|
		(aDot beginsWith: '.HEAD ') ifTrue: ["should be case-insensitive"
			aDot _ aDot copyFrom: 7 to: aDot size. "pop prefix"
			"now search for level"
			i_1.
			[(aDot at: i) isDigit] whileTrue: [i_i+1].
			nextLevel _ (aDot copyFrom: 1 to: i-1) asNumber.
			aDot _ aDot copyFrom: i+3 to: aDot size "skip +/-"].

		"now find mom for this level."
		lastMeme _ self privInsertText: aDot asText
					afterMeme: lastMeme
					ofLevel: oldLevel
					asNewLevel: nextLevel
					givenAncestor: theAncestor.
		oldLevel _ nextLevel].

	^ theAncestor
	"open it!!"! !

!Meme class methodsFor: 'convertFrom' stamp: 'miso 10/9/1999 09:19'!
fromTabbedString: theString
	"return a CodeMemeTree mirroring the tab structure in s. do not set the commented/semanticType attribute, it just create generic CodeMeme"
	| theMom oldLevel newLevel lastMeme ancestor sentenceArray  |
	sentenceArray _ self substringsOf: theString.
	theMom _ ancestor _ lastMeme _ self new.
	oldLevel _ 0.
	sentenceArray do: [
		:s  |
		newLevel _ 0.
		[s first == Character tab] whileTrue: [
			newLevel _ newLevel + 1.
			s _ s copyFrom: 2 to: s size].
		newLevel > oldLevel ifTrue: [
			theMom _ lastMeme].
		newLevel < oldLevel ifTrue: [
			"find the relevant oldLevel!!"
			"use as a counter, we don't need it anymore"
			oldLevel _ 0.
			theMom _ ancestor.
			[oldLevel < newLevel] whileTrue: [
				theMom _ theMom daughters last.
				oldLevel _ oldLevel + 1]].
		lastMeme _ self string: s mom: theMom after: #last.
		oldLevel _ newLevel].
	^ ancestor
	"this parses level 0 as ancestor"
		"mom _ nil"
		"newLevel _ 0"
		"oldLevel _ nil.
memeArray do: [: s |
	[s first == Character tab] whileTrue: [
		newLevel _ newLevel + 1.
		s _ s copyFrom: 2 to: s size].

	newLevel = 0 ifTrue: [
		mom	ifNil: [oldLevel _ 0]
				ifNotNil: [^ self error: 'more than one ancestor']].
	newLevel > oldLevel	ifTrue: [mom _ lastMeme].
	newLevel < oldLevel	ifTrue: [mom _ mom owner].
	mom	ifNil: [ancestor _ lastMeme _ MemeAncestor new]
			ifNotNil: [lastMeme _  Meme addWithString: s mom: mom after: #last].
	lastMeme sisterBefore: s.

	oldLevel _ newLevel.
].
^ ancestor"! !

!Meme class methodsFor: 'priv' stamp: 'miso 10/28/1999 13:41'!
initialize
	"Meme initialize. Run every time start a new golgi?"

	"this empty array object is shared by all memes with no daughters"
	EmptyArray _ OrderedCollection new.

! !

!Meme class methodsFor: 'priv' stamp: 'miso 6/7/2001 12:13'!
internalPathSeparator
	
	^ '!!'
! !

!Meme class methodsFor: 'creation' stamp: 'miso 10/9/1999 09:21'!
ofType: aSymbolOrNil

	aSymbolOrNil = #soup ifTrue: [^ MemeSoup new].
	^ self new semanticType: aSymbolOrNil! !

!Meme class methodsFor: 'convertFrom' stamp: 'miso 10/9/1999 09:18'!
privInsertText: t afterMeme: m ofLevel: lastLevel asNewLevel: newLevel givenAncestor: a

	| theMom i |
	newLevel > lastLevel ifTrue: [
		theMom _ m].
	newLevel <= lastLevel ifTrue: [
		"find the relevant oldLevel!!!!!!!!"
		"use as a counter, we don't need it anymore"
		i _ 1.
		theMom _ a.
		[i < newLevel] whileTrue: [
			theMom _ theMom daughters last.
			i _ i + 1]].
	^ self text: t mom: theMom after: #last.! !

!Meme class methodsFor: 'creation' stamp: 'miso 10/9/1999 09:18'!
string: aString mom: aMom after: aMeme
	
	^ self text: aString asText
				mom: aMom 
				after: aMeme 
! !

!Meme class methodsFor: 'priv' stamp: 'miso 6/7/2001 12:14'!
stringurl2array: stringUrl

	^ (stringUrl findTokens: self internalPathSeparator) collect: [: e | e unescapePercents]
! !

!Meme class methodsFor: 'convertFrom' stamp: 'ms 12/21/1998 21:52'!
substringsOf: s
"Like string.substrings but separator list is smaller: tab and space do not count, only cr, lf. Answer an array of the substrings that compose the receiver."

	| result aStream char lineEnders |
	lineEnders _ Array with: 10 with: 12 with: 13. "line feed, form feed, cr"
	result _ WriteStream on: (Array new: 10).
	aStream _ WriteStream on: (String new: 16).
	1 to: s size do: [:i |
		char _ s at: i.
		(lineEnders includes: char asciiValue)
		 ifTrue: [aStream isEmpty ifFalse: [result nextPut: aStream contents. aStream reset]]
		 ifFalse: [aStream nextPut: char]].
	aStream isEmpty ifFalse: [result nextPut: aStream contents].
	^ result contents! !

!Meme class methodsFor: 'creation' stamp: 'miso 10/9/1999 09:18'!
text: aText mom: aMom after: aMeme
	
	^ self new 
		putSelfInto: aMom after: aMeme;
		content: aText! !

!Meme class methodsFor: 'creation' stamp: 'miso 10/29/1999 13:09'!
withMeat: theContent

	^ self new content: theContent asText.! !

!Meme class methodsFor: 'creation' stamp: 'miso 10/28/1999 13:39'!
withMom: aMom
	| m |
	m _ self new.
	m putSelfInto: aMom after: #last.
	^ m! !

!Meme methodsFor: 'create/copy' stamp: 'ms 7/3/1999 23:30'!
becomeLike: originMeme

	self content: originMeme content copy.
	self semanticType: originMeme semanticType.! !

!Meme methodsFor: 'create/copy' stamp: 'ms 11/4/1998 11:47'!
copy

	^self class new becomeLike: self.
! !

!Meme methodsFor: 'create/copy' stamp: 'ms 3/6/1999 20:45'!
copyWithDaughters

	| copyOfSelf |
	copyOfSelf _ self copy.
	self daughters do: [:d| 
		copyOfSelf registerIncomingDaughter: d copyWithDaughters after: #last].
	^ copyOfSelf! !

!Meme methodsFor: 'statistics' stamp: 'miso 11/1/1999 10:56'!
countCharsInSelfAndDaughters

	| n |
	n _ self contentString size.
	self daughters do: [:d|
		n _ n + (d countCharsInSelfAndDaughters)].
	^ n
! !

!Meme methodsFor: 'statistics' stamp: 'miso 10/6/1999 19:30'!
countDaughtersByWalking
"temp - double check with the direct recursion verbs"

	Smalltalk at: #GolgiTempCountingTest put: 0.
	self walkDaughtersWith: [: n |
		Smalltalk at: #GolgiTempCountingTest put: (Smalltalk at: #GolgiTempCountingTest)+1 ].
	^ Smalltalk at: #GolgiTempCountingTest! !

!Meme methodsFor: 'statistics' stamp: 'miso 10/6/1999 19:30'!
countPrimaryDaughtersByWalking
"temp - double check with the direct recursion verbs"

	Smalltalk at: #GolgiTempCountingTest put: 0.
	self walkPrimaryDaughtersWith: [: n |
		Smalltalk at: #GolgiTempCountingTest put: (Smalltalk at: #GolgiTempCountingTest)+1 ].
	^ Smalltalk at: #GolgiTempCountingTest! !

!Meme methodsFor: 'name & address' stamp: 'miso 10/5/1999 22:32'!
daughterCalled: nameAsString
	self daughters do: [
		:a  |
		(a name sameAs: nameAsString) ifTrue: [
			^ a]].
	^ nil! !

!Meme methodsFor: 'name & address' stamp: 'miso 6/12/1999 22:49'!
daughterNames
	^ self daughters collect: [:a  | a name]
! !

!Meme methodsFor: 'name & address' stamp: 'miso 10/13/1999 15:30'!
externalName

	^ self name , FileDirectory dot , Utilities dateTimeSuffix, '.meme'
! !

!Meme methodsFor: 'backup-debug' stamp: ''!
healPseudoJumps
	| t1 |
	t1 _ moms
				select: [:t2 | t2 contentString = 'jumpPoints' and: [Soups first jumpPointsHolder ~~ t2]].
	t1 isEmpty ifTrue: [^ self].
	t1 do: [:t3 | self moveSelfOutOf: t3]! !

!Meme methodsFor: 'create/copy' stamp: 'ms 3/9/1999 01:40'!
initialize
	super initialize.
	content _ Text fromString: ''. 
! !

!Meme methodsFor: 'name & address' stamp: 'miso 1/3/2005 11:30'!
internalPath
	
	| p |
	p := ''.
	self internalPathAsArray do: [: pathItem | p := p, pathItem, self class internalPathSeparator].
	(p endsWith: self class internalPathSeparator) ifTrue: [p := p copyFrom: 1 to: p size - 1].
	^p
! !

!Meme methodsFor: 'name & address' stamp: 'mich 10/5/1999 18:46'!
internalPathAsArray
	^  self primaryMom
		ifNil: [Array with: self localPath]
		ifNotNil: [ self primaryMom internalPathAsArray copyWith: self localPath]
! !

!Meme methodsFor: 'name & address' stamp: 'mich 10/5/1999 18:17'!
internalUrl

	^ self class name, '/', self internalPath! !

!Meme methodsFor: 'name & address' stamp: 'mich 10/5/1999 18:25'!
localPath
"my own contribution to the url, the part of the path that I represent"

	^ self name encodeForHTTP

	"^ self meatString"! !

!Meme methodsFor: 'name & address' stamp: 'miso 11/1/1999 10:56'!
name

	^ self contentString withSeparatorsCompacted truncateTo: 64 " asLegalSelector"

	"what you really want is String translateWith: table, making all spaces into underscores, instead of asLegalSelector" 
 ! !

!Meme methodsFor: 'backup-debug' stamp: 'miso 1/12/2005 01:05'!
primaryMom
"override this in memes because some memes have a jumpPoint as a mom, and if it becomes a primary mom, backup/restore goes boink."

	self moms isEmpty ifTrue: [^ nil].
	^ self moms detect: [:t1 | t1 contentString ~= 'jumpPoints'] ifNone: []! !

!Meme methodsFor: 'statistics' stamp: 'miso 10/6/1999 13:49'!
printSecondaryDaughtersRecursively
"see a list of secondary links in the transcript. some appear twice if they are under a shared tree. will appear only once in primaryTree"

	| sDaughter |
	
	sDaughter _ self daughters select: [:d| (self isPrimaryDaughter: d) not].

	sDaughter do: [: d | 
		Transcript show: (self shortName), ' -> ', (d shortName), ' [', ((d countDaughtersRecursively + 1) printString), ']'; cr].

	self daughters do: [:d | d printSecondaryDaughtersRecursively].
! !

!Meme methodsFor: 'backup-debug' stamp: ''!
pseudoJumps
	^ moms
		select: [:t1 | t1 contentString = 'jumpPoints' and: [Soups first jumpPointsHolder ~~ t1]]! !

!Meme methodsFor: 'backup-debug' stamp: 'miso 1/12/2005 01:07'!
secondaryMoms
"to avoid the survival of jumpoints across backup/restore, pretend they don't exist in my secondary moms. See also the comment in #primaryMoms. (Note that we cannot simply do that in #moms because that would create a copy of the mom-array and new daughters would be inserted in that copy rather than in the orginal array). Finally, the correct fix for all of this is to hold jumpPoints in an orderedCollection rather than a meme..."

	^ super secondaryMoms select: [:t1 | t1 contentString ~= 'jumpPoints']! !

!Meme methodsFor: 'name & address' stamp: 'mich 10/5/1999 18:26'!
shortName

	^ self name truncateTo: 32! !

!Meme methodsFor: 'name & address' stamp: 'miso 10/28/1999 13:06'!
submemeFromRelativePath: aPathArray	
"by the time we get it, aPathArray is relative to us"

	| next nextPathArray |
	aPathArray isEmpty ifTrue: [^self].
	next _ aPathArray first.
	nextPathArray _ aPathArray copyWithout: next.
	next _ self daughterCalled: next asString.
	next ifNil: [^nil].
	^ next submemeFromRelativePath: nextPathArray.! !

Meme subclass: #MemeSoup
	instanceVariableNames: 'jumpPointsHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Golgi-Basics'!

!MemeSoup commentStamp: 'miso 1/12/2005 19:25' prior: 0!
I am the root node of a (directed) meme-graph, ie the container of all the memes in that graph. The graph is sometimes nicknamed a soup - or data soup - and I thus represent a whole MemeSoup.

I am different from the other memes in that I hold a list of 'jumpoints', important entry points that one wants to get to quickly (they show in menus) and in that I have no mom myself.!

!MemeSoup class methodsFor: 'the fridge containing all soups' stamp: 'miso 10/12/1999 00:30'!
checkFridgeIsNotEmpty

	self fridge isEmpty ifTrue: [self fridge add: MemeSoup new]
! !

!MemeSoup class methodsFor: 'the fridge containing all soups' stamp: 'miso 1/9/2005 00:08'!
fridge

^ Golgi soups
! !

!MemeSoup class methodsFor: 'reaching' stamp: 'miso 6/7/2001 12:17'!
memeFromAddress: stringAddress

	| s url |
	url _ Meme stringurl2array: stringAddress.
	stringAddress first == $!! 
		ifTrue: ["url is relative to default soup"
			s _ self mainSoup]
		ifFalse: [
			s _ self "soupNamed: url first". self halt.
			url _ url copyWithout: url first. "make it relative to the soup"].

	^ s submemeFromRelativePath: url.! !

!MemeSoup class methodsFor: 'the fridge containing all soups' stamp: 'miso 11/2/1999 18:41'!
primordialSoup

	self checkFridgeIsNotEmpty.
	^ self fridge first! !

!MemeSoup class methodsFor: 'backup' stamp: 'miso 1/11/2005 10:26'!
privFromTokenisedString: arrayOfTokens
"format of tokenisedString = #(stats parensString jumpPoints)"

	| theSoup |
	super privFromTokenisedString: arrayOfTokens. "transforms the second element into a tree"
	theSoup := arrayOfTokens second.
	arrayOfTokens size > 2 ifTrue: [
	 	arrayOfTokens third do: [: p | theSoup addJumpPoint: (theSoup submemeFromPath: p)]].
	^ theSoup

! !

!MemeSoup methodsFor: 'jumpPoints' stamp: 'miso 10/29/1999 13:36'!
addJumpPoint: aMeme
	"no check for overwrite or for Meme-ness of value "

	aMeme putSelfInto: jumpPointsHolder after: #last.
	^ true! !

!MemeSoup methodsFor: 'jumpPoints' stamp: 'miso 10/29/1999 13:36'!
deleteJumpPoint: aMeme

	^ aMeme moveSelfOutOf: jumpPointsHolder! !

!MemeSoup methodsFor: 'name & address' stamp: 'miso 10/12/1999 10:39'!
externalName

	^ self name , FileDirectory dot , Utilities dateTimeSuffix, '.soup'
! !

!MemeSoup methodsFor: 'i/o' stamp: 'miso 11/1/1999 21:42'!
fileOutOn: aStream
"be careful that we remain a single chunk"

	super fileOutOn: aStream.
	aStream cr.
	"now save our jumpPoints, creating alist of stringPaths"
	aStream nextPutAll: '('.
	self jumpPoints do: [: m | aStream nextPutAll: m internalPath printString; space].
	aStream nextPut: $).
	aStream nextChunkPut: ' '. "make the whole Soup a chunk for import"
	"aStream nextChunkPut: (self jumpPoints collect: [: m | m internalPath ]) asString."
! !

!MemeSoup methodsFor: 'i/o' stamp: 'miso 11/1/1999 21:41'!
fileOutOn: aStream as: formatSymbolOrNil
"be careful that we remain a single chunk"

	formatSymbolOrNil ifNil: [^ self fileOutOn: aStream].

	^ self error: 'cannot fileOut in this format because it is not implemented yet'! !

!MemeSoup methodsFor: 'i/o' stamp: 'miso 10/29/1999 13:36'!
initialize

	super initialize.
	self semanticType: #soup.
	content _ 'Gazpacho' asText.
	jumpPointsHolder _ Meme withMeat: 'jumpPoints'.! !

!MemeSoup methodsFor: 'jumpPoints' stamp: 'miso 10/29/1999 13:36'!
jumpPoints

	^ jumpPointsHolder daughters! !

!MemeSoup methodsFor: 'jumpPoints' stamp: 'miso 10/29/1999 13:39'!
jumpPointsHolder

	^ jumpPointsHolder! !

!MemeSoup methodsFor: 'jumpPoints' stamp: 'miso 10/29/1999 13:25'!
jumppointNamed: nameAsString
	self jumpPoints do: [
		:a  |
		(a name sameAs: nameAsString) ifTrue: [
			^ a]].
	^ nil! !

!MemeSoup methodsFor: 'menu & menuItems' stamp: 'miso 10/30/1999 16:52'!
menuForDelete
	"propose a menu of available outlines to delete."
	| choice h  |
	self outlines size = 0 ifTrue: [
		^ false].
	choice _ MenuMorph new.
	choice addTitle: 'delete'.
	self outlines do: [
		:d  |
		choice add: d content target: d selector: #wannaSuicide].
	h _ World hands first.
	choice popUpAt: h position forHand: h! !

!MemeSoup methodsFor: 'menu & menuItems' stamp: 'mich 1/12/2005 23:34'!
newOutline
"an 'outline' is defined as a top-level meme, so just create a new daughter!!"

^ Meme string: 'outline #', (self daughters size + 1) asString
		mom: self 
		after: #last! !

!MemeSoup methodsFor: 'name & address' stamp: 'miso 4/18/1999 10:52'!
path
	^ {}! !

!MemeSoup methodsFor: 'name & address' stamp: 'miso 6/7/2001 12:15'!
submemeFromPath: aPathString
"path is absolute"

	| aPathArray |
	aPathArray _ self class stringurl2array: aPathString.
	aPathArray isEmpty ifTrue: [^ nil ].
	(self name sameAs: aPathArray first) ifFalse: [^ nil].
	^ self submemeFromRelativePath: ( aPathArray copyWithout: aPathArray first)! !

!MemeSoup methodsFor: 'jumpPoints' stamp: 'miso 10/29/1999 13:33'!
theIncomingArea

	^ (self jumppointNamed: #incoming) ifNil: [self addJumpPoint: (Meme withMeat: 'incoming')]! !

!MemeSoup methodsFor: 'menu & menuItems' stamp: 'mich 1/13/2005 00:40'!
worldMenu
"Build a menu with all jumpPoints"
	| menu nextSoup |
	menu _ MenuMorph new defaultTarget: Golgi.
	menu addTitle: 'open'.
	menu addStayUpItem.
	menu addLine.
	self jumpPoints do: [:a|
		menu add: a contentString selector: #view: argument: a].
	menu addLine.
	self daughters do: [:a|
		menu add: a contentString selector: #view: argument: a].
	menu addLine.
	"menu add: 'root' selector: #view: argument: self."
	menu add: 'new outline' selector: #viewNewOutlineIn: argument: self.
	(MemeSoup fridge size > 1) & (MemeSoup fridge last ~= self)
		ifTrue: [
			nextSoup _ MemeSoup fridge after: self. 
			menu add: 'next soup... (', nextSoup name, ')' target: nextSoup selector: #worldMenu].

	^ menu
! !

!Node class methodsFor: 'creation' stamp: 'miso 11/1/1999 21:59'!
fromBackupFile: aFileRef

	^ self fromStream: (FileStream readOnlyFileNamed: aFileRef).! !

!Node class methodsFor: 'creation' stamp: 'miso 1/11/2005 12:32'!
fromRecursiveArray: theArray
"import a monolithic (recursive) array in standard format: 'content (secondaryMoms) type (daughters). Do a first pass ignoring secondary moms, and a second pass resolving them once all is in."

	| tree |
	tree _ self fromRecursiveArray: theArray curMom: nil.
	tree walkSelfAndPrimaryDaughtersWith: [: n |
		n unpackResolveUrlsForMomOrDaughter: true.
		n unpackResolveUrlsForMomOrDaughter: false].
	"tree removeShadows."	"<-- created a bug when primary mom was a shadow"

	^ tree "ancestor". 	"#ancestor in case my top node has shadow moms"
! !

!Node class methodsFor: 'creation' stamp: 'miso 1/11/2005 12:27'!
fromRecursiveArray: theArray curMom: mom
"fill in those values that come from structure of array: main mom and non-url daughters, and keep urls raw, they will be dealt with in a second pass"

	| curNode e |
	theArray size ~~ 4 ifTrue: [
		^ self error: 'cannot create memes from this array, it has a wrong format'].

	curNode _ Meme ofType: theArray third.
	curNode contentFromFlattenedText: theArray first.
	mom ifNotNil: [mom registerIncomingDaughter: curNode after: #last].
	"do not use putSelfInto:after:, we might have urls as moms and that would choke on the isInside: check"

	theArray second do: [: k | curNode moms add: k]. 			"add 2nd-ary moms as urls"
	1 to: theArray fourth size do: [: k | 							"daughters"
		e _  theArray fourth at: k.
		(e isKindOf: String) 
			ifTrue: [curNode daughters add: e]
			ifFalse: [self fromRecursiveArray: e curMom: curNode]].

	^ curNode! !

!Node class methodsFor: 'deprecated' stamp: 'miso 1/11/2005 12:32'!
fromRecursiveArray: theArray keepingShadow: keepShadows
"deprecated: since there was only one caller, and it always called us with false for keepShadows, simplify life by implementing this method there (#fromRecursiveArray:), eliminating the <keepShadow> boolean"
"import a monolithic (recursive) array in standard format: 'content (secondaryMoms) type (daughters). Do a first pass ignoring secondary moms, and a second pass resolving them once all is in."

	| tree |
	tree _ self fromRecursiveArray: theArray curMom: nil.
	tree walkSelfAndPrimaryDaughtersWith: [: n |
		n unpackResolveUrlsForMomOrDaughter: true.
		n unpackResolveUrlsForMomOrDaughter: false].
	keepShadows ifFalse: [tree removeShadows].

	^ tree "ancestor. in case my top node has shadow moms"
! !

!Node class methodsFor: 'deprecated' stamp: 'miso 1/11/2005 12:32'!
fromRecursiveArray: theArray removingShadow: keepShadows
"deprecated: call it keepingShadow, not removingShadow!!" 
"import a monolithic (recursive) array in standard format: 'content (secondaryMoms) type (daughters). Do a first pass ignoring secondary moms, and a second pass resolving them once all is in."

	| tree |
	tree _ self fromRecursiveArray: theArray curMom: nil.
	tree walkSelfAndPrimaryDaughtersWith: [: n |
		n unpackResolveUrlsForMomOrDaughter: true.
		n unpackResolveUrlsForMomOrDaughter: false].
	keepShadows ifFalse: [tree removeShadows].

	^ tree "ancestor. in case my top node has shadow moms"
! !

!Node class methodsFor: 'creation' stamp: 'miso 11/1/1999 21:59'!
fromStream: aFileStream

	^ self fromString: aFileStream contentsOfEntireFile.! !

!Node class methodsFor: 'creation' stamp: 'miso 11/1/1999 22:32'!
fromString: aParensString
"Node fromString: Soups first asScannableString"

	^ self privFromTokenisedString: (Scanner new scanTokens: aParensString)
! !

!Node class methodsFor: 'creation' stamp: 'miso 10/30/1999 16:43'!
new
	^ super new initialize.! !

!Node class methodsFor: 'creation' stamp: 'miso 11/2/1999 00:35'!
privCompareNewStats: newStats toOld: oldStats 
"check that the stats before teh export are the same as those after the export. Interrupt with an error if not, else do nothing.
Format: #(versionFormat=1 primaryDaughterNum daughterNum charNum primaryDaughterNumWalking daughterNumWalking"

	newStats first = oldStats first ifFalse: ["not the same format of stats..." ^self].
	2 to: 6 do: [:i | (newStats at: i) = (oldStats  at: i) ifFalse: [^self error: 'the stats of the new file do not correspond to the old file...']]! !

!Node class methodsFor: 'creation' stamp: 'miso 11/1/1999 22:29'!
privFromTokenisedString: arrayOfTokens
"format of tokenisedString = #(stats parensString)"

	arrayOfTokens at: 2 put: (self fromRecursiveArray: arrayOfTokens second).
	self privCompareNewStats: arrayOfTokens second stats toOld: arrayOfTokens first.
	^ arrayOfTokens second! !

!Node class methodsFor: 'misc' stamp: 'miso 11/2/1999 18:42'!
testTheBackupProcess

	| m lsd |
	m _ MemeSoup primordialSoup.

	m showStats. "so that we know what we are backing up"
	lsd _ m backup.

	Transcript 	cr; cr; 
				show: ' I have now been backed up into: ', lsd printString; cr;
				show: '  --  restoring  --'; cr.
	lsd _ MemeSoup fromBackupFile: lsd.

	Transcript show: ' And now the same stats after a restore from the backup: -- '; cr; cr.
	lsd showStats.
	Golgi view: lsd.
	^ lsd
! !

!Node methodsFor: 'moms' stamp: 'ms 7/2/1999 22:16'!
addMom: aNewMom
	(self moms includes: aNewMom) ifTrue: [^ self error: 'cannot add', aNewMom printString, 'as mom to ', self printString, 'since it is already my mom'].
	self moms addLast: aNewMom! !

!Node methodsFor: 'moms' stamp: 'mich 10/4/1999 14:45'!
ancestor
	| curMom  |
	curMom _ self.
	[curMom primaryMom = nil] whileFalse: [
		curMom _ curMom primaryMom].
	^ curMom! !

!Node methodsFor: 'backup/unpack' stamp: 'miso 11/1/1999 21:50'!
asCompilableString
"a (string-)representation of a representation of an array. After being evaluated by the compiler (Compiler evaluate:), this produces a string which represents an array. This latter string can then be scanned to obtain the actual array. Useful for fileIns: those get evaluated, and we funnel that evaluation through a scanner in fromString:. (Used when creating a file that can be filedIn to reconstruct all data in an img, as opposed to backup files which are directly read with 'fromBackupFile:')"

	^ self asScannableString fullPrintString! !

!Node methodsFor: 'converting' stamp: 'miso 7/4/1999 19:02'!
asParensString
	^ String streamContents: [:s | self asParensStringOn: s]! !

!Node methodsFor: '  onto stream' stamp: 'miso 11/1/1999 20:47'!
asParensStringOn: aStream
"a representation of an Array. each item of the array is a representation of a meme"

	^ self asParensStringOn: aStream withPrimaryMomRef: false! !

!Node methodsFor: '  onto stream' stamp: 'miso 1/11/2005 12:07'!
asParensStringOn: aStream withPrimaryMomRef: putPrimaryMom
"do not export daughters which are aliases/links, or else we will export them several times, creating confusion on import. These daughters have links to us in their mom fields, this will be used on import to restore the link"

aStream nextPutAll: '('.

	self asStringOn: aStream withPrimaryMomRef: putPrimaryMom withDaughtersRef: false.

	aStream nextPutAll: '('. 
		self daughters do: [: d | 
			(self isPrimaryDaughter: d )
				ifTrue: [d asParensStringOn: aStream]
				ifFalse: [d internalPath storeOn: aStream. aStream space]].
	aStream nextPutAll: ')'.

aStream nextPutAll: ')'.
! !

!Node methodsFor: 'backup/unpack' stamp: 'miso 11/1/1999 21:50'!
asScannableString
"a (string-)representation of an array. After being scanned (Scanner new scanTokens:), this produces the actual array."

	^ (String streamContents: [: strm | self fileOutOn: strm]) ! !

!Node methodsFor: 'converting' stamp: 'miso 11/1/1999 11:09'!
asShortString
	| maxLength s |
	maxLength _ 20.
	s _ self contentString.
	s size > maxLength ifTrue: [s _ (s copyFrom: 1 to: maxLength), ' ...'].
	^s ! !

!Node methodsFor: 'converting' stamp: 'miso 7/5/1999 13:04'!
asString
	^ String streamContents: [:s | self asStringOn: s  
									withPrimaryMomRef: true 
									withDaughtersRef: true]! !

!Node methodsFor: '  onto stream' stamp: 'miso 1/11/2005 12:04'!
asStringOn: aStream withPrimaryMomRef: putPrimMom withDaughtersRef: putDaughters
"a simple non-recursive flattening of myself."

	| theMoms |
	self contentAsFlatText storeOn: aStream.

	putPrimMom ifTrue: [theMoms _ self moms] ifFalse: [theMoms _ self secondaryMoms].
	aStream nextPutAll: '('.
	theMoms do: [: m | m internalPathAsArray isEmpty ifFalse: [ m internalPath storeOn: aStream. aStream space]].
	aStream nextPutAll: ')'; space.

	aStream nextPutAll: self semanticType printString; space.
	putDaughters ifTrue: [
		self daughters do: [: d | d internalPath storeOn: aStream. aStream space]].! !

!Node methodsFor: 'converting' stamp: 'miso 7/5/1999 13:05'!
asStringWithoutDaughters

	^ String streamContents: [:s | self asStringOn: s
									withPrimaryMomRef: true 
									withDaughtersRef: false]! !

!Node methodsFor: 'backup/unpack' stamp: 'miso 10/31/1999 01:19'!
backup

	| file fileName |
	fileName _ self externalName.
	Cursor write showWhile: [
		file _ FileStream newFileNamed: fileName.
		self fileOutOn: file.
		file close].
	^ fileName! !

!Node methodsFor: 'name & address' stamp: 'miso 7/5/1999 15:07'!
containsRelativeInternalUrl: rUrl

	(self localPath sameAs: rUrl first) ifFalse: [^false].
	rUrl size = 1 ifTrue: [^self].
	self daughters do: [: d | d containsRelativeInternalUrl: (rUrl copyWithout: rUrl first)].
	^ false! !

!Node methodsFor: 'content' stamp: 'miso 11/1/1999 10:55'!
content
	^ content! !

!Node methodsFor: 'content' stamp: 'miso 7/4/1999 18:09'!
content: c
	content _ c.		"^ meat acceptContents: t		contents: is more careful"
	self dataReaders do: [:r| r dataChanged].! !

!Node methodsFor: 'content' stamp: 'miso 11/2/1999 14:20'!
contentAsFlatText
"For fileOut. used to do with an adaptation of nextChunkPutWithStyle: but embedding !!s was too fragile"

	| t |
	t _ self content.
	t class == String ifTrue: [^t].
	t runs coalesce.
	"check if there is any style worth saving"
	(t runs values = (OrderedCollection new)) ifTrue: [^t asString].
	(t runs values = (OrderedCollection with: (Array with: TextFontChange font1))) ifTrue: [^t asString]. 
	^ String streamContents: [:stream | 
		stream nextPutAll: ']sqStyle['.
		t runs writeScanOn: stream.
		stream nextPutAll: ']'.
		stream nextPutAll: t asString].
! !

!Node methodsFor: 'content' stamp: 'miso 10/31/1999 17:52'!
contentFromFlattenedText: aString
"For fileIn. used to do with an adaptation of nextChunkPutWithStyle: but embedding !!s was too fragile"

	self content: (self unflattenContentString: aString)! !

!Node methodsFor: 'content' stamp: 'miso 11/1/1999 10:55'!
contentString
	"guarantee that meat will be a string"
	^ self content asString! !

!Node methodsFor: 'daughters' stamp: 'mich 10/5/1999 14:06'!
countDaughtersRecursively

	| n |
	n _ self daughters size.
	self daughters do: [:d|
		n _ n + (d countDaughtersRecursively)].
	^ n
! !

!Node methodsFor: 'daughters' stamp: 'miso 10/6/1999 00:00'!
countPrimaryDaughtersRecursively

	| n |
	n _ self primaryDaughters size.
	self primaryDaughters do: [:d|
		n _ n + (d countPrimaryDaughtersRecursively)].
	^ n! !

!Node methodsFor: '  add/delete' stamp: 'ms 7/2/1999 23:23'!
createDaughterAfter: existingNode
"return nil if do not accept creation. Remember that existingNode can take some Symbol values. will need to be augmented to infer the type of node to create"

	| newMeme typeOfNode |
	"first decide what kind of meme should be created"
	existingNode = #root
		ifTrue: [typeOfNode _ self class]
		ifFalse: [typeOfNode _ existingNode class].
	newMeme _ typeOfNode new.

	newMeme putSelfInto:  self after: existingNode.
	^ newMeme.! !

!Node methodsFor: 'accessors' stamp: 'ms 7/2/1999 16:17'!
dataReaders
	^ self dependents! !

!Node methodsFor: 'daughters' stamp: 'miso 10/30/1999 20:18'!
daughterAfter: aDaughter

	^ self daughterAtIndex: (self daughterIndex: aDaughter) + 1! !

!Node methodsFor: 'daughters' stamp: 'miso 10/30/1999 20:20'!
daughterAtIndex: anInteger

	anInteger < 1 ifTrue: [ ^ #root ].
	anInteger > daughters size ifTrue: [ ^ #last ].
	^ daughters at: anInteger.! !

!Node methodsFor: 'daughters' stamp: 'miso 10/30/1999 20:17'!
daughterBefore: aDaughter

	^ self daughterAtIndex: (self daughterIndex: aDaughter) - 1
! !

!Node methodsFor: 'daughters' stamp: 'miso 10/30/1999 19:46'!
daughterIndex: d
"returns 0 if d is not my daughter"
	^ daughters indexOf: d! !

!Node methodsFor: 'accessors' stamp: 'ms 3/6/1999 04:19'!
daughters
	^ daughters! !

!Node methodsFor: 'accessors' stamp: 'ms 3/6/1999 04:20'!
daughters: d
	daughters_d! !

!Node methodsFor: '  add/delete' stamp: 'mich 1/13/2005 00:52'!
defaultDaughterClass

^ self class! !

!Node methodsFor: '  add/delete' stamp: 'miso 10/30/1999 19:46'!
deregisterDaughter: aDaughter

	| daughterIndex |
	(daughterIndex _ self daughterIndex: aDaughter) = 0 ifTrue: [^false].

	self daughters removeAt: daughterIndex.
	aDaughter killMom: self.
	^ true.! !

!Node methodsFor: 'name & address' stamp: 'miso 10/12/1999 10:35'!
externalName

	^ self name , FileDirectory dot , Utilities dateTimeSuffix, '.tree'
! !

!Node methodsFor: 'backup/unpack' stamp: 'miso 11/1/1999 21:45'!
fileOutOn: aStream

	aStream nextPutAll: self stats asString.
	self asParensStringOn: aStream! !

!Node methodsFor: 'backup/unpack' stamp: 'miso 11/1/1999 21:28'!
fileOutString
"a (string-)representation of a representation of an array. After being evaluated by teh compiler (Compiler evaluate: ), this produces an string which represents an array. This latter string can then be scanned to obtain the actual array. Useful for fileIns: those get evaluated, and we funnel that evaluation through a scanner in fromString:. (This is used when created a file that can be filedIn to reconstruct all data in an img, as opposed to backup files which are directly read with 'fromBackupFile:')"

	^ (String streamContents: [: strm | self fileOutOn: strm]) fullPrintString! !

!Node methodsFor: 'moms' stamp: 'ms 7/2/1999 19:04'!
indexInMom: aMom
	^ aMom daughterIndex: self ! !

!Node methodsFor: 'private' stamp: 'mich 10/4/1999 21:44'!
initDaughtersAndMoms

	daughters _ OrderedCollection new.
	moms _ OrderedCollection new.! !

!Node methodsFor: 'private' stamp: 'mich 10/4/1999 21:44'!
initialize

	self initDaughtersAndMoms! !

!Node methodsFor: 'moms' stamp: 'miso 10/8/1999 21:24'!
isInside: aPutativeAncestor
"this is used during mv operations to check that we are not mving into something inside ourselves... (or into ourselves). Walk all moms up from aPutativeAncestor, if I can reach nil without meeting it, I am not in there"

	aPutativeAncestor = self ifTrue: [^true].
	self moms do: [: m | (m isInside: aPutativeAncestor) ifTrue: [^true]].
	^ false! !

!Node methodsFor: 'daughters' stamp: 'mich 10/4/1999 08:56'!
isPrimaryDaughter: d

	^ d primaryMom = self! !

!Node methodsFor: 'backup/unpack' stamp: 'miso 10/12/1999 00:23'!
isShadow

	^ self semanticType = #shadow! !

!Node methodsFor: 'moms' stamp: 'miso 11/2/1999 17:26'!
killMom: aMom

	self moms remove: aMom ifAbsent: [self error: 'cannot kill mom, ',  aMom printString, ', coz it is not one of my moms....'].! !

!Node methodsFor: 'accessors' stamp: 'ms 7/2/1999 22:20'!
moms
	^moms! !

!Node methodsFor: 'moving' stamp: 'miso 11/2/1999 17:29'!
moveSelfOutOf: anOldMom
"if anOldMom is my only mom, this is = suicide. In that case I also tell all my daughters to stop hanging on to me (in case of them survives because it has several moms)"

	(anOldMom deregisterDaughter: self) ifFalse: [^ false]. "I wasn't in oldMomNode"
	self moms isEmpty ifTrue: [self daughters do: [: d | d killMom: self]].

	self dataReaders do: [:r| r dataGotOutOf: anOldMom].
	^ true! !

!Node methodsFor: 'moving' stamp: 'miso 10/30/1999 20:29'!
moveSelfOutOf: oldMomNode into: newMomNode after: indexOrSymbolOrMeme
"First transform daughterDescriptor into an index-independent descriptr, so that we can delete/add things safely (changing indexes). Notify both the mover and the new host of the movement. Return nil if cannot do it."

	| memeOrSymbol oldSisterBefore success |
	(newMomNode isInside: self) ifTrue: [^ false].

	"make sure indexOrSymbolOrMeme is not an index, so we can add/delete without obsoleting it"
	(indexOrSymbolOrMeme isKindOf: Number) 
		ifTrue: [memeOrSymbol _ newMomNode daughterAt: indexOrSymbolOrMeme]
		ifFalse: [memeOrSymbol _ indexOrSymbolOrMeme].

	"keep track of index in old mom, incase smthng goes wrong and we need to put it back there"
	oldSisterBefore _ oldMomNode daughterBefore: self.

	success _ oldMomNode deregisterDaughter: self.
		success ifFalse: [^ false]. "I wasn't in oldMomNode"
	success _ newMomNode registerIncomingDaughter: self after: memeOrSymbol.
		success = 0 ifTrue: [ "wasn't added. repair and tell user smthg is wrong"
			oldMomNode registerIncomingDaughter: self after: oldSisterBefore.
			self notify: 'I couldn''t be added to ', newMomNode printString, '. Nothing is lost in your data, but please contact the author abut this bug'.
			^ false].

	self dataReaders do: [:r| r dataMovedOutOf: oldMomNode into: newMomNode after: memeOrSymbol].	
	newMomNode dataReaders do: [:r| r dataHasNewDaughter: self]. "in case someone views newMom with me"

	^ true

"need to deregister *before* registering in the new place (against robustness) to avoid big complications when mving within same mom, especially when mving up (this would change our old index)."
! !

!Node methodsFor: 'accessors' stamp: 'ms 7/2/1999 16:18'!
newDataReader: dr
	^ self addDependent: dr! !

!Node methodsFor: 'daughters' stamp: 'mich 10/4/1999 08:53'!
primaryDaughters
"answer with those daughters for whom I am a primaryMom"

	^ self daughters select: [:d| d primaryMom = self] ! !

!Node methodsFor: 'moms' stamp: 'mich 10/4/1999 17:05'!
primaryMom
"there is no real primary mom (all are equivalent), but some rare cases need to get just one mom (eg. url formation, ancestor search): give them the first"

	^ self moms isEmpty ifTrue: [^nil]
						ifFalse: [^self moms first]! !

!Node methodsFor: '  onto stream' stamp: 'mich 10/5/1999 13:01'!
printOn: aStream
	"used in inspectors, debuggers, etc."

	^ aStream nextPutAll: '#' , self asShortString, ' (', self identityHash printString, ')'! !

!Node methodsFor: 'moving' stamp: 'miso 10/8/1999 21:29'!
putSelfInto: newMom after: existingDaughter
"like moveSelfInto excep that it does not undo a previous state, it just adds itself to a new mom. Used for newly created memes."

	(newMom isInside: self) ifTrue: [^ false].
	newMom registerIncomingDaughter: self after: existingDaughter.
	newMom dataReaders do: [:d | d dataHasNewDaughter: self].
	^ true "might want to check index returned by register"
! !

!Node methodsFor: '  add/delete' stamp: 'miso 10/30/1999 19:55'!
registerIncomingDaughter: aNewDaughter after: daughterOrSymbol
"The most primitive verb to add meme someplace, set both the mom and daughter sides. Building block for the self verbs. Returns the index of the newly added meme, and < 1 if something went wrong."

	| index |
	index _ nil.
	(self daughters includes: aNewDaughter) ifTrue: [^0].

	daughterOrSymbol ifNil: [
		self daughters addFirst: aNewDaughter. 
		index _ 1].
	(daughterOrSymbol = #root) ifTrue: [
		self daughters addFirst: aNewDaughter..
		index _ 1].
	(daughterOrSymbol = #last) ifTrue: 
		[ self daughters addLast: aNewDaughter. index _ self daughters size ].
	(index == nil) ifTrue: [
		"paranoia" (self daughters includes: daughterOrSymbol) ifFalse: [^0]. 
		self daughters add: aNewDaughter after: daughterOrSymbol.
		index _ (self daughters indexOf: daughterOrSymbol)+1. "there are 2 occurences of newM now"].

	aNewDaughter addMom: self.

	^ index! !

!Node methodsFor: 'backup/unpack' stamp: 'miso 10/12/1999 00:27'!
removeShadows
"remove all shadow memes from the tree below me. this can be pretty drastic: if a shadow links to subtrees, one of the two will be lost after this!!"

	self isShadow ifTrue: [
		self moms do: [: m | m  deregisterDaughter: self].
		self daughters do: [: d | self deregisterDaughter: d]].
	self daughters do: [: d | d removeShadows]! !

!Node methodsFor: 'name & address' stamp: 'miso 7/5/1999 15:07'!
resolveRelativeInternalUrl: rUrl

	(self localPath sameAs: rUrl first) ifFalse: [^nil].
	rUrl size = 1 ifTrue: [^self].
	self daughters do: [: d | d resolveRelativeInternalUrl: (rUrl copyWithout: rUrl first)].
	^ nil! !

!Node methodsFor: 'moms' stamp: 'miso 7/4/1999 18:51'!
secondaryMoms
	^ self moms copyWithout: self primaryMom! !

!Node methodsFor: 'accessors' stamp: 'ms 3/6/1999 04:27'!
semanticType

	^ semanticType! !

!Node methodsFor: 'accessors' stamp: 'ms 3/6/1999 04:27'!
semanticType: s

	semanticType _ s! !

!Node methodsFor: 'statistics' stamp: 'miso 11/1/1999 21:20'!
showStats 
"sizeStats"

	| stats |
	stats _ self stats.
	Transcript show: 'The node called ', self name, ' has: '; cr.
	Transcript show: '   ', (stats second) printString, ' primary daughters'; cr.
	Transcript show: '   ', (stats third) printString, ' simple daughters'; cr.
	Transcript show: '   and contains ', (stats fourth) printString, ' characters'; cr; cr.

	Transcript show: '   If counted with the walking methode, I have: '; cr.
	Transcript show: '   ', (stats fifth) printString, ' primary daughters +self'; cr.
	Transcript show: '   ', (stats sixth) printString, ' simple daughters +self'; cr.

! !

!Node methodsFor: 'accessors' stamp: 'miso 11/1/1999 23:11'!
sisterAfterMeInMom: aMomOfMine
	^ aMomOfMine daughterAfter: self! !

!Node methodsFor: 'accessors' stamp: 'miso 11/1/1999 23:12'!
sisterBeforeMeInMom: aMomOfMine
	^ aMomOfMine daughterBefore: self! !

!Node methodsFor: 'statistics' stamp: 'miso 1/11/2005 11:32'!
stats 
"sizeStats"

	| resu |
	resu _ Array new: 6.
	resu at: 1 put: 1. "versionNumber of statsSerialisation convention"
	resu at: 2 put: self countPrimaryDaughtersRecursively.
	resu at: 3 put: self countDaughtersRecursively.
	resu at: 4 put: self countCharsInSelfAndDaughters.
	resu at: 5 put: self countPrimaryDaughtersByWalking.
	resu at: 6 put: self countDaughtersByWalking.
	^ resu

! !

!Node methodsFor: '  onto stream' stamp: 'miso 7/5/1999 01:25'!
storeOn: aStream
	"for file out. note that this files out not only this meme but all its daughters"
	self asParensStringOn: aStream
! !

!Node methodsFor: 'content' stamp: 'miso 1/3/2005 12:03'!
textForSearchOn: searchString

| maxLength originalText offset text |
maxLength := 100.
originalText := self content asText.
offset := originalText findString: searchString startingAt: 0.

originalText size > maxLength 
	ifTrue: [
		offset > (maxLength - searchString size)
			ifTrue: [
				text := originalText copyFrom: 1 to: maxLength // 2 - 5.
				text := text, '  [...]  ' asText allBold.
				text := text, (originalText copyFrom: offset - (maxLength // 2) to: offset + (maxLength // 2))]
			ifFalse: [
				text := originalText copyFrom: 1 to: maxLength]]
	ifFalse: [text := originalText copy].

"text addAttribute: TextEmphasis italic from: offset to: offset + searchString size - 1."


^ text! !

!Node methodsFor: 'content' stamp: 'miso 10/31/1999 17:53'!
unflattenContentString: aString
"private. unflatten the string found in fileOuts"

	| peek stream runsRaw runStream runs |
	stream _ ReadStream on: aString.

	"Test for ]sqStyle[ tag"
	peek _ stream skipSeparatorsAndPeekNext.
	peek = $] ifFalse: [^ aString asText].  "no tag"
	(stream upTo: $[) = ']sqStyle' ifFalse: [^ aString asText].  "different tag"

	"Read and decode the style chunk"
	runsRaw _ stream upTo: $].	"style encoding"
	runStream _ ReadStream on: runsRaw from: 1 to: runsRaw size.
	runs _ RunArray scanFrom: runStream.

	^ Text basicNew setString: stream upToEnd setRunsChecking: runs.


"	t _ self content.
	t class == String ifTrue: [^t].
	t runs coalesce.
	(t runs values = (OrderedCollection with: (Array with: TextFontChange font1))) ifTrue: [^t asString]. // no style, save as string
	^ String streamContents: [:stream | 
		stream nextPutAll: ']sqStyle['.
		t runs writeScanOn: stream.
		stream nextPutAll: ']'.
		stream nextPutAll: t asString].
"
! !

!Node methodsFor: 'backup/unpack' stamp: 'miso 10/31/1999 17:53'!
unpackCreateShadowWithMeat: meatString mom: m

	| curNode |
	curNode _ (Meme ofType: 'shadow') contentFromFlattenedText: meatString.
	m ifNotNil: [curNode putSelfInto: m after: #last].
	^ curNode
! !

!Node methodsFor: 'backup/unpack' stamp: 'mich 10/5/1999 18:53'!
unpackCreateShadowsForUrl: url relativeTo: topMeme
"construct a series of shadow memes mimicking the url, one for each path segment"

	| prevMom |
	prevMom _ topMeme.
	url do: [: s | prevMom _ self unpackCreateShadowWithMeat: s mom: prevMom].
	^ prevMom! !

!Node methodsFor: 'backup/unpack' stamp: 'miso 10/12/1999 00:24'!
unpackResolveUrlsForMomOrDaughter: doMom

	| myCollec herCollec e theNode |
	doMom ifTrue: [myCollec _ self moms] ifFalse: [myCollec _ self daughters].
	1 to: myCollec size do: [: k | 
		e _ myCollec at: k.
		(e isKindOf: String) ifTrue: [
"self halt."
			k=1 ifTrue: [myCollec at: k put: nil]. 		"preventive, in case this is the first mom and we need to call things like 'ancestor' or 'internalUrl' "
			theNode _ self unpackSearchForShadowAndCreateIfNotFound: e.
			myCollec at: k put: theNode. 				"link me to it"
			doMom ifTrue: [herCollec _ theNode daughters] "link it to me"
				ifFalse: [herCollec _ theNode moms].
			theNode isShadow 
				ifTrue: [herCollec addLast: self]			"order irrelevant in shadows"
				ifFalse: [herCollec at: (herCollec indexOf: self internalPath) put: self]]].
! !

!Node methodsFor: 'backup/unpack' stamp: 'miso 6/7/2001 12:15'!
unpackSearchForShadowAndCreateIfNotFound: url
"find the object, or construct it with its moms"

	| urlParts |
	urlParts _ self unpackSearchForUrl: (self class stringurl2array: url). "this searches for a preexisting shadow url"

	urlParts second ifNil: [^urlParts first]. 	"found a shadow"
	^ self unpackCreateShadowsForUrl: urlParts second relativeTo: urlParts first.
! !

!Node methodsFor: 'backup/unpack' stamp: 'miso 10/6/1999 15:42'!
unpackSearchForUrl: url
"if not found but found a partial path, return an array with the lowermost element found and the url relative to that. Ie, searching for 'a/b/c/d', if a and b exist, return [b, 'c/d'}. So this is really a 'makeAsRelativeToMyTreeAsYouCan' method'. Url is already an array of localParts "

	| ancestor foundObject theObject urlParts |
	ancestor _ self ancestor.

	(ancestor name sameAs: url first) ifFalse: [^Array with: nil with: url].

	urlParts _ url.
	foundObject _ ancestor.
	"urlParts _ urlParts copyWithout: urlParts first."

	[foundObject = nil] whileFalse: [
		urlParts _ urlParts copyWithout: urlParts first. 
		urlParts isEmpty ifTrue: [^ Array with: foundObject with: nil].
		theObject _ foundObject.
		foundObject _ theObject submemeFromRelativePath: (Array with: urlParts first).
		].

	^ Array with: theObject with:  urlParts.
! !

!Node methodsFor: 'daughters' stamp: 'miso 1/11/2005 11:40'!
walkDaughtersWith: aOneArgumentBlock
"the block is evaluated with each daughters in turn as the argument"

	self daughters do: [:s | 
		aOneArgumentBlock value: s.
		s walkDaughtersWith: aOneArgumentBlock]! !

!Node methodsFor: 'daughters' stamp: 'miso 1/11/2005 11:41'!
walkPrimaryDaughtersWith: aOneArgumentBlock
"the block is evaluated with each primaryDaughter in turn as the argument"

	self primaryDaughters do: [:s | 
		aOneArgumentBlock value: s.
		s walkPrimaryDaughtersWith: aOneArgumentBlock]! !

!Node methodsFor: 'daughters' stamp: 'miso 1/11/2005 11:39'!
walkSelfAndDaughtersWith: aOneArgumentBlock
"the block is evaluated with each daughters in turn as the argument"

	aOneArgumentBlock value: self.
	self daughters do: [:s | s walkSelfAndDaughtersWith: aOneArgumentBlock]! !

!Node methodsFor: 'daughters' stamp: 'miso 1/11/2005 12:31'!
walkSelfAndPrimaryDaughtersWith: aOneArgumentBlock
"the block is evaluated with each primaryDaughter in turn as the argument"

	aOneArgumentBlock value: self.
	self primaryDaughters do: [:s | s walkSelfAndPrimaryDaughtersWith: aOneArgumentBlock]! !

!Preferences class methodsFor: '*golgi-override' stamp: 'miso 6/27/2000 15:07'!
personalizeUserMenu: aMenu
	"The user has clicked on the morphic desktop with the yellow mouse button (option+click on the Mac); a menu is being constructed to present to the user in response; its default target is the current world.  In this method, you are invited to add items to the menu as per personal preferences.
	The default implementation, for illustrative purposes, sets the menu title to 'personal', and adds items for go-to-previous-project, show/hide flaps, and load code updates"
	
	aMenu addTitle: 'personal'.  "Remove or modify this as per personal choice"

	aMenu add: #outlines target: Golgi action: #start.
	"aMenu add: 'previous project' action: #goBack."
	"aMenu addUpdating: #suppressFlapsString target: Utilities action: #toggleFlapSuppressionInProject."
	"aMenu add: 'load latest code updates' target: Utilities action: #updateFromServer."
	aMenu add: 'debug golgi' target: Golgi action: #debugMenu.! !

!ColorForm class methodsFor: '*golgi' stamp: 'miso 12/28/2004 11:19'!
vanishAllBut: aColor in: aFormOrCursor
	"Return a ColorForm copied from the given Form or Cursor with white mapped to transparent."

	| f map |
	aFormOrCursor depth <= 8 ifFalse: [
		^ self error: 'argument depth must be 8-bits per pixel or less'].
	(aFormOrCursor isKindOf: ColorForm) ifTrue: [
		f _ aFormOrCursor deepCopy.
		map _ aFormOrCursor colors.
	] ifFalse: [
		f _ ColorForm extent: aFormOrCursor extent depth: aFormOrCursor depth.
		f copyBits: aFormOrCursor boundingBox
			from: aFormOrCursor
			at: 0@0
			clippingBox: aFormOrCursor boundingBox
			rule: Form over
			fillColor: nil.
		map _ Color indexedColors copyFrom: 1 to: (1 bitShift: aFormOrCursor depth)].
	map _ map collect: [:c |
		c = aColor ifFalse: [Color transparent] ifTrue: [c]].
	f colors: map.
	^ f
! !
Golgi initialize!
GoHeadersPane initialize!
Meme initialize!
